/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CError.c       :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CError.h"

/*================================================================*/
/*
const char *CError_TypStackErrorTbl[CERR_MAX]={
    "None",
    "Unimplemented",
    "Invalid Family",
    "Invalid Genus",
    "Invalid Species",
    "Object ID Mismatch",
    "CRC Mismatch",
    "Unable to allocate memory",
    "Unable to find Link Table Entry",
    "Duplicate entry"
};
*/
void CError_Init(CError *new, uint8 Species)
{
    /* Set the ClassID */
    new->ClassID = (uint32)FmARCH << 16 | GnERROR << 8 | Species;

    /* Initialize the Datasize */
    new->DataSize = 0;

    /* Initialize the Description */
    memset(new->Description, 0, CERR_DESC_SIZE);

    /* Initialize the Data */
    memset(new->Data, 0, CERR_MAX_DATA_SIZE);
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CError_GetPayloadSize(CError *pObject)
{
    uint32 Size = SIZEOF(uint16)                        /*Size of the struct*/
                  + CERR_STATIC_DATA_SIZE;              /*Static data size*/

    Size += SIZEOF(uint32) + pObject->DataSize;

    Size += SIZEOF(uint16);            /* CRC bytes */

    return Size;

}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CError_SizeAll(CError *pObject)
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CError_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CError object into the buffer passed in.
*********************************************************************************/
boolean CError_Serialize(CError *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;
        uint16 i;

        /* Set the classID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the payload size */
        WriteInt16(&pTemp, (uint16)CError_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the static struct size */
        WriteInt16(&pTemp, (uint16)CERR_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the class ID of the errored message */
        WriteInt32(&pTemp, pObject->ErrorClassID, &ByteAligned);

        /* Set the ErrorCode */
        WriteInt16(&pTemp, pObject->ErrorCode, &ByteAligned);

        /* Set the Description and increment the pointer by CERR_DESC_SIZE */
        /* Write the packed data buffer based on the number of 8 bit octets in the
         * buffer divided by 2 since this is a 16 bit char platform 
         */             
        for(i=0; i< (CERR_DESC_SIZE/2); i++)
        {
            WriteInt16(&pTemp, pObject->Description[i], &ByteAligned);
        }       

        /* Set the DataSize */
        WriteInt32(&pTemp, pObject->DataSize, &ByteAligned);
        
        /* Set the Data */
        if (pObject->DataSize > 0)
        {
            /* Write the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by 2 since this is a 16 bit char platform 
             */             
            for(i=0; i< (pObject->DataSize/2); i++)
            {
                WriteInt16(&pTemp, pObject->Data[i], &ByteAligned);
            }
            
            /* Check for remaining byte from above division */
            if (pObject->DataSize % 2 == 1)             
                WriteInt8(&pTemp, (pObject->Data[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);
        }
        
        /* Set the CRC */
        CRCSize = CError_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Deserializes the contents of the buffer into the CError object passed in.
*********************************************************************************/
boolean CError_Deserialize(CError *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;
    
    if (pTemp != NULL)
    {
        uint16  Size; 
        uint16  StoredCRC;
        uint32  CRCSize;
        uint16  CalcCRC;
        uint16 i;
        
        /* Deserialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        Size = ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the size of the static structure */
        Size = ReadInt16(&pTemp, &ByteAligned);

        /* Get the class ID of the errored message */
        pObject->ErrorClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the ErrorCode */
        pObject->ErrorCode = ReadInt16(&pTemp, &ByteAligned);

        /* Get the Description and increment the pointer by CERR_DESC_SIZE */
        for(i=0; i<(CERR_DESC_SIZE/2); i++)
            pObject->Description[i] = ReadInt16(&pTemp, &ByteAligned);
 
        /* Get the DataSize */
        pObject->DataSize = ReadInt32(&pTemp, &ByteAligned);
        
        /* Get the Data */
        if (pObject->DataSize > 0)
        {           
            for(i=0; i<(pObject->DataSize/2); i++)
                pObject->Data[i] = ReadInt16(&pTemp, &ByteAligned);

            if (pObject->DataSize % 2 == 1)
                pObject->Data[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;                
        }
        else
            memset(pObject->Data, 0, MAX_DATA_MEMBER_SIZE);


        /* Calculate the CRC */
        CRCSize = CError_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
            
        TOUCH(Size);
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CError(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint8 Sample[] = "Error";
    uint32 Result = 0;
    uint8 *pTemp;
    boolean ByteAligned = TRUE;
    uint32 i;

    CError SrcObj;
    CError DestObj;

    CError_Init(&SrcObj, CERR_SpWACPSTACK);
    CError_Init(&DestObj, CERR_SpWACPSTACK);

    pTemp = (uint8*)SrcObj.Data;
    
    for (i=0; i < sizeof(Sample); i++)
    {
        WriteInt8(&pTemp, Sample[i], &ByteAligned);
    }
    
    SrcObj.DataSize = sizeof(Sample);
    SrcObj.ErrorClassID = (uint32)FmNIBP << 16 | GnCOMMAND << 8 | 0;
    SrcObj.ErrorCode = CERR_UNIMPL;

    pTemp = (uint8*)SrcObj.Description;
    
    for (i=0; i < sizeof(Sample); i++)
    {
        WriteInt8(&pTemp, Sample[i], &ByteAligned);
    }

    Result = CError_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CError_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif




