/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFlashCfgDevice.c :                                               //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CFlashCfgDevice.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CFlashCfgDevice_Init(CFlashCfgDevice *new)
{
    new->ClassID = (uint32)FmFLASH << 16 | GnCONFIG << 8 | (uint8)CFLSCDEV_SpDEVICEFLASHCFG;
    new->DirtyFlag = 0;
    new->m_FlashSize = 0;
    new->m_WordSize = 0;
    new->m_Sectors = 0;
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CFlashCfgDevice_GetPayloadSize(CFlashCfgDevice *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CFLSCDEV_STATIC_DATA_SIZE;            /*Static data size*/

    Size += SIZEOF(uint16);            /* CRC bytes */

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CFlashCfgDevice_SizeAll(CFlashCfgDevice *pObject)
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CFlashCfgDevice_GetPayloadSize(pObject);
}


/*********************************************************************************
Serializes the contents of the CFlashCfgDevice object into the buffer passed in.
*********************************************************************************/
boolean CFlashCfgDevice_Serialize(CFlashCfgDevice *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;    
    uint8 *pTemp = pBuf;
    
    if (pTemp != NULL)
    {
        uint32 CRCSize;
        
        /* Set the classID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the payload size */
        WriteInt16(&pTemp, (uint16)CFlashCfgDevice_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the version */
        WriteInt16(&pTemp, (uint16)CFLSCDEV_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CFLSCDEV_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_FlashSize */
        WriteInt32(&pTemp, pObject->m_FlashSize, &ByteAligned);

        /* Set the m_WordSize */
        WriteInt8(&pTemp, pObject->m_WordSize, &ByteAligned);
        
        /* Set the m_Sectors */
        WriteInt8(&pTemp, pObject->m_Sectors, &ByteAligned);
        
        /* Set the CRC */        
        CRCSize = CFlashCfgDevice_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Deserializes the contents of the buffer into the CFlashCfgDevice object passed in.
*********************************************************************************/
boolean CFlashCfgDevice_Deserialize(CFlashCfgDevice *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;
    
    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);

        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_FlashSize */
        pObject->m_FlashSize = ReadInt32(&pTemp, &ByteAligned);

        /* Get the m_WordSize */
        pObject->m_WordSize = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the m_Sectors */
        pObject->m_Sectors = ReadInt8(&pTemp, &ByteAligned);
        
        /* Calculate the CRC */
        CRCSize = CFlashCfgDevice_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CFlashCfgDevice(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result = 0;

    CFlashCfgDevice SrcObj;
    CFlashCfgDevice DestObj;

    CFlashCfgDevice_Init(&SrcObj);
    CFlashCfgDevice_Init(&DestObj);
    
    SrcObj.DirtyFlag = TRUE;
    SrcObj.m_FlashSize = 1000;
    SrcObj.m_WordSize = 16;
    SrcObj.m_Sectors = 50;
     
    Result = CFlashCfgDevice_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CFlashCfgDevice_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CFlashCfgDevice -------------------------------- */
