/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFlashCfgSector.c :                                               //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CFlashCfgSector.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CFlashCfgSector_Init(CFlashCfgSector *new)
{
    new->ClassID = (uint32)FmFLASH << 16 | GnCONFIG << 8 | (uint8)CFLSCSEC_SpSECTORFLASHCFG;
    new->DirtyFlag = 0;
    new->m_Sector = 0;
    new->m_SectorSize = 0;
    new->m_Protected = 0;
    new->m_Content = 0;
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CFlashCfgSector_GetPayloadSize(CFlashCfgSector *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CFLSCSEC_STATIC_DATA_SIZE;            /*Static data size*/

    Size += SIZEOF(uint16);            /* CRC bytes */

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CFlashCfgSector_SizeAll ( CFlashCfgSector *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CFlashCfgSector_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CFlashCfgSector object into the buffer passed in.
*********************************************************************************/
boolean CFlashCfgSector_Serialize (CFlashCfgSector *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;
    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CFlashCfgSector_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CFLSCSEC_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CFLSCSEC_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_Sector */
        WriteInt8(&pTemp, pObject->m_Sector, &ByteAligned);

        /* Set the m_SectorSize */
        WriteInt32(&pTemp, pObject->m_SectorSize, &ByteAligned);

        /* Set the m_Protected */
        WriteInt8(&pTemp, pObject->m_Protected, &ByteAligned);

        /* Set the m_Content */
        WriteInt8(&pTemp, pObject->m_Content, &ByteAligned);

        /* Set the CRC */
        CRCSize = CFlashCfgSector_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CFlashCfgSector_Deserialize(CFlashCfgSector *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CFlashCfgSector static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_Sector */
        pObject->m_Sector = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_SectorSize */
        pObject->m_SectorSize = ReadInt32(&pTemp, &ByteAligned);

        /* Get the m_Protected */
        pObject->m_Protected = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_Content */
        pObject->m_Content = ReadInt8(&pTemp, &ByteAligned);

        /* Calculate the CRC */
        CRCSize = CFlashCfgSector_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CFlashCfgSector(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result = 0;

    CFlashCfgSector SrcObj;
    CFlashCfgSector DestObj;

    CFlashCfgSector_Init(&SrcObj);
    CFlashCfgSector_Init(&DestObj);
    
    SrcObj.DirtyFlag = TRUE;
    SrcObj.m_Sector = 3;
    SrcObj.m_SectorSize = 500;
    SrcObj.m_Protected = 1;
    SrcObj.m_Content = 33;

    Result = CFlashCfgSector_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CFlashCfgSector_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CFlashCfgSector -------------------------------- */
