/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFlashDSegment.c :                                                //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CFlashDSegment.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CFlashDSegment_Init(CFlashDSegment *new)
{
    new->ClassID = (uint32)FmFLASH << 16 | GnDATA << 8 | (uint8)CFLSDSEG_SpFLASHSEG;
    new->DirtyFlag = 0;
    new->m_Sector = 0;
    new->m_SegmentSize = 0;
    new->m_Start = 0;
    new->m_ContentsSize = 0;
    memset(new->m_Contents, 0, MAX_DATA_MEMBER_SIZE);
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CFlashDSegment_GetPayloadSize(CFlashDSegment *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CFLSDSEG_STATIC_DATA_SIZE;            /*Static data size*/

    Size += SIZEOF(uint16) + pObject->m_ContentsSize;

    Size += SIZEOF(uint16);            /* CRC bytes */

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CFlashDSegment_SizeAll ( CFlashDSegment *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CFlashDSegment_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CFlashDSegment object into the buffer passed in.
*********************************************************************************/
boolean CFlashDSegment_Serialize (CFlashDSegment *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;
    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CFlashDSegment_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CFLSDSEG_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CFLSDSEG_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_Sector */
        WriteInt8(&pTemp, pObject->m_Sector, &ByteAligned);

        /* Set the m_SegmentSize */
        WriteInt32(&pTemp, pObject->m_SegmentSize, &ByteAligned);

        /* Set the m_Start */
        WriteInt32(&pTemp, pObject->m_Start, &ByteAligned);

         /* Set the m_ContentsSize */
        WriteInt16(&pTemp, pObject->m_ContentsSize, &ByteAligned);
        
        /* Set the m_Contents */
        if (pObject->m_ContentsSize > 0)
        {
            uint16 i;

            /* Write the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by SIZEOF(uint16) since this is a 16 bit char platform 
             */             
            for(i=0; i< (pObject->m_ContentsSize/SIZEOF(uint16)); i++)
            {
                WriteInt16(&pTemp, pObject->m_Contents[i], &ByteAligned);
            }

            /* Check for remaining byte from above division */
            if (pObject->m_ContentsSize % 2 == 1)             
                WriteInt8(&pTemp, (pObject->m_Contents[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);            
         }

        /* Set the CRC */
        CRCSize = CFlashDSegment_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CFlashDSegment_Deserialize(CFlashDSegment *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CFlashDSegment static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_Sector */
        pObject->m_Sector = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_SegmentSize */
        pObject->m_SegmentSize = ReadInt32(&pTemp, &ByteAligned);

        /* Get the m_Start */
        pObject->m_Start = ReadInt32(&pTemp, &ByteAligned);

        /* Get the m_ContentsSize */
        pObject->m_ContentsSize = ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the m_Contents */
        if (pObject->m_ContentsSize > 0)
        {
            uint16 i;
            
            /* Read the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by SIZEOF(uint16) since this is a 16 bit char platform 
             */             
            for(i=0; i<(pObject->m_ContentsSize/SIZEOF(uint16)); i++)
                pObject->m_Contents[i] = ReadInt16(&pTemp, &ByteAligned);
                
            if (pObject->m_ContentsSize % 2 == 1)
                pObject->m_Contents[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;

        }

        /* Calculate the CRC */
        CRCSize = CFlashDSegment_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CFlashDSegment(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result = 0;
    uint8 *pTemp;
    boolean ByteAligned = TRUE;

    CFlashDSegment SrcObj;
    CFlashDSegment DestObj;

    CFlashDSegment_Init(&SrcObj);
    CFlashDSegment_Init(&DestObj);
    
    pTemp = (uint8*)SrcObj.m_Contents;
    SrcObj.m_ContentsSize = 4;
    
    WriteInt16(&pTemp, 0x6162, &ByteAligned);
    WriteInt16(&pTemp, 0x6364, &ByteAligned);

    SrcObj.DirtyFlag = TRUE;
    SrcObj.m_Sector = 3;
    SrcObj.m_SegmentSize = 500;
    SrcObj.m_Start = 1;
    
    Result = CFlashDSegment_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CFlashDSegment_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CFlashDSegment -------------------------------- */
