/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CMsgSvr.c      :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CMsgSvr.h"

void CMsgSvr_Init(CMsgSvr *new, uint16 Family, uint8 Genus, uint8 Species)
{
    new->ClassID = (uint32)Family << 16 | Genus << 8 | Species;
    new->DataSize = 0;
    memset(new->Data, 0, MAX_DATA_MEMBER_SIZE);
}


/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CMsgSvr_GetPayloadSize( CMsgSvr *pObject )
{
    uint32 size  = SIZEOF(uint8)          /* Size of encryption/compression byte*/
                 + SIZEOF(uint32)         /*Size of the data array*/
                 + pObject->DataSize      /*data array size*/
                 + SIZEOF(uint16);        /* CRC bytes */

    return size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CMsgSvr_SizeAll(CMsgSvr *pObject)
{
    /* ClassID + Payload Length(uint32 aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint32) + CMsgSvr_GetPayloadSize(pObject);
}


/*********************************************************************************
Serializes the contents of the CMsgSvr object into the buffer passed in.
*********************************************************************************/
boolean CMsgSvr_Serialize(CMsgSvr *pObject, uint8 *pBuf)
{
    uint8   *pTemp = pBuf;
    byte    EncryptByte = 0;
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;
    
    if (pTemp != NULL)
    {
        uint16 CRCSize;
        
        /* Serialize the DtData members */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        WriteInt32(&pTemp, CMsgSvr_GetPayloadSize(pObject), &ByteAligned);
        
        /* Serialize the CMsgSvr members */
        /* Set the Encryption byte */
        WriteInt8(&pTemp, EncryptByte, &ByteAligned);

        /* Set the Data size */
        WriteInt32(&pTemp, pObject->DataSize, &ByteAligned);
        
        /* Set the Data */
        if (pObject->DataSize > 0)
        {
            uint16 i;

            /* Write the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by 2 since this is a 16 bit char platform 
             */             
            for(i=0; i< (pObject->DataSize/2); i++)
            {
                WriteInt16(&pTemp, pObject->Data[i], &ByteAligned);
            }
            
            /* Check for remaining byte from above division */
            if (pObject->DataSize % 2 == 1)             
                WriteInt8(&pTemp, (pObject->Data[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);
        }

        /* Set the CRC */        
        CRCSize = CMsgSvr_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}
    
/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CMsgSvr_Deserialize( CMsgSvr *pObject, uint8 *pBuf)
{   
    uint8   *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;
        
    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt32(&pTemp, &ByteAligned);
        
        /* Get the Encrypt/Compression Byte */
        ReadInt8(&pTemp, &ByteAligned);
        
        /* Deserialize the CMsgSvr dynamic members */
        /* Get the DataSize */
        pObject->DataSize = ReadInt32(&pTemp, &ByteAligned);
        
        /* Get the m_Sample */
        if (pObject->DataSize > 0)
        {
            uint16 i;
            
            for(i=0; i<(pObject->DataSize/2); i++)
                pObject->Data[i] = ReadInt16(&pTemp, &ByteAligned);

            if (pObject->DataSize % 2 == 1)
                pObject->Data[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;
        }

        /* Calculate the CRC */
        CRCSize = CMsgSvr_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}



