/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CRNDZApp.c     :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CRNDZApp.h"

#define SEND_COMMAND(Family, CommandID) WACP_SendMsg(Family, GnCOMMAND, CommandID, NULL, 0)
#define SEND_REQUEST(Family, RequestID) WACP_SendMsg(Family, GnREQUEST, RequestID, NULL, 0)
#define SEND_RESPONSE(Family, ResponseID, dataBuffer,dataSize) WACP_SendMsg(Family, GnRESPONSE, ResponseID, dataBuffer, dataSize)

static boolean RNDZCompleteFlag;
static boolean RNDZStatusFlag;

boolean CRNDZApp_RxMsg(uint8 Genus, uint8 MsgID, uint8* pBuf)
{
    boolean Result = FALSE;

    TOUCH(pBuf);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            if (MsgID == RNDZ_CCMD_SpSEND_HOST_READY)
            {
                
                /* Properly terminate the RNDZ Session */
                CRNDZApp_Terminate(TRUE);

            }
            else
                CRNDZApp_Terminate(FALSE);

            break;
        }
        case GnRESPONSE:
        {
            if ( MsgID == RNDZ_CRSP_SpRNDZ_ACCEPT)
            {
                /* Send the START HOST PROC command */
                Result = SEND_COMMAND(FmRENDEZVOUS, RNDZ_CCMD_SpSTART_HOST_PROC);
            }
            
            break;
        }
    };

    return (Result);
}

/*================================================================================
InitClient : Sends a command to initiate a Rendezvous session
================================================================================*/
boolean CRNDZApp_InitClient()
{
    /* Start the RNDZ Connection */
    boolean Result=FALSE;
    boolean ByteAligned=TRUE;
    uint8 MsgBuf[] = "RNDZCONNECT";
    uint8 TempBuf[RNDZ_CONNECT_SIZE];
    uint8 *pTemp = TempBuf;
    uint8 i;
    
    /* Pack the message */
    for (i=0; i< RNDZ_CONNECT_SIZE; i++)
    {
        WriteInt8(&pTemp, MsgBuf[i] & 0xFF, &ByteAligned);
    }
    
    /* Start by creating a reqeust */
    Result = WACP_SendMsg(FmRENDEZVOUS, GnREQUEST, RNDZ_CRST_SpSTART_RNDZ, TempBuf, RNDZ_CONNECT_SIZE);

    return Result;
}

/*================================================================================
Terminate : 
================================================================================*/
void CRNDZApp_Terminate(boolean bStatus)
{         
    RNDZCompleteFlag = TRUE;
    RNDZStatusFlag = bStatus;   
}
/*================================================================================
AppStart
================================================================================*/
boolean CRNDZApp_AppStart()
{
    boolean Result = TRUE;
    uint8   TimeOut;

    /* Initialize the status flags */
    RNDZCompleteFlag = FALSE;
    RNDZStatusFlag = FALSE; 
    
    /* Set the Port to RNDZ */
    InterChange_SetPort(RNDZ_PORT_DEFAULT);
    
#ifndef STACK_SERIAL_DTR
    /* If DTR is not supported, then the client must manually start the connection */
    Result = CRNDZApp_InitClient(); 
#endif

    /* Check for a successful connection for clients */
    if (Result == (boolean)TRUE)
    {    
        /* Wait until we the RNDZ session completes */
        while(RNDZCompleteFlag == FALSE)
        {
            SYS_SLEEP(RNDZ_WAIT_TIME);
            
            /* Exit if RNDZ has not completed by the timeout */
            if (++TimeOut > RNDZ_TIMEOUT)
            	break;
        } 
    }
    
    /* Set the port to WACP upon a successful RNDZ Session */
    if (RNDZStatusFlag == TRUE)
    {
        InterChange_SetPort(WACP_PORT_DEFAULT);     
    }
        
    return(RNDZStatusFlag);
}

