/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CTOCDStd.c :                                              //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CTOCDStd.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CTOCDStd_Init(CTOCDStd *new, uint8 Species)
{
    new->ClassID = (uint32)FmTOC << 16 | GnDATA << 8 | (uint8)Species;
    new->DirtyFlag = 0;
    memset(&new->m_SDateTime, 0, sizeof(new->m_SDateTime));
    memset(new->m_Contents, 0, CTOCDSTD_MAX_DATA_SIZE);
    new->m_ContentsSize = 0;
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CTOCDStd_GetPayloadSize(CTOCDStd *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CTOCDSTD_STATIC_DATA_SIZE;            /*Static data size*/

    Size += SIZEOF(uint16) + pObject->m_ContentsSize;

    Size += SIZEOF(uint16);            /* CRC bytes */

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CTOCDStd_SizeAll ( CTOCDStd *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CTOCDStd_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CTOCDStd object into the buffer passed in.
*********************************************************************************/
boolean CTOCDStd_Serialize (CTOCDStd *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;
    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CTOCDStd_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CTOCDSTD_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CTOCDSTD_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_SDateTime structure */
        WriteInt16(&pTemp, pObject->m_SDateTime.year, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_SDateTime.month, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_SDateTime.day, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_SDateTime.hour, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_SDateTime.minute, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_SDateTime.second, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_SDateTime.format, &ByteAligned);

         /* Set the m_ContentsSize */
        WriteInt16(&pTemp, pObject->m_ContentsSize, &ByteAligned);
        
        /* Set the m_Contents */
        if (pObject->m_ContentsSize > 0)
        {
            uint16 i;

            /* Write the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by SIZEOF(uint32) since this is a 16 bit char platform.
             */             
            for(i=0; i<(pObject->m_ContentsSize/SIZEOF(uint32)); i++)
            {
                WriteInt32(&pTemp, pObject->m_Contents[i], &ByteAligned);
            }
        }

       /* Set the CRC */
        CRCSize = CTOCDStd_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CTOCDStd_Deserialize(CTOCDStd *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CTOCDStd static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_SDateTime */
        pObject->m_SDateTime.year = ReadInt16(&pTemp, &ByteAligned);
        pObject->m_SDateTime.month = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_SDateTime.day = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_SDateTime.hour = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_SDateTime.minute = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_SDateTime.second = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_SDateTime.format = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_ContentsSize */
        pObject->m_ContentsSize = ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the m_Contents */
        if (pObject->m_ContentsSize > 0)
        {
            uint16 i;
            
            for(i=0; i<(pObject->m_ContentsSize/SIZEOF(uint32)); i++)
                pObject->m_Contents[i] = ReadInt32(&pTemp, &ByteAligned);
        }

        /* Calculate the CRC */
        CRCSize = CTOCDStd_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}
 
#if defined(COOA_OBJECT_TEST)
boolean Test_CTOCDStd(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint8 *pTemp;
    boolean ByteAligned = TRUE;
    uint32 Result;

    CTOCDStd SrcObj;
    CTOCDStd DestObj;

    CTOCDStd_Init(&SrcObj, CTOCDSTD_SpFLASHSECTOR);
    CTOCDStd_Init(&DestObj, CTOCDSTD_SpFLASHSECTOR);

    pTemp = (uint8*)SrcObj.m_Contents;
    WriteInt32(&pTemp, 0x61626364, &ByteAligned);
    WriteInt32(&pTemp, 0x65666768, &ByteAligned);

    SrcObj.m_ContentsSize = 8;
    
    SrcObj.DirtyFlag = TRUE;
    SrcObj.m_SDateTime.year = 2002;
    SrcObj.m_SDateTime.month = 12;
    SrcObj.m_SDateTime.day = 31;
    SrcObj.m_SDateTime.hour = 2;
    SrcObj.m_SDateTime.minute = 06;
    SrcObj.m_SDateTime.second = 00;
    SrcObj.m_SDateTime.format = 00;
    
    Result = CTOCDStd_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CTOCDStd_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CTOCDStd -------------------------------- */
