/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPDIndexer.c:                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CWACPDIndexer.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CWACPDIndexer_Init(CWACPDIndexer *new)
{
    new->ClassID = (uint32)FmWACP << 16 | GnDATA << 8 | (uint8)CWCPDIDX_SpOBJECTINDEX;
    new->DirtyFlag = 0;
    new->m_NumericID = 0;
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CWACPDIndexer_GetPayloadSize(CWACPDIndexer *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CWCPDIDX_STATIC_DATA_SIZE;        /*Static data size*/
    Size += SIZEOF(uint16);            /* CRC bytes */

    TOUCH(pObject);

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CWACPDIndexer_SizeAll ( CWACPDIndexer *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CWACPDIndexer_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CWACPDIndexer object into the buffer passed in.
*********************************************************************************/
boolean CWACPDIndexer_Serialize (CWACPDIndexer *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    uint8 *pTemp = pBuf;
    
    if (pTemp != NULL)
    {
        uint32 CRCSize;
        boolean ByteAligned = TRUE;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CWACPDIndexer_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CWCPDIDX_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CWCPDIDX_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the Runtime */
        WriteInt16(&pTemp, pObject->m_NumericID, &ByteAligned);

        /* Set the CRC */
        CRCSize = CWACPDIndexer_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CWACPDIndexer_Deserialize(CWACPDIndexer *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;
        boolean ByteAligned = TRUE;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CWACPDIndexer static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Set the Runtime */
        pObject->m_NumericID = ReadInt16(&pTemp, &ByteAligned);

        /* Calculate the CRC */
        CRCSize = CWACPDIndexer_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}
 
#if defined(COOA_OBJECT_TEST)
boolean Test_CWACPDIndexer(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result;

    CWACPDIndexer SrcObj;
    CWACPDIndexer DestObj;

    CWACPDIndexer_Init(&SrcObj);
    CWACPDIndexer_Init(&DestObj);

    SrcObj.m_NumericID = 1;
    
    Result = CWACPDIndexer_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CWACPDIndexer_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CWACPDIndexer -------------------------------- */
