/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWaveSampleCStd.c :                                               //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CWaveSampleCStd.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CWaveSampleCStd_Init(CWaveSampleCStd *new, uint8 Species)
{
    new->ClassID = (uint32)FmWAVESAMPLE << 16 | GnCONFIG << 8 | Species;
    new->DirtyFlag = 0;
    new->m_IsActive = 0;
    new->m_SampleFrequency = 0;
    new->m_SampleSize = 0;
    new->m_SampleCount = 0;
    new->m_Channels = 0;
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CWaveSampleCStd_GetPayloadSize(CWaveSampleCStd *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CWVSCSTD_STATIC_DATA_SIZE;            /*Static data size*/

    Size += SIZEOF(uint16);            /* CRC bytes */

    TOUCH(pObject);

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CWaveSampleCStd_SizeAll ( CWaveSampleCStd *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CWaveSampleCStd_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CWaveSampleCStd object into the buffer passed in.
*********************************************************************************/
boolean CWaveSampleCStd_Serialize (CWaveSampleCStd *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;
    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CWaveSampleCStd_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CWVSCSTD_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CWVSCSTD_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_IsActive */
        WriteInt8(&pTemp, pObject->m_IsActive, &ByteAligned);

        /* Set the m_SampleFrequency */
        WriteInt16(&pTemp, (uint16)pObject->m_SampleFrequency, &ByteAligned);

        /* Set the m_SampleSize */
        WriteInt8(&pTemp, pObject->m_SampleSize, &ByteAligned);

        /* Set the m_SampleCount */
        WriteInt16(&pTemp, (uint16)pObject->m_SampleCount, &ByteAligned);

         /* Set the m_Channels */
        WriteInt8(&pTemp, pObject->m_Channels, &ByteAligned);

       /* Set the CRC */
        CRCSize = CWaveSampleCStd_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CWaveSampleCStd_Deserialize(CWaveSampleCStd *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CWaveSampleCStd static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_IsActive */
        pObject->m_IsActive = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_SampleFrequency */
        pObject->m_SampleFrequency = ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_SampleSize */
        pObject->m_SampleSize = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_SampleCount */
        pObject->m_SampleCount = ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the m_Channels */
        pObject->m_Channels = ReadInt8(&pTemp, &ByteAligned);

        /* Calculate the CRC */
        CRCSize = CWaveSampleCStd_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CWaveSampleCStd(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result = 0;

    CWaveSampleCStd SrcObj;
    CWaveSampleCStd DestObj;

    CWaveSampleCStd_Init(&SrcObj, CWVSCSTD_SpECGWAVCFG);
    CWaveSampleCStd_Init(&DestObj, CWVSCSTD_SpECGWAVCFG);

    SrcObj.m_IsActive = TRUE;
    SrcObj.m_SampleFrequency = 8;
    SrcObj.m_SampleSize = 18;
    SrcObj.m_SampleCount = 8;
    SrcObj.m_Channels = 1;

    Result = CWaveSampleCStd_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CWaveSampleCStd_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif

  
/* ---------------------------End of DataClass CWaveSampleCStd -------------------------------- */
