/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWaveSampleDStd.c :                                               //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CWaveSampleDStd.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CWaveSampleDStd_Init(CWaveSampleDStd *new, uint8 Species)
{
    new->ClassID = (uint32)FmWAVESAMPLE << 16 | GnDATA << 8 | Species;
    new->m_SampleSize=0;
    new->m_SequenceNumber = 0;
    new->DirtyFlag = 0;
    memset(new->m_Sample, 0, MAX_DATA_MEMBER_SIZE);
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CWaveSampleDStd_GetPayloadSize(CWaveSampleDStd *pObject)
{
    uint32 MySize = SIZEOF(uint16)                  /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the struct*/
            + CWSPDSTD_STATIC_DATA_SIZE;            /*Static data size*/

    MySize += SIZEOF(uint16) + pObject->m_SampleSize;

    MySize = MySize + SIZEOF(uint16);            /* CRC bytes */

    return MySize;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CWaveSampleDStd_SizeAll(CWaveSampleDStd *pObject)
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CWaveSampleDStd_GetPayloadSize(pObject);
}


/*********************************************************************************
Serializes the contents of the CWaveSampleDStd object into the buffer passed in.
*********************************************************************************/
boolean CWaveSampleDStd_Serialize(CWaveSampleDStd *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;    
    uint8 *pTemp = pBuf;
    
    if (pTemp != NULL)
    {
        uint32 CRCSize;
        
        /* Set the classID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the payload size */
        WriteInt16(&pTemp, (uint16)CWaveSampleDStd_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the version */
        WriteInt16(&pTemp, (uint16)CWSPDSTD_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CWSPDSTD_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_SequenceNumber */
        WriteInt32(&pTemp, pObject->m_SequenceNumber, &ByteAligned);

        /* Set the m_SampleSize */
        WriteInt16(&pTemp, pObject->m_SampleSize, &ByteAligned);
        
        /* Set the m_Sample */
        if (pObject->m_SampleSize > 0)
        {
            uint16 i;

            /* Write the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by 2 since this is a 16 bit char platform 
             */             
            for(i=0; i< (pObject->m_SampleSize/2); i++)
            {
                WriteInt16(&pTemp, pObject->m_Sample[i], &ByteAligned);
            }
            
            /* Check for remaining byte from above division */
            if (pObject->m_SampleSize % 2 == 1)             
                WriteInt8(&pTemp, (pObject->m_Sample[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);
        }

        /* Set the CRC */        
        CRCSize = CWaveSampleDStd_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Deserializes the contents of the buffer into the CWaveSampleDStd object passed in.
*********************************************************************************/
boolean CWaveSampleDStd_Deserialize(CWaveSampleDStd *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;
    
    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);

        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_SequenceNumber */
        pObject->m_SequenceNumber = ReadInt32(&pTemp, &ByteAligned);

        /* Get the m_SampleSize */
        pObject->m_SampleSize = ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the m_Sample */
        if (pObject->m_SampleSize > 0)
        {
            uint16 i;
            
            for(i=0; i<(pObject->m_SampleSize/2); i++)
                pObject->m_Sample[i] = ReadInt16(&pTemp, &ByteAligned);

            if (pObject->m_SampleSize % 2 == 1)
                pObject->m_Sample[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;
        }

        /* Calculate the CRC */
        CRCSize = CWaveSampleDStd_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CWaveSampleDStd(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint8 *pTemp;
    boolean ByteAligned = TRUE;
    uint32 Result = 0;

    CWaveSampleDStd SrcObj;
    CWaveSampleDStd DestObj;

    CWaveSampleDStd_Init(&SrcObj, CWSPDSTD_SpSTANDARD);
    CWaveSampleDStd_Init(&DestObj, CWSPDSTD_SpSTANDARD);

    pTemp = (uint8*)SrcObj.m_Sample;
    
    WriteInt8(&pTemp, 'a', &ByteAligned);
    WriteInt8(&pTemp, 'b', &ByteAligned);
    WriteInt8(&pTemp, 'c', &ByteAligned);
    WriteInt8(&pTemp, 'd', &ByteAligned);
    WriteInt8(&pTemp, 'e', &ByteAligned);

    
    SrcObj.DirtyFlag = TRUE;
    SrcObj.m_SampleSize = 5;
    SrcObj.m_SequenceNumber = 3;

    Result = CWaveSampleDStd_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CWaveSampleDStd_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif

/* ---------------------------End of DataClass CWaveSampleDStd -------------------------------- */
