/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Mem.c          :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "Mem.h"

const uint16 CRCTable[ 256 ] = {
    0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
    0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
    0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
    0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
    0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
    0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
    0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
    0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
    0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
    0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
    0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
    0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
    0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
    0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
    0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x738,
    0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
    0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
    0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
    0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
    0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
    0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
    0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
    0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
    0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
    0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
    0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
    0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
    0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
    0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
    0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
    0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
    0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0xf78
};

/*================================================================================
ReadInt8 Endian-proof function to read a 8 bit integer from a buffer.
This assumes the Media Endianness is BIG_ENDIAN which is the default 
for all WACP messages.
================================================================================*/
uint8 ReadInt8(uint8** ppBuf, boolean *ByteAligned)
{
    uint8  *pBuf = *ppBuf;      /* Dereference buffer pointer */
    uint16 BitMask;
    uint8  Result;
    
    /* Increment pointer if the byte alignment is */    
    if (*ByteAligned == TRUE)
    {
        BitMask = HIGH_BYTE_MASK;
        Result = (*pBuf & BitMask) >> 8;   
    }
    else
    {
        BitMask = LOW_BYTE_MASK;
        *ppBuf += sizeof(uint8);
        Result = (*pBuf & BitMask);
    }
       
    /* Toggle the ByteAligned flag */
    *ByteAligned = !(*ByteAligned);        
    
    return Result;
}

/*================================================================================
ReadInt16 Endian-proof function to read a 16 bit integer from a buffer.
This assumes the Media Endianness is BIG_ENDIAN which is the default 
for all WACP messages.
================================================================================*/
uint16 ReadInt16(uint8** ppBuf, boolean *ByteAligned)
{
    uint8  *pBuf = *ppBuf;      /* Dereference buffer pointer */
    uint16  Result;
    
    *ppBuf += sizeof(uint16);    /* Increment pointer */    

    /* Check the byte alignment*/   
    if (*ByteAligned == TRUE)
    {
        Result = *pBuf;
    }
    else
    {
        Result = (uint16)(*pBuf & LOW_BYTE_MASK) << 8 |
                 (uint16)(*(pBuf+1) & HIGH_BYTE_MASK) >> 8;        
    }

    return (Result);
}

/*================================================================================
ReadInt32 Endian-proof function to read an 32 bit integer from a buffer.
This assumes the Media Endianness is BIG_ENDIAN which is the default 
for all WACP messages.
================================================================================*/
uint32 ReadInt32(uint8** ppBuf, boolean *ByteAligned)
{
    uint8 * pBuf = *ppBuf;       /* Dereference buffer pointer */
    uint32  Result;
    
    *ppBuf += sizeof(uint32);    /* Increment pointer */    

    /* Check the byte alignment*/   
    if (*ByteAligned == TRUE)
    {
        Result = (uint32)(*pBuf) << 16 | (uint32)(*(pBuf+1));
    }
    else
    {
        /* Get the low 8 bits from the current word and the high
         * 8 bits from the next word to create the high 16 bits of
         * the 32 bit value. The low 16 bits are created from the 
         * next low and high 8 bits and combined with the high 16 bits
         */
        Result = (((uint32)(*pBuf & LOW_BYTE_MASK) << 8 |
                   (uint32)(*(pBuf+1) & HIGH_BYTE_MASK) >> 8)) << 16 |        
                   (uint32)(*(pBuf+1) & LOW_BYTE_MASK) << 8 |
                   (uint32)(*(pBuf+2) & HIGH_BYTE_MASK) >> 8;        
    }

    return (Result);
}

/*================================================================================
WriteInt8 Endian-proof function to write a 8 bit integer into a buffer.
This assumes the Media Endianness is BIG_ENDIAN which is the default 
for all WACP messages.
================================================================================*/
void WriteInt8(uint8** ppBuf, uint8 Val, boolean *ByteAligned)
{
    uint8 * pBuf = *ppBuf;       /* Dereference buffer pointer */    
    uint16 BitMask;
    
    /* Check the byte alignment*/   
    if (*ByteAligned == TRUE)
    {
        /* Shift Val into the high 8 bits of the 16 bit word */
        Val = (Val << 8) & HIGH_BYTE_MASK;
        
        /* Set the BitMask to LOW_BYTE_MASK */
        BitMask = LOW_BYTE_MASK;    
    }
    else
    {
        /* Grab the low 8 bits of the 16 bit word */
        Val = Val & LOW_BYTE_MASK;

        /* Set the BitMask to HIGH_BYTE_MASK*/
        BitMask = HIGH_BYTE_MASK;
        *ppBuf += sizeof(uint8);    
    }
    
    *ByteAligned = !(*ByteAligned);               
    *pBuf = (uint8)(*pBuf & BitMask)  | (uint8)(Val) ;
}

/*================================================================================
WriteInt16 Endian-proof function to write a 16 bit integer into a buffer.
This assumes the Media Endianness is BIG_ENDIAN which is the default 
for all WACP messages.
================================================================================*/
void WriteInt16(uint8** ppBuf, uint16 Val, boolean *ByteAligned)
{
    uint8 * pBuf = *ppBuf;       /* Dereference buffer pointer */
    *ppBuf += sizeof(uint16);    /* Increment pointer */
    
    /* Check the byte alignment*/   
    if (*ByteAligned == TRUE)
    {
        *(pBuf) = (uint8)(Val & HIGH_BYTE_MASK)  | (uint8)(Val) & LOW_BYTE_MASK;
    }
    else
    {
        uint8 HighBits, LowBits;

        /* Separate the 16 bit value into high and low bits and flip flop them */
        HighBits = (Val & HIGH_BYTE_MASK) >> 8;
        LowBits = (Val & LOW_BYTE_MASK) << 8;

        /* Set the high bits into the low bits of the current word boundary */
        *pBuf = (uint8)(*pBuf & HIGH_BYTE_MASK)  | HighBits;

        /* Set the low bits into the high bits of the next word boundary */
        *(pBuf+1) = (uint8)(*(pBuf+1) & LOW_BYTE_MASK)  | LowBits;
    }
}

/*================================================================================
WriteInt32 Endian-proof function to write a 32 bit integer into a buffer.
This assumes the Media Endianness is BIG_ENDIAN which is the default 
for all WACP messages.
================================================================================*/
void WriteInt32(uint8** ppBuf, uint32 Val, boolean *ByteAligned)
{
    uint8 *pBuf = *ppBuf;       /* Dereference buffer pointer */

    *ppBuf += sizeof(uint32);    /* Increment pointer */
    
    /* Check the byte alignment*/   
    if (*ByteAligned == TRUE)
    {    
        *(pBuf)   = (uint16)(Val >> 16);    
        *(pBuf+1) = (uint16)Val;
    }
    else
    {
        uint16 UpperHighBits, UpperLowBits, HighBits, LowBits;

        /* Separate the 16 bit value into high and low bits and flip flop them */
        UpperHighBits = ((uint16)(Val >> 16) & HIGH_BYTE_MASK) >> 8;
        UpperLowBits = ((uint16)(Val >> 16) & LOW_BYTE_MASK) << 8;
        HighBits = ((uint16)Val & HIGH_BYTE_MASK) >> 8;
        LowBits = ((uint16)Val & LOW_BYTE_MASK) << 8;
        
        /* Set the high bits into the low bits of the current word boundary */
        *pBuf = (uint8)(*pBuf & HIGH_BYTE_MASK)  | UpperHighBits;

        /* Set the low bits into the high bits of the next word boundary */
        *(pBuf+1) = (uint8)(*(pBuf+1) & LOW_BYTE_MASK)  | UpperLowBits;
        
        /* Set the high bits into the low bits of the current word boundary */
        *(pBuf+1) = (uint8)(*(pBuf+1) & HIGH_BYTE_MASK)  | HighBits;

        /* Set the low bits into the high bits of the next word boundary */
        *(pBuf+2) = (uint8)(*(pBuf+2) & LOW_BYTE_MASK)  | LowBits;
    }
}

/*-D******************************************************************
 *
 *  FUNCTION: uint16 CalculateCRC(uint16 CurrentCRC,
 *                              uint8 *pBuf,
 *                              uint32 BufSize);
 *
 *  PARAMETERS: CurrentCRC - Current CRC value.
 *              pBuf - pointer to data to calc. CRC on
 *              BufSize - length of Buffer to calc. CRC on
 *              ByteAligned - Flag used to specify whether the data
 *                            is aligned on a byte boundary or not.
 *
 *  DESCRIPTION: Calculates the 16-bit CCITT crc. The algorithm 
 *               uses the value of CurrentCRC to compute that
 *               new crc value. To begin a crc computation, the
 *               initial value of CurrentCRC should be set to
 *               0xFFFF.
 *
 *  RETURNS: 16-bit CRC
 *
 *-E*****************************************************************/

uint16 CalculateCRC(uint16 CurrentCRC, uint8 *pBuf, uint32 BufSize, boolean ByteAligned)
{
    uint16 CRC = CurrentCRC;
    while (BufSize--)
    {
        CRC = (CRC >> 8) ^ CRCTable[ReadInt8(&pBuf,&ByteAligned) ^ (CRC & 0x00FF)];
    }
    return (CRC);
}

