/*-D*************************************************************************
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// StdVarTyp.h    :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   WACP                                           //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :                                                  //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(STDVARTYP_H)
#define STDVARTYP_H

#include "target.h"
#include "SysUtil.h"

#if defined(__cplusplus)
    #include <string>
#endif


/************************************************************************
  Derived data types {C lover (Knut's) aliases}
*************************************************************************/
#if !defined (INT8_DEFINED)
#define INT8_DEFINED

typedef signed char         int8;
typedef short               int16;
typedef long                int32;

typedef unsigned char       uint8;
typedef unsigned short      uint16;
typedef unsigned long       uint32;

/*typedef float               float32;*/
typedef double              float64;

typedef void* HANDLE;
/*****************************************
uint64 and int64 for temporary usage
*****************************************/
#if defined (_INTEGRAL_MAX_BITS) && \
  _INTEGRAL_MAX_BITS >= 64
typedef signed __int64      int64;
typedef unsigned __int64    uint64;
#else
/* #error __int64 type not supported  */
#endif

typedef unsigned char       byte;
typedef unsigned short      word;
typedef unsigned char       boolean;

typedef char                ansichar;

#if defined(_WCHAR_T ) || defined(_WCHAR_T_DEFINED)
  typedef wchar_t           widechar;
#else
  typedef unsigned short    widechar;
#endif

#endif /* #if !defined(INT8_DEFINED) */

typedef struct MOIB_TIME {
    uint16 year;    /* e.g 1994 */
    byte month;     /* Month e.g 08 */
    byte day;       /* Day of month e.g. 23 */
    byte hour;      /* Hour .e.g. 15 (3pm) */
    byte minute;    /* Minute e.g. 30 */
    byte second;    /* Seconds e.g. 59 */
    byte format;
} MOIB_TIME;
#define MOIB_TIME_SIZE (SIZEOF(uint16)+6*(SIZEOF(uint8)))   /* Size without padding */

/************************************************************************
A wchar and char object that is a varying-length sequence of elements or
what is more poularly known as a 'string'
*************************************************************************/
#if defined(__cplusplus)
    typedef std::basic_string <char> string;
    typedef std::basic_string <wchar_t> wstring;
#endif

/************************************************************************
  Limits for derived data types
*************************************************************************/
/*  From: LIMITS.H */
#define INT8_MIN    (-128)              /* minimum  int8  value */
#define INT8_MAX    127                 /* maximum  int8  value */
#define UINT8_MAX   255                 /* maximum uint8  value */

#define INT16_MIN   (-32767-1)          /* minimum  int16 value */
#define INT16_MAX   32767               /* maximum  int16 value */
#define UINT16_MAX  65535U              /* maximum uint16 value */

#define INT32_MIN   (-2147483647L-1)    /* minimum  int32 value */
#define INT32_MAX   2147483647L         /* maximum  int32 value */
#define UINT32_MAX  4294967295UL        /* maximum uint32 value */

/* From: FLOAT.H */
#define FLOAT32_MIN (1.175494351e-38F)  /* min float32 value */
#define FLOAT32_MAX (3.402823466e+38F)  /* max float32 value */

#define FLOAT64_MIN (2.2250738585072014e-308)   /* min float64 value */
#define FLOAT64_MAX (1.7976931348623158e+308)   /* max float64 value */


/************************************************************************
  The following definitions are provided in case the compiler/development
  environment does not provide them.
*************************************************************************/

/* -- TRUE / FALSE -- */
#if !defined (TRUE)
  #define TRUE          (1 == 1)
#endif

#if !defined (FALSE)
  #define FALSE         (!TRUE)
#endif

/* -- ON / OFF -- */
#if !defined (ON)
  #define ON            1
#endif

#if !defined (OFF)
  #define OFF           0
#endif


/* -- NULL -- */
#if !defined NULL
  #define NULL          ((void*) 0 )
#endif


#endif  /* #if !defined (STDVARTYP_H) */

/* End of File */

