/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CRNDZApp.c     :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CRNDZApp.h"
#include "Wacp.h"
#include "CWACPSemaphore.h"
#include "CConObjSvr.h"
#include "CRNDZDStd.h"

#if defined(ENCRYPTION_ON)
#include "AESCrypt.h"
#include "CEncryption.h"
#endif

#define SEND_RNDZ_COMMAND(pSourceID, CommandID, pDataBuffer, DataSize) WACP_SendMsg(pSourceID, FmRENDEZVOUS, GnCOMMAND, CommandID, pDataBuffer, DataSize, FALSE)
#define SEND_RNDZ_REQUEST(pSourceID, RequestID) WACP_SendMsg(pSourceID, FmRENDEZVOUS, GnREQUEST, RequestID, NULL, 0, FALSE)
#define SEND_RNDZ_RESPONSE(pSourceID, ResponseID, pDataBuffer, DataSize) WACP_SendMsg(pSourceID, FmRENDEZVOUS, GnRESPONSE, ResponseID, pDataBuffer, DataSize, FALSE)

static CWACPSemaphore  ExitSem;

/* Static GUIDStd object used for sending GUID in RNDZ */
static CRNDZDStd GUIDStd;

boolean CRNDZApp_RxMsg(char* pSourceID, uint8 Genus, uint8 MsgID, uint8* pBuf)
{
    boolean     Result = FALSE;
    ConnArgs*   pConnArgs = NULL;
    uint8*      pObjBuffer = NULL;
    uint32      ObjSize = 0;

    TOUCH(pBuf);

    CRNDZDStd_Init(&GUIDStd);

    pConnArgs = CLinkTable_GetEntry(&LinkTable, pSourceID);
    if (pConnArgs)
    {
        switch(Genus)
        {
            case GnCOMMAND:
            {
                /* RNDZ_CCMD_SpSTART_HOST_PROC is sent from the client once RNDZ initialization is
                * completed. This includes negotiating the encryption type and key if encryption is
                * enabled. 
                */
                if (MsgID == RNDZ_CCMD_SpSTART_HOST_PROC)
                {
#if defined(TRACE_SERVER_ON)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                        "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                        "%s: \nReceived command for RNDZ_CCMD_SpSTART_HOST_PROC\n", pSourceID);
#endif
					if ((pConnArgs->bRNDZDisabled == TRUE) ||
						(pConnArgs->ValidationState == RNDZ_VALIDATION_START_RNDZ_RECVD) ||
                        (pConnArgs->ValidationState == RNDZ_VALIDATION_KEY_EXCHANGE))
                    {
                        pConnArgs->ValidationState = RNDZ_VALIDATION_COMPLETE;

#if defined(TRACE_SERVER_ON)
                        CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
                            "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                            "%s: \nSending Command SEND_HOST_READY\n", pSourceID);
#endif

                        /* Store the GUID from the Client */
                        if (pBuf != NULL)
                        {
                            if (pConnArgs->pGUIDStd == NULL)
                            {
                                /* Allocate room for the GUID object if it doesn't exists */
                                /* This is free'd up in CInterchange::Disconnect() */
                                pConnArgs->pGUIDStd = SYS_MALLOC(sizeof(CRNDZDStd));
                            }

                            if (pConnArgs->pGUIDStd)
                            {                            
                                CRNDZDStd_Init(pConnArgs->pGUIDStd);
                                Result = CRNDZDStd_Deserialize((CRNDZDStd*)pConnArgs->pGUIDStd, pBuf);
                            }

#if defined(TRACE_SERVER_ON) && ((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
                            if (Result)
                            {
                                uint32 XMLSize = CRNDZDStd_serialize_to_XMLbuf(pConnArgs->pGUIDStd, NULL);
                                char *pTemp = (char*)SYS_MALLOC(XMLSize);
                                if (pTemp != NULL)
                                {
                                    CRNDZDStd_serialize_to_XMLbuf(pConnArgs->pGUIDStd, pTemp);
                                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                                        "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                                        "%s: \nReceived GUID\n\n%s", pSourceID, pTemp);
                                    SYS_FREE(pTemp);
                                }
                            }                            
#endif
                        }

                        /* Send out the Host's GUID */
                        pObjBuffer = (uint8*)SYS_MALLOC(ObjSize = CRNDZDStd_SizeAll(pConnArgs->pGUIDStd));
                        if (pObjBuffer)
                        {
                            CRNDZDStd_Serialize(pConnArgs->pGUIDStd, pObjBuffer);
                        }

                        /* Send the HOST READY command */
                        Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpSEND_HOST_READY, pObjBuffer, ObjSize);
                        
                        /* Free up the memory allocation */
                        SYS_FREE(pObjBuffer);
                        pObjBuffer = NULL;

                        /* Properly terminate the RNDZ Session */
                        CRNDZApp_Terminate(pSourceID, TRUE);
                        
                        Interchange_UpdateStatus(pSourceID);
                    }
                    else
                    {
                        CRNDZApp_Terminate(pSourceID, FALSE);
                    }
                }
                /* RNDZ_CCMD_SpSEND_HOST_READY is sent from the host indicating a successful
                 * RNDZ authentication. This signals the client to terminate the RNDZ client
                 * and to start up the WACP session.
                 */
                else if (MsgID == RNDZ_CCMD_SpSEND_HOST_READY)
                {
#if defined(TRACE_SERVER_ON)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                        "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                        "%s: \nReceived command for RNDZ_CCMD_SpSEND_HOST_READY\n", pSourceID);
#endif
                    /* Store the GUID from the Host */
                    if (pBuf != NULL)
                    {
                        if (pConnArgs->pGUIDStd == NULL)
                        {
                            /* Allocate room for the GUID object if it doesn't exists */
                            /* This is free'd up in CInterchange::Disconnect() */
                            pConnArgs->pGUIDStd = SYS_MALLOC(sizeof(CRNDZDStd));
                        }

                        if (pConnArgs->pGUIDStd)
                        {
                            CRNDZDStd_Init(pConnArgs->pGUIDStd);
                            Result = CRNDZDStd_Deserialize(pConnArgs->pGUIDStd, pBuf);
                        }

#if defined(TRACE_SERVER_ON) && ((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
                        if (Result)
                        {
                            uint32 XMLSize = CRNDZDStd_serialize_to_XMLbuf(pConnArgs->pGUIDStd, NULL);
                            char *pTemp = (char*)SYS_MALLOC(XMLSize);
                            if (pTemp != NULL)
                            {
                                CRNDZDStd_serialize_to_XMLbuf(pConnArgs->pGUIDStd, pTemp);
                                CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                                    "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                                    "%s: \nReceived GUID\n\n%s", pSourceID, pTemp);
                                SYS_FREE(pTemp);
                            }

                        }
#endif
                    }

                    Result = TRUE;

                    /* Properly terminate the RNDZ Session */
                    CRNDZApp_Terminate(pSourceID, TRUE);
                    
                    Interchange_UpdateStatus(pSourceID);
                }
                else
                    CRNDZApp_Terminate(pSourceID, FALSE);

                break;
            }
            case GnRESPONSE:
            {
                /* Send out the Host's GUID */
                pObjBuffer = (uint8*)SYS_MALLOC(ObjSize = CRNDZDStd_SizeAll(&GUIDStd));
                if (pObjBuffer)
                {
                    CRNDZDStd_Serialize(&GUIDStd, pObjBuffer);
                }

                if (MsgID == RNDZ_CRSP_SpRNDZ_ACCEPT)
                {
#if defined(TRACE_SERVER_ON)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                        "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                        "%s: \nReceived Response CSS_SpRNDZ_ACCEPT\n", pSourceID);
#endif

#if defined(ENCRYPTION_ON)
                    /* If we are supporting encryption then send a request to get the encryption key */
                    if (pConnArgs->EncryptionType)
                    {
#if defined(TRACE_SERVER_ON)
                        CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                            "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                            "%s: \nSending Request RNDZ_CRST_SpGET_ENCRYPT_KEY\n", pSourceID);
#endif
                        Result = SEND_RNDZ_REQUEST(pSourceID, RNDZ_CRST_SpGET_ENCRYPT_KEY);
                    }
                    else 
#endif /* #if defined(ENCRYPTION_ON) */
                    {
#if defined(TRACE_SERVER_ON)
                        CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
                            "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                            "%s: \nSending Command RNDZ_CCMD_SpSTART_HOST_PROC\n", pSourceID);
#endif

                        /* Send the START HOST PROC command */
                        Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpSTART_HOST_PROC, pObjBuffer, ObjSize);
                    }
                }/* if (MsgID == RNDZ_CRSP_SpRNDZ_ACCEPT) */

#if defined(ENCRYPTION_ON)
                /* RNDZ_CRSP_SpSEND_ENCRYPT_KEY is a response from the server that contains the
                 * session encryption key. 
                 */
                else if (MsgID == RNDZ_CRSP_SpSEND_ENCRYPT_KEY)
                {
                    pConnArgs = CLinkTable_GetEntry(&LinkTable, pSourceID);
                    if (pConnArgs)
                    {
#if defined(TRACE_SERVER_ON)
                        CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                            "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                            "RxResponse: Received Response RNDZ_CRSP_SpSEND_ENCRYPT_KEY\n");
#endif 
                        /* Store the session key */           
                        pConnArgs->pKey = (uint8*)SYS_MALLOC(AES_ENCRYPTION_KEY_SIZE);

                        if ((pConnArgs->pKey != NULL) && (pBuf))
                        {
                            /* Store this session key */
                            memcpy(pConnArgs->pKey, pBuf, AES_ENCRYPTION_KEY_SIZE);            
                        }
                        
#if defined(TRACE_SERVER_ON)
                        CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
                            "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                            "%s: \nSending Command RNDZ_CCMD_SpSTART_HOST_PROC\n", pSourceID);
#endif
                        /* Start the host process */
                        Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpSTART_HOST_PROC, pObjBuffer, ObjSize);
                    }

                    break;
                }
#endif /* #if defined(ENCRYPTION_ON) */

                /* Free up the memory allocation */
                SYS_FREE(pObjBuffer);
                pObjBuffer = NULL;

                break;
            }
            case GnREQUEST:
            {
                if (MsgID == RNDZ_CRST_SpSTART_RNDZ)
                {
#if defined(TRACE_SERVER_ON)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                        "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                        "%s: \nReceived Request RNDZ_CRST_SpSTART_RNDZ\n", pSourceID);
#endif 
                    if(strstr((const char*)pBuf, "RNDZCONNECT"))
                    {   
                        if ((pConnArgs->bRNDZDisabled == TRUE) ||
							(pConnArgs->ValidationState == RNDZ_VALIDATION_NONE))
                        {
#if defined(TRACE_SERVER_ON)
                            CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                                "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                                "%s: \nRendezvous request validated\n", pSourceID);
#endif 
                            /* Update the ValidationState */
                            pConnArgs->ValidationState = RNDZ_VALIDATION_START_RNDZ_RECVD;

#if defined(TRACE_SERVER_ON)
                            CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
                                "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                                "%s: \nSending Command RNDZ_CCMD_SpSTART_HOST_PROC\n", pSourceID);
#endif
                            /* We received a start request, and the message header was correct.
                             * Send back a Pseudo-ACK and continue to process according to our
                             * local DDS validation. 
                             */
                            Result = SEND_RNDZ_RESPONSE(pSourceID, RNDZ_CRSP_SpRNDZ_ACCEPT, NULL, 0);
                        }
#if defined(TRACE_SERVER_ON)
                        else
                        {
                            CTraceSvr_tprintf(&TraceServer, TRACE_Type_ERROR,
                                "CRNDZApp", "RxMsg", FmRENDEZVOUS, Genus, MsgID, 
                                "%s: RNDZ connection already in progress\n", pSourceID);
                        }
#endif 
                    }
                }
                break;
            }
            default:
            {
                break;
            }
        };
    }/* if (pConnArgs)*/

    return (Result);
}

/*================================================================================
InitClient : Sends a command to initiate a Rendezvous session
================================================================================*/
boolean CRNDZApp_InitClient(const char* pSourceID)
{
    /* Start the RNDZ Connection */
    boolean Result=FALSE;
    
#if defined(TRACE_SERVER_ON)
    CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
        "CRNDZApp", "InitClient", FmRENDEZVOUS, GnREQUEST, RNDZ_CRST_SpSTART_RNDZ, 
        "%s: \nSending Request RNDZ_CRST_SpSTART_RNDZ", pSourceID);
#endif 

    /*Start by creating a reqeust*/
    Result = WACP_SendMsg(pSourceID, FmRENDEZVOUS, GnREQUEST, RNDZ_CRST_SpSTART_RNDZ, (uint8*)RNDZ_CONNECT_MSG, RNDZ_CONNECT_SIZE, FALSE);

    return Result;
}

/*================================================================================
Terminate : 
================================================================================*/
void CRNDZApp_Terminate(char* pSourceID, boolean bStatus)
{
    ConnArgs *pConnArgs = NULL;

    pConnArgs = CLinkTable_GetEntry(&LinkTable, pSourceID);
    if (pConnArgs)
    {
        if (!pConnArgs->bRNDZStatus)
		{
			/* Update the RNDZ Status */
			pConnArgs->bRNDZStatus = bStatus;

			/* Check for a valid handle since it's possible for the hardware
			* connection triggers like DTR to cause the host device to 
			* finish a RNDZ connection before a call to Connect is made 
			*/
			if (ExitSem.hHandle == 0)
			{
				/* Create the ExitEvent */
				CWACPSemaphore_Init(&ExitSem, TERMINATE_RNDZ_APP, 0, 1);
			}

			/* Send an event to the RNDZ App class to terminate */
			CWACPSemaphore_Raise(&ExitSem);
		}

        /* If this is a HOST then we need to notify the application the RNDZ status */
        if (pConnArgs->bSvrFlag == TRUE)
        {
            if (pConnArgs->bRNDZStatus)
                WACP_RecvMsg(pSourceID, FmCONNECTION, GnEVENT, CONN_CEVNT_SpCONNECTED, NULL);
        }
    }
}
/*================================================================================
AppStart
================================================================================*/
boolean CRNDZApp_AppStart(boolean SvrFlag, uint32 Timeout, const ConnArgs *pConnArgs)
{
    boolean Result = TRUE;
    if (!SvrFlag)
    {
        Result = CRNDZApp_InitClient(pConnArgs->SourceID);
    }

    /* Check for a successful connection for clients */
    if (Result == TRUE)
    {
        /* Check for a valid handle since it's possible for the hardware
         * connection triggers like DTR to cause the host device to 
         * finish a RNDZ connection before a call to Connect is made 
         */
        if (ExitSem.hHandle == 0)
        {
            /* Create the ExitEvent */
            CWACPSemaphore_Init(&ExitSem, TERMINATE_RNDZ_APP, 0, 1);
        }

        /* Wait for the ExitEvent to fire from Terminate() or until we time out.*/
        CWACPSemaphore_Wait(&ExitSem, Timeout);

        /* Clean up the Exit Event object */
        CWACPSemaphore_Destroy(&ExitSem);
    }

#if defined(TRACE_SERVER_ON)
    CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
            "CRNDZApp", "AppStart", 0, 0, 0, 
            "%s: \nRNDZ authentication %s", pConnArgs->SourceID, (pConnArgs->bRNDZStatus ? "Passed" : "Failed"));
#endif
    return(pConnArgs->bRNDZStatus);
}

