/**************************************************************************** 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CSerialCommSvr.h:                                                 //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#if !defined(CSERIALCOMMSVR_H)
    #define  CSERIALCOMMSVR_H

#include "Target.h"
#include "Common.h"
#include <stdio.h>
#include "CWACPMutex.h"

#define CONNECTION_TIMEOUT    250  /* milliseconds */

typedef struct SerialAddr
{
    /* User configurable */     
    uint32  BaudRate;           /* Rate of connection Ex. 9600, 115200, 230400, etc... */
    boolean bAutoDiscovery;     /* TRUE - Server/Host auto detects client via hardware trigger 
                                 * FALSE - Client must send initial RNDZ connect message.
                                 */ 

    /* Stack modified values */
    uint16  Port;
    char*   pSourceID;
}SerialAddr;


typedef struct CSerialCommSvr
{
    HANDLE          hPort;            /* Serial Port handle */
    HANDLE          hReadThread;      /* Handle to the read thread */
    OVERLAPPED      OSWrite, OSRead;
    SetRecvBufPtr   pSetRecvMsg;
    TerminatePtr    pTerminate;
    char            SourceID[ADDR_BUF_SIZE];
    uint16          Port;
    enum STACK_TYPE StackType;
    SerialAddr      Config;
    uint32          BaudRate;
    CWACPMutex      MutexLock;
}CSerialCommSvr;

void            CSerialCommSvr_Init(CSerialCommSvr* pObject, 
                                    boolean (*RecvMsg)(char* pSourceID, uint8* pBuf, uint32 MsgSize),
                                    void (*Terminate)(char* pSourceID, uint16 Port));
void            CSerialCommSvr_Destroy(CSerialCommSvr* pObject);
boolean         CSerialCommSvr_SetPort(CSerialCommSvr* pObject, uint16 NewPort);
void            CSerialCommSvr_SetConfig(CSerialCommSvr* pObject, const void *pAddr);
boolean         CSerialCommSvr_Connect(CSerialCommSvr* pObject, const uint8* pSourceID, int PortNum);
boolean         CSerialCommSvr_SendMessage(CSerialCommSvr* pObject, const uint8* pBuf, uint32 pBufSize);
uint32 WACPAPI  CSerialCommSvr_StartClient(void *pData);


#endif /* #if !defined(CSERIALCOMMSVR_H) */







