/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// DMOIBDefiner.h                                                    //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(DMOIBDEFINER_H)
    #define  DMOIBDEFINER_H

/*
     (.)_(.)
     { . . }
      \_*_/
    / .   . \
   {_}     {_}
     |     |
    {_}-_-{_}
*/

// This is defined as PROTOTYPE by default for .h files. In the 
// DMOIBDefinitions.cpp file, it redefines DMOIB_DECL as IMPLEMENT, then 
// includes the .h file with the definitions. This will process the definitions
// in implementation mode and build all arrays automatically. This allows the 
// developer to update definitions by only modifying the .h file, and never
// needing to touch the .cpp file.

#define DMOIB_DECL PROTOTYPE

// Default all of these to NO so unwanted elements are included. AppGen will
// generate definition files that enable each of these separately in those 
// files themselves.
#define INCLUDE_DICTIONARY      NO
#define INCLUDE_STRING_TABLE    NO
#define INCLUDE_LINK_TABLE      NO
/*
===============================================================================
    The following set of macros are the only ones that should appear in 
    the actual definition .h file. They are the ones that are used explicitly
    to define dictionaries/string tables/link tables. The other macros in this
    file are helpers for these main macros.
===============================================================================
    Macros and types for declaring DMOIB object definitions
===============================================================================
*/

#define DMOIB_DICTIONARY(DictionaryName) \
        COMBINE3(DMOIB_DECL, _DMOIB_DICTIONARY_, INCLUDE_DICTIONARY)(DictionaryName)
#define DMOIB_DICTIONARY_CONTINUED(DictionaryName) \
        COMBINE3(DMOIB_DECL, _DMOIB_DICTIONARY_CONTINUED_, INCLUDE_DICTIONARY)(DictionaryName)

#define DMOIB_OBJECT(Family, Genus, Species, Version, ClassName) \
        COMBINE(DMOIB_DECL, _DMOIB_OBJECT)(Family, Genus, Species, Version, ClassName)

#define DMOIB_AVAILABLE_SPECIES(ClassName) \
        COMBINE(DMOIB_DECL, _DMOIB_AVAILABLE_SPECIES)(ClassName)
#define DMOIB_SPECIES(Name, Value) \
        COMBINE(DMOIB_DECL, _DMOIB_SPECIES)(Name, Value)

#define DMOIB_MEMBERS(Members) \
        COMBINE(DMOIB_DECL, _DMOIB_MEMBERS)(Members)

#if defined(CRA383_SUPPORT)
#define DMOIB_MEMBER(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        COMBINE(DMOIB_DECL, _DMOIB_MEMBER)(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale)
#else
#define DMOIB_MEMBER(Type, MemberName, MinVersion, MaxVersion, Alias) \
        COMBINE(DMOIB_DECL, _DMOIB_MEMBER)(Type, MemberName, MinVersion, MaxVersion, Alias)
#endif

#define DMOIB_ENUMERATION(Name) \
        COMBINE(DMOIB_DECL, _DMOIB_ENUMERATION)(Name)
#define DMOIB_ENUMERATION_LINKED(MemberName) \
        COMBINE(DMOIB_DECL, _DMOIB_ENUMERATION_LINKED)(MemberName)
#define DMOIB_ENUMERATOR(Name, Value) \
        COMBINE(DMOIB_DECL, _DMOIB_ENUMERATOR)(Name, Value)

/*
===============================================================================
    Macros and types for declaring a String Table
===============================================================================
*/

#define DMOIB_STRING_TABLE(StringTableName) \
        COMBINE3(DMOIB_DECL, _DMOIB_STRING_TABLE_, INCLUDE_STRING_TABLE)(StringTableName)
#define DMOIB_STRING_TABLE_CONTINUED(StringTableName) \
        COMBINE3(DMOIB_DECL, _DMOIB_STRING_TABLE_CONTINUED_, INCLUDE_STRING_TABLE)(StringTableName)

#define DMOIB_LANGUAGE_TABLE(Language) \
        COMBINE(DMOIB_DECL, _DMOIB_LANGUAGE_TABLE)(Language) 
#define DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
        COMBINE(DMOIB_DECL, _DMOIB_LANGUAGE_TABLE_CONTINUED)(Language) 

#define DMOIB_STRING(SymbolicLink, Text) \
        COMBINE(DMOIB_DECL, _DMOIB_STRING)(SymbolicLink, Text)
#define DMOIB_STRING_END() \
        COMBINE(DMOIB_DECL, _DMOIB_STRING_END)()

/*
===============================================================================
    Macros and types for declaring supported languages
===============================================================================
*/

#define DMOIB_SUPPORTED_LANGUAGES \
        COMBINE(DMOIB_DECL, _DMOIB_SUPPORTED_LANGUAGES)
#define DMOIB_LANGUAGE(LanguageID, Value) \
        COMBINE(DMOIB_DECL, _DMOIB_LANGUAGE)(LanguageID, Value)

/*
===============================================================================
    Macros and types for declaring a Link Table
===============================================================================
*/


#define DMOIB_LINK_TABLE(LinkTableName) \
        COMBINE3(DMOIB_DECL, _DMOIB_LINK_TABLE_, INCLUDE_LINK_TABLE)(LinkTableName) 
#define DMOIB_LINK_TABLE_CONTINUED(LinkTableName) \
        COMBINE3(DMOIB_DECL, _DMOIB_LINK_TABLE_CONTINUED_, INCLUDE_LINK_TABLE)(LinkTableName) 

#define DMOIB_ENTRY(SymbolicLink) \
        COMBINE(DMOIB_DECL, _DMOIB_ENTRY)(SymbolicLink)
#define DMOIB_LINK(ClassName, ElementID) \
        COMBINE(DMOIB_DECL, _DMOIB_LINK)(ClassName, ElementID)

#define DMOIB_LINK_CLASS(ClassName) \
        DMOIB_LINK(ClassName, ClassName)
#define DMOIB_LINK_SPECIES(ClassName, Species) \
        DMOIB_LINK(ClassName, Species)
#define DMOIB_LINK_MEMBER(ClassName, MemberID) \
        DMOIB_LINK(ClassName, MemberID)
#define DMOIB_LINK_ENUMERATION(ClassName, Enumeration) \
        DMOIB_LINK(ClassName, Enumeration)
#define DMOIB_LINK_ENUMERATOR(ClassName, Enumeration, Enumerator) \
        DMOIB_LINK(ClassName, Enumerator)

/*
===============================================================================
    Macros and types for declaring a String Table
===============================================================================
*/
#define DMOIB_DEFINITIONS_BEGIN() \
        COMBINE(DMOIB_DECL, _DMOIB_DEFINITIONS_BEGIN)()
#define DMOIB_DEFINITIONS_END() \
        COMBINE(DMOIB_DECL, _DMOIB_DEFINITIONS_END)()

/*
===============================================================================
    Macros for defining data types with arguments (such as static array sizes)
===============================================================================
*/
#define int8(Size)                      int8_ARRAY(Size)
#define int16(Size)                     int16_ARRAY(Size)
#define int32(Size)                     int32_ARRAY(Size)

#define uint8(Size)                     uint8_ARRAY(Size)
#define uint16(Size)                    uint16_ARRAY(Size)
#define uint32(Size)                    uint32_ARRAY(Size)

#define float32(Size)                   float32_ARRAY(Size)
#define float64(Size)                   float64_ARRAY(Size)

#define float(Size)                     float_ARRAY(Size)
#define double(Size)                    double_ARRAY(Size)

#define byte(Size)                      byte_ARRAY(Size)
#define word(Size)                      word_ARRAY(Size)
#define boolean(Size)                   boolean_ARRAY(Size)

#define ansichar(Size)                  ansichar_ARRAY(Size)
#define widechar(Size)                  widechar_ARRAY(Size)

#define MOIB_TIME(Size)                 MOIB_TIME_ARRAY(Size)

#define DMOIBObject(ClassName)          DMOIBObject_TYPE(ClassName)
#define DMOIBVector(ClassName, Size)    DMOIBVector_TYPE(ClassName, Size)

/*
===============================================================================
    END MAIN MACROS

    All macros after this point are helper macros that should not be used 
    directly.
===============================================================================
*/
/*
===============================================================================
    General Macros for helping create definitions
===============================================================================
*/
#define INCLUDE_YES(x)  x
#define INCLUDE_NO(x)   

// Macros for combining 2 terms into a single term
#define COMBINE(x, y)   COMBINE_(x, y)
#define COMBINE_(x, y)  x##y

#define COMBINE3(x, y, z)   COMBINE3_(x, y, z)
#define COMBINE3_(x, y, z)  x##y##z

#define PASS_THROUGH(x) x
#define ALIGN(x, y) ((((x) + ((y)-1))/(y))*(y))
#define ALIGN_TO_BOUNDARY(x) ALIGN(x, 4)

// Character replacement Macros for those extra tricky expansions
#define OPEN_P      (
#define CLOSE_P     )
#define COMMA       ,

// Macros for creating empty space
#define DO_NOTHING  // Intentionally Blank
#define OMIT(x)     // Intentionally Blank

#define DEFINITION_TEXT(Text) Text
#define DEFINITION_STRLEN strlen

#define UNAVAILABLE ""

#define MAKE_UNIQUE_NAME(Name) COMBINE(Name, __LINE__)

// Macros for extracting XML constant information. Can be turned on/off at the
// developer's discretion by DMOIB_INCLUDE_XML_IN_DEFINITIONS in target.h.
#if (defined(DMOIB_INCLUDE_XML_IN_DEFINITIONS) && \
            (DMOIB_INCLUDE_XML_IN_DEFINITIONS == 1))
#define XML_VALUE(x) x
#define XML_CLASSNAME(Family, Genus, Species, Version, ClassName) \
        DEFINITION_TEXT(#ClassName)
#else
#define XML_VALUE(x) 
#define XML_CLASSNAME(Family, Genus, Species, Version, ClassName) \
        DEFINITION_TEXT("")
#endif


// Macros for convertin uint32 and uint16 values into byte-form, defaulting
// to big-endian (always the case for definitions)
#define UINT32_TO_UINT8(Value)  (uint8)((((uint32)(Value)) >> 24) & 0x0FF), \
                                (uint8)((((uint32)(Value)) >> 16) & 0x0FF), \
                                (uint8)((((uint32)(Value)) >> 8)  & 0x0FF), \
                                (uint8)(((uint32)(Value)) & 0x0FF)
#define UINT16_TO_UINT8(Value)  (uint8)(((Value) >> 8)  & 0x0FF), \
                                (uint8)((Value) & 0x0FF)

// Macro for determining the printed string length of a constant integer value
#define STRLEN_OF_INT(Value)    (((Value) / 10) == 0 ? 1 : \
                                 ((Value) / 100) == 0 ? 2 : \
                                 ((Value) / 1000) == 0 ? 3 : \
                                 ((Value) / 10000) == 0 ? 4 : \
                                 ((Value) / 100000) == 0 ? 5 : \
                                 ((Value) / 1000000) == 0 ? 6 : \
                                 ((Value) / 10000000) == 0 ? 7 : \
                                 ((Value) / 100000000) == 0 ? 8 : \
                                 ((Value) / 1000000000) == 0 ? 9 : 10 ) + \
                                 ((Value < 0) ? 1 : 0)

#define BYTE_CODE_FOR(Type)      BYTE_CODE_FOR_##Type     +0
#define DEF_SIZEOF(Type)        (DEF_SIZE_OF_##Type       +0)
#define XML_SIZEOF(Type)        (XML_SIZE_OF_##Type       +0)


/*
===============================================================================
    Macros to check how the preprocessor expands #defines 
    During development, it was discovered that MSVC and GCC process macro
    expansions differently in some situations. Even though both claim to comply
    with C++ standards, the produce differing output. Therefore, these macros
    are defined to implicitly determine the "style" of the preprocessor at
    compile time. Evaluation of the PP_STYLE macro will result in a value of
    1 (MSVC style) or 2 (GCC style). This can be checked at compile time with 
    #if (PP_STYLE == PP_STYLE_MSVC), and the appropriate measures taken.
===============================================================================
*/
#define PP_TEST(x)  PP_BLANK(X x)
#define PP_BLANK(x) 1
#define PP_STEP(x)  CLOSE_P COMBINE(PP_STEP,2)(x)
#define PP_STEP2(x) PP_STEP3
#define PP_STEP3(x) + PP_STEP4 OPEN_P
#define PP_STEP4()  1

#define PP_STYLE PP_TEST( PP_STEP(X)(X) )

#define PP_STYLE_GCC    2
#define PP_STYLE_MSVC   1


/*
===============================================================================
    Macros for defining the bit-fields for all known DMOIB data types. Each
    data type will be evaluated 4 times: 
        1.) To create a byte-code representation in the definition
        2.) To determine the space the encoding takes up in the definition
        3.) To determine the static memory space the type would occupy in an
            instantiated object
        4.) To determine the space the type text would take up in an XML string
===============================================================================
*/
/*
===============================================================================
    Macros for pointer data types
===============================================================================
*/
#define BYTE_CODE_FOR_int8              DMOIB_TYPE_INT8_PTR     ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_int16             DMOIB_TYPE_INT16_PTR    ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_int32             DMOIB_TYPE_INT32_PTR    ^ DMOIB_TYPE_FLAG_PTR

#define BYTE_CODE_FOR_uint8             DMOIB_TYPE_UINT8_PTR    ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_uint16            DMOIB_TYPE_UINT16_PTR   ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_uint32            DMOIB_TYPE_UINT32_PTR   ^ DMOIB_TYPE_FLAG_PTR

#define BYTE_CODE_FOR_float32           DMOIB_TYPE_FLOAT32_PTR  ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_float64           DMOIB_TYPE_FLOAT64_PTR  ^ DMOIB_TYPE_FLAG_PTR

#define BYTE_CODE_FOR_float             DMOIB_TYPE_FLOAT32_PTR  ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_double            DMOIB_TYPE_FLOAT64_PTR  ^ DMOIB_TYPE_FLAG_PTR

#define BYTE_CODE_FOR_byte              DMOIB_TYPE_BYTE_PTR     ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_word              DMOIB_TYPE_WORD_PTR     ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_boolean           DMOIB_TYPE_BOOLEAN_PTR  ^ DMOIB_TYPE_FLAG_PTR

#define BYTE_CODE_FOR_ansichar          DMOIB_TYPE_ANSICHAR_PTR ^ DMOIB_TYPE_FLAG_PTR
#define BYTE_CODE_FOR_widechar          DMOIB_TYPE_WIDECHAR_PTR ^ DMOIB_TYPE_FLAG_PTR

#define BYTE_CODE_FOR_MOIB_TIME         DMOIB_TYPE_TIME_PTR     ^ DMOIB_TYPE_FLAG_PTR
/*
===============================================================================
    Macros for Static arrays of data types
===============================================================================
*/

#define ADD_COMMA(x,y)      ,x ,y
#define DONT_ADD_COMMA(x,y) 
#define OMIT3(x,y,z)  
#define PASS3(x, y, z)      x##_COMMA(y,z)

#define UNBOUNDED0       0 | DMOIB_TYPE_FLAG_PTR DONT_ADD, +0, +0) OMIT3 ( 0
#define UNBOUNDED1       DONT_ADD, +0, +0) OMIT3 ( 0

#define ARRAY_EXPAND(Size) \
        COMBINE(PASS, 3(/**/Size##0/**/ * 0 | DMOIB_TYPE_FLAG_ARR ADD, \
        (((uint16)/**/(Size)/**/) >> 8) & 0xFF , \
        ( (uint16)/**/(Size)/**/)       & 0xFF + 0 *) )

#define BYTE_CODE_FOR_int8_ARRAY(Size)        DMOIB_TYPE_INT8     + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_int16_ARRAY(Size)       DMOIB_TYPE_INT16    + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_int32_ARRAY(Size)       DMOIB_TYPE_INT32    + ARRAY_EXPAND(Size)
                          
#define BYTE_CODE_FOR_uint8_ARRAY(Size)       DMOIB_TYPE_UINT8    + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_uint16_ARRAY(Size)      DMOIB_TYPE_UINT16   + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_uint32_ARRAY(Size)      DMOIB_TYPE_UINT32   + ARRAY_EXPAND(Size)
                          
#define BYTE_CODE_FOR_float32_ARRAY(Size)     DMOIB_TYPE_FLOAT32  + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_float64_ARRAY(Size)     DMOIB_TYPE_FLOAT64  + ARRAY_EXPAND(Size)

#define BYTE_CODE_FOR_float_ARRAY(Size)       DMOIB_TYPE_FLOAT32  + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_double_ARRAY(Size)      DMOIB_TYPE_FLOAT64  + ARRAY_EXPAND(Size)
                          
#define BYTE_CODE_FOR_byte_ARRAY(Size)        DMOIB_TYPE_BYTE     + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_word_ARRAY(Size)        DMOIB_TYPE_WORD     + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_boolean_ARRAY(Size)     DMOIB_TYPE_BOOLEAN  + ARRAY_EXPAND(Size)
                          
#define BYTE_CODE_FOR_ansichar_ARRAY(Size)    DMOIB_TYPE_ANSICHAR + ARRAY_EXPAND(Size)
#define BYTE_CODE_FOR_widechar_ARRAY(Size)    DMOIB_TYPE_WIDECHAR + ARRAY_EXPAND(Size)

#define BYTE_CODE_FOR_MOIB_TIME_ARRAY(Size)   DMOIB_TYPE_TIME     + ARRAY_EXPAND(Size)

/*
===============================================================================
    Macros for Aggregate object types
===============================================================================
*/
#define BYTE_CODE_FOR_DMOIBObject_TYPE(ClassName) \
        DMOIB_TYPE_OBJECT, UINT32_TO_UINT8(ClassName::CLASSID) + 0 * 

#define BYTE_CODE_FOR_DMOIBVector_TYPE(ClassName, Size) \
        (Size ? DMOIB_TYPE_VECTOR_ARR : DMOIB_TYPE_VECTOR_PTR), \
        UINT32_TO_UINT8(ClassName::CLASSID) + \
        COMBINE(PASS, 3(/**/Size##1/**/ * 0 ADD, \
        (((uint16)/**/(Size)/**/) >> 8) & 0xFF , \
        ( (uint16)/**/(Size)/**/)       & 0xFF + 0 *) )
/*
===============================================================================
    Extra Macros for Static DMOIB Implementation
===============================================================================
*/
/*
===============================================================================
    Macros for single data types
===============================================================================
*/
#define DEF_SIZE_OF_int8        0
#define DEF_SIZE_OF_int16       0
#define DEF_SIZE_OF_int32       0

#define DEF_SIZE_OF_uint8       0
#define DEF_SIZE_OF_uint16      0
#define DEF_SIZE_OF_uint32      0

#define DEF_SIZE_OF_float32     0
#define DEF_SIZE_OF_float64     0

#define DEF_SIZE_OF_float       0
#define DEF_SIZE_OF_double      0

#define DEF_SIZE_OF_byte        0
#define DEF_SIZE_OF_word        0
#define DEF_SIZE_OF_boolean     0

#define DEF_SIZE_OF_ansichar    0
#define DEF_SIZE_OF_widechar    0

#define DEF_SIZE_OF_MOIB_TIME   0

/*
===============================================================================
    Macros for Static arrays of data types
===============================================================================
*/
#define DEF_SIZE_OF_int8_ARRAY(Size)        (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_int16_ARRAY(Size)       (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_int32_ARRAY(Size)       (Size ? SIZEOF_ARRAY_SIZE : 0)
                          
#define DEF_SIZE_OF_uint8_ARRAY(Size)       (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_uint16_ARRAY(Size)      (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_uint32_ARRAY(Size)      (Size ? SIZEOF_ARRAY_SIZE : 0)
                          
#define DEF_SIZE_OF_float32_ARRAY(Size)     (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_float64_ARRAY(Size)     (Size ? SIZEOF_ARRAY_SIZE : 0)

#define DEF_SIZE_OF_float_ARRAY(Size)       (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_double_ARRAY(Size)      (Size ? SIZEOF_ARRAY_SIZE : 0)
                          
#define DEF_SIZE_OF_byte_ARRAY(Size)        (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_word_ARRAY(Size)        (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_boolean_ARRAY(Size)     (Size ? SIZEOF_ARRAY_SIZE : 0)
                          
#define DEF_SIZE_OF_ansichar_ARRAY(Size)    (Size ? SIZEOF_ARRAY_SIZE : 0)
#define DEF_SIZE_OF_widechar_ARRAY(Size)    (Size ? SIZEOF_ARRAY_SIZE : 0)

#define DEF_SIZE_OF_MOIB_TIME_ARRAY(Size)   (Size ? SIZEOF_ARRAY_SIZE : 0)

/*
===============================================================================
    Macros for Aggregate object types
===============================================================================
*/
#define DEF_SIZE_OF_DMOIBObject_TYPE(ClassName)         SIZEOF_CLASSID
#define DEF_SIZE_OF_DMOIBVector_TYPE(ClassName, Size)   ((Size ? SIZEOF_ARRAY_SIZE : 0) \
                                                               + SIZEOF_CLASSID)

/*
===============================================================================
    Macros for pointer data types
===============================================================================
*/
#define XML_SIZE_OF_int8        (sizeof("int8") - 1)
#define XML_SIZE_OF_int16       (sizeof("int16") - 1)
#define XML_SIZE_OF_int32       (sizeof("int32") - 1)
                           
#define XML_SIZE_OF_uint8       (sizeof("uint8") - 1)
#define XML_SIZE_OF_uint16      (sizeof("uint16") - 1)
#define XML_SIZE_OF_uint32      (sizeof("uint32") - 1)
                           
#define XML_SIZE_OF_float32     (sizeof("float32") - 1)
#define XML_SIZE_OF_float64     (sizeof("float64") - 1)

#define XML_SIZE_OF_float       (sizeof("float32") - 1)
#define XML_SIZE_OF_double      (sizeof("float64") - 1)
                           
#define XML_SIZE_OF_byte        (sizeof("uint8") - 1)
#define XML_SIZE_OF_word        (sizeof("uint16") - 1)
#define XML_SIZE_OF_boolean     (sizeof("uint8") - 1)
                           
#define XML_SIZE_OF_ansichar    (sizeof("ansichar") - 1)
#define XML_SIZE_OF_widechar    (sizeof("widechar") - 1)

#define XML_SIZE_OF_MOIB_TIME   (sizeof("TIME") - 1)
                           
/*
===============================================================================
    Macros for Static arrays of data types
===============================================================================
*/
#define XML_SIZE_OF_ARRAY(Size) \
    + sizeof(" " XML_ATT_SIZE "\"\"") -1 \
    + (Size == UNBOUNDED ? sizeof(XML_VAL_UNBOUNDED) -1 : STRLEN_OF_INT(Size)) 

#define XML_SIZE_OF_int8_ARRAY(Size)        XML_SIZE_OF_int8      + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_int16_ARRAY(Size)       XML_SIZE_OF_int16     + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_int32_ARRAY(Size)       XML_SIZE_OF_int32     + XML_SIZE_OF_ARRAY(Size)
                                                        
#define XML_SIZE_OF_uint8_ARRAY(Size)       XML_SIZE_OF_uint8     + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_uint16_ARRAY(Size)      XML_SIZE_OF_uint16    + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_uint32_ARRAY(Size)      XML_SIZE_OF_uint32    + XML_SIZE_OF_ARRAY(Size)
                                                        
#define XML_SIZE_OF_float32_ARRAY(Size)     XML_SIZE_OF_float32   + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_float64_ARRAY(Size)     XML_SIZE_OF_float64   + XML_SIZE_OF_ARRAY(Size)

#define XML_SIZE_OF_float_ARRAY(Size)       XML_SIZE_OF_float     + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_double_ARRAY(Size)      XML_SIZE_OF_double    + XML_SIZE_OF_ARRAY(Size)
                                                        
#define XML_SIZE_OF_byte_ARRAY(Size)        XML_SIZE_OF_byte      + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_word_ARRAY(Size)        XML_SIZE_OF_word      + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_boolean_ARRAY(Size)     XML_SIZE_OF_boolean   + XML_SIZE_OF_ARRAY(Size)
                                                        
#define XML_SIZE_OF_ansichar_ARRAY(Size)    XML_SIZE_OF_ansichar  + XML_SIZE_OF_ARRAY(Size)
#define XML_SIZE_OF_widechar_ARRAY(Size)    XML_SIZE_OF_widechar  + XML_SIZE_OF_ARRAY(Size)

#define XML_SIZE_OF_MOIB_TIME_ARRAY(Size)   XML_SIZE_OF_MOIB_TIME  + XML_SIZE_OF_ARRAY(Size)

/*
===============================================================================
    Macros for Aggregate object types
===============================================================================
*/
#define XML_SIZE_OF_DMOIBObject_TYPE(ClassName) \
        (sizeof(XML_VAL_OBJECT" " XML_ATT_CLASS "\"\"" #ClassName) - 1) 

#define XML_SIZE_OF_DMOIBVector_TYPE(ClassName, Size) \
        XML_SIZE_OF_DMOIBObject_TYPE(ClassName) + XML_SIZE_OF_ARRAY(Size)


/*
===============================================================================
    The meat and potatoes of the macros...the rest of the macros in this file
    are multi-pass macros that will evaluate the definitions available in the
    DMOIBDefintions.h file. There are 2 main passes, and multiple sub-passes 
    within those 2. The first main pass is a PROTOTYPE pass, which declares all 
    of the constants and enumerations that will be used by external developers.
    The second main pass is the IMPLEMENT pass, which creates all of the byte
    arrays that were declared in the PROTOTYPE pass, and fills them with const
    data. 
    
    The IMPLEMENT pass is only evaluated in the DMOIBDefinitions.cpp file
    to prevent conflicting declarations of global variables.

===============================================================================
    Macros for DMOIB Prototypes
===============================================================================
*/
#define PROTOTYPE_DMOIB_OBJECT(Family, Genus, Species, Version, ClassName) \
       (PROTOTYPE_DMOIB_OBJECT)(Family, Genus, Species, Version, ClassName)

#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES(ClassName) \
       (PROTOTYPE_DMOIB_AVAILABLE_SPECIES)(ClassName)

#define PROTOTYPE_DMOIB_SPECIES(Name, Value) \
       (PROTOTYPE_DMOIB_SPECIES)(Name, Value)

#define PROTOTYPE_DMOIB_MEMBERS(Members) \
       (PROTOTYPE_DMOIB_MEMBERS)(Members)

#if defined(CRA383_SUPPORT)
#define PROTOTYPE_DMOIB_MEMBER(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
       (PROTOTYPE_DMOIB_MEMBER)(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale)
#else
#define PROTOTYPE_DMOIB_MEMBER(Type, MemberName, MinVersion, MaxVersion, Alias) \
       (PROTOTYPE_DMOIB_MEMBER)(Type, MemberName, MinVersion, MaxVersion, Alias)
#endif

#define PROTOTYPE_DMOIB_ENUMERATION_LINKED(MemberName) \
       (PROTOTYPE_DMOIB_ENUMERATION_LINKED_INTERNAL)(MemberName)            

#define PROTOTYPE_DMOIB_ENUMERATION(Name) \
       (PROTOTYPE_DMOIB_ENUMERATION)(Name) 

#define PROTOTYPE_DMOIB_ENUMERATOR(Name, Value) \
       (PROTOTYPE_DMOIB_ENUMERATOR_INTERNAL)(Name, Value)

#define PROTOTYPE_DMOIB_ENTRY(SymbolicLink) \
       (PROTOTYPE_DMOIB_ENTRY)(SymbolicLink) 

#define PROTOTYPE_DMOIB_LINK(ClassName, ElementID) \
       (PROTOTYPE_DMOIB_LINK)(ClassName, ElementID) 

#define PROTOTYPE_DMOIB_DEFINITIONS_BEGIN() \
        namespace { namespace
#define PROTOTYPE_DMOIB_DEFINITIONS_END() \
        {};}

//===============================================
// Dictionary base
//-----------------------------------------------------------------------------
// Do nothing if INCLUDE_DICTIONARY is set to NO
#define PROTOTYPE_DMOIB_DICTIONARY_NO(DictionaryName) \
        PROTOTYPE_DMOIB_DICTIONARY_NO_
#define PROTOTYPE_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        PROTOTYPE_DMOIB_DICTIONARY_NO_
#define PROTOTYPE_DMOIB_DICTIONARY_NO_(Definitions) 

#define PROTOTYPE_DMOIB_DICTIONARY_YES(DictionaryName) \
        PROTOTYPE_DMOIB_DEFINITIONS_END() \
        void DMOIBInitialize(void); \
        namespace DictionaryName \
        { \
            void* Dictionary(void); \
        } \
        using namespace DictionaryName; \
        namespace DictionaryName \
        PROTOTYPE_DMOIB_DICTIONARY_YES_
#define PROTOTYPE_DMOIB_DICTIONARY_YES_(DMOIBObjects) \
        { \
            DECLARE_SPECIES     DMOIBObjects (END) \
            DECLARE_CONSTANTS   DMOIBObjects (END) \
        } \
        PROTOTYPE_DMOIB_DEFINITIONS_BEGIN()

#define PROTOTYPE_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        PROTOTYPE_DMOIB_DEFINITIONS_END() \
        namespace DictionaryName \
        PROTOTYPE_DMOIB_DICTIONARY_CONTINUED_YES_
#define PROTOTYPE_DMOIB_DICTIONARY_CONTINUED_YES_(DMOIBObjects) \
        { \
            DECLARE_SPECIES     DMOIBObjects (END) \
            DECLARE_CONSTANTS   DMOIBObjects (END) \
        } \
        PROTOTYPE_DMOIB_DEFINITIONS_BEGIN()

//===============================================
#define DECLARE_SPECIES(x)          x##_DECLARE_SPECIES
#define END_DECLARE_SPECIES         DO_NOTHING

#define DECLARE_CONSTANTS(x)        x##_DECLARE_CONSTANTS
#define END_DECLARE_CONSTANTS       DO_NOTHING

//===============================================
// DECLARE_SPECIES
//-----------------------------------------------------------------------------
#define PROTOTYPE_DMOIB_OBJECT_DECLARE_SPECIES(Family, Genus, Species, Version, ClassName) \
        namespace ClassName \
        { \
        PROTOTYPE_DMOIB_OBJECT_DECLARE_SPECIES_
#define PROTOTYPE_DMOIB_OBJECT_DECLARE_SPECIES_(Internals) \
            enum Species_e \
            { \
                DECLARE_SPECIES Internals (END) \
            }; \
        } \
        DECLARE_SPECIES

#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_SPECIES(ClassName) \
        PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_SPECIES_
#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_SPECIES_(Species) \
        DECLARE_SPECIES Species (END) \
        DECLARE_SPECIES

#define PROTOTYPE_DMOIB_SPECIES_DECLARE_SPECIES(Name, Value) \
        Name##EntryValue, \
        Name = ((1##Value + 1) != 2 ? Value + 0 : Name##EntryValue ), \
        DECLARE_SPECIES

#define PROTOTYPE_DMOIB_MEMBERS_DECLARE_SPECIES(Members) \
        DECLARE_SPECIES

#define PROTOTYPE_DMOIB_ENUMERATION_LINKED_INTERNAL_DECLARE_SPECIES(MemberName) \
        PROTOTYPE_DMOIB_ENUMERATION_DECLARE_SPECIES_
#define PROTOTYPE_DMOIB_ENUMERATION_DECLARE_SPECIES(Name) \
        PROTOTYPE_DMOIB_ENUMERATION_DECLARE_SPECIES_
#define PROTOTYPE_DMOIB_ENUMERATION_DECLARE_SPECIES_(Enumerators) \
        DECLARE_SPECIES

//===============================================
// DECLARE_CONSTANTS
//-----------------------------------------------------------------------------
#define PROTOTYPE_DMOIB_OBJECT_DECLARE_CONSTANTS(Family, Genus, Species, Version, ClassName) \
        using namespace ClassName; \
        namespace ClassName \
        { \
            void* Definition(void); \
            enum ClassInfo_e \
            { \
                FAMILY = Family, \
                GENUS = Genus, \
                SPECIES = Species, \
                CLASSID = (((Family << 16) | (Genus << 8) | Species)), \
                VERSION = Version, \
            }; \
        PROTOTYPE_DMOIB_OBJECT_DECLARE_CONSTANTS_
#define PROTOTYPE_DMOIB_OBJECT_DECLARE_CONSTANTS_(Internals) \
            enum ClassConstants_e \
            { \
                /* This will expand to one set of DMOIB_MEMBERS and n sets of DMOIB_ENUMERATIONS */ \
                DECLARE_CONSTANTS Internals (END) \
            }; \
            enum EnumerationInit_e \
            { \
                DECLARE_ENUMERATION_NAMESPACES Internals (END) \
                ENUMERATION_END \
            }; \
        } \
        DECLARE_CONSTANTS

//===============================================
#define DECLARE_DMOIB_MEMBER_NAME(x)          x##_DECLARE_DMOIB_MEMBER_NAME
#define END_DECLARE_DMOIB_MEMBER_NAME         DO_NOTHING

#define DECLARE_DMOIB_MEMBER_OFFSET(x)        x##_DECLARE_DMOIB_MEMBER_OFFSET
#define END_DECLARE_DMOIB_MEMBER_OFFSET       DO_NOTHING

#define DECLARE_ENUMERATION_NAMESPACES(x)     x##_DECLARE_ENUMERATION_NAMESPACES
#define END_DECLARE_ENUMERATION_NAMESPACES    DO_NOTHING

//===============================================
#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_CONSTANTS(ClassName) \
        ClassName##SymbolicLinkEntryValue, \
        ClassName##SymbolicLink = 0xFFFF, \
        ClassName = 0xFFFF, \
        ClassName##SymbolicLinkExitValue = ClassName##SymbolicLinkEntryValue - 1, \
        PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_CONSTANTS_
#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_CONSTANTS_(Species) \
        DECLARE_CONSTANTS Species (END) \
        DECLARE_CONSTANTS

#define PROTOTYPE_DMOIB_SPECIES_DECLARE_CONSTANTS(Name, Value) \
        Name##SymbolicLinkEntryValue, \
        Name##SymbolicLink = 0xFFFF, \
        Name##SymbolicLinkExitValue = Name##SymbolicLinkEntryValue - 1, \
        DECLARE_CONSTANTS

#define PROTOTYPE_DMOIB_MEMBERS_DECLARE_CONSTANTS(Members) \
        DECLARE_DMOIB_MEMBER_NAME Members (END) \
        DMOIB_MEMBER_COUNT, \
        MEMORY_SIZE_INIT = 0, \
        DECLARE_DMOIB_MEMBER_OFFSET Members (END) \
        ENUM_INIT = 0xFFFF - 2, \
        DECLARE_CONSTANTS
//^^
//===============================================

#if defined(CRA383_SUPPORT)
#define PROTOTYPE_DMOIB_MEMBER_DECLARE_DMOIB_MEMBER_NAME(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale)  \
        MemberName, \
        DECLARE_DMOIB_MEMBER_NAME 

#define PROTOTYPE_DMOIB_MEMBER_DECLARE_DMOIB_MEMBER_OFFSET(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale)  \
        MemberName##EnumOffsetEntryValue, \
        MemberName##EnumOffset = 0xFFFF, \
        MemberName##SymbolicLink = 0xFFFF, \
        MemberName##EnumOffsetExitValue = MemberName##EnumOffsetEntryValue - 1, \
        DECLARE_DMOIB_MEMBER_OFFSET 
#else
#define PROTOTYPE_DMOIB_MEMBER_DECLARE_DMOIB_MEMBER_NAME(Type, MemberName, MinVersion, MaxVersion, Alias)  \
        MemberName, \
        DECLARE_DMOIB_MEMBER_NAME 

#define PROTOTYPE_DMOIB_MEMBER_DECLARE_DMOIB_MEMBER_OFFSET(Type, MemberName, MinVersion, MaxVersion, Alias)  \
        MemberName##EnumOffsetEntryValue, \
        MemberName##EnumOffset = 0xFFFF, \
        MemberName##SymbolicLink = 0xFFFF, \
        MemberName##EnumOffsetExitValue = MemberName##EnumOffsetEntryValue - 1, \
        DECLARE_DMOIB_MEMBER_OFFSET 
#endif

//===============================================
#define PROTOTYPE_DMOIB_ENUMERATION_LINKED_INTERNAL_DECLARE_CONSTANTS(MemberName) \
        MemberName##_DMOIB_ENUMERATION_IN, \
        MemberName##_DMOIB_ENUMERATION_OUT = (MemberName##_DMOIB_ENUMERATION_IN & 0xFFFF) - 1, \
        PROTOTYPE_DMOIB_ENUMERATION_DECLARE_CONSTANTS_
#define PROTOTYPE_DMOIB_ENUMERATION_DECLARE_CONSTANTS(Name) \
        Name##EntryValue, \
        Name##SymbolicLink = 0xFFFF, \
        Name = (Name##EntryValue & 0xFFFF), \
        Name##ExitValue = Name - 2, \
        PROTOTYPE_DMOIB_ENUMERATION_DECLARE_CONSTANTS_
#define PROTOTYPE_DMOIB_ENUMERATION_DECLARE_CONSTANTS_(Enumerators) \
        DECLARE_CONSTANTS Enumerators (END) \
        DECLARE_CONSTANTS

#define PROTOTYPE_DMOIB_ENUMERATOR_INTERNAL_DECLARE_CONSTANTS(Name, Value) \
        Name##EntryValue, \
        Name##SymbolicLink = 0xFFFF, \
        Name = ((1##Value + 1) != 2 ? Value + 0 : (Name##EntryValue >> 16)), \
        Name##ExitValue = (Name##EntryValue & 0xFFFF) + ((Name + 1) << 16) - 1, \
        DECLARE_CONSTANTS

//===============================================
#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_ENUMERATION_NAMESPACES(ClassName) \
        PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_ENUMERATION_NAMESPACES_
#define PROTOTYPE_DMOIB_AVAILABLE_SPECIES_DECLARE_ENUMERATION_NAMESPACES_(Species) \
        DECLARE_ENUMERATION_NAMESPACES Species (END) \
        DECLARE_ENUMERATION_NAMESPACES
#define PROTOTYPE_DMOIB_SPECIES_DECLARE_ENUMERATION_NAMESPACES(Name, Value) \
        DECLARE_ENUMERATION_NAMESPACES

#define PROTOTYPE_DMOIB_MEMBERS_DECLARE_ENUMERATION_NAMESPACES(Members) \
        DECLARE_ENUMERATION_NAMESPACES Members (END) \
        DECLARE_ENUMERATION_NAMESPACES

#if defined(CRA383_SUPPORT)
#define PROTOTYPE_DMOIB_MEMBER_DECLARE_ENUMERATION_NAMESPACES(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale)  \
        DECLARE_ENUMERATION_NAMESPACES 
#else
#define PROTOTYPE_DMOIB_MEMBER_DECLARE_ENUMERATION_NAMESPACES(Type, MemberName, MinVersion, MaxVersion, Alias)  \
        DECLARE_ENUMERATION_NAMESPACES 
#endif

#define PROTOTYPE_DMOIB_ENUMERATION_LINKED_INTERNAL_DECLARE_ENUMERATION_NAMESPACES(MemberName) \
        PROTOTYPE_DMOIB_ENUMERATION_DECLARE_ENUMERATION_NAMESPACES_
#define PROTOTYPE_DMOIB_ENUMERATION_DECLARE_ENUMERATION_NAMESPACES(Name) \
        PROTOTYPE_DMOIB_ENUMERATION_DECLARE_ENUMERATION_NAMESPACES_
#define PROTOTYPE_DMOIB_ENUMERATION_DECLARE_ENUMERATION_NAMESPACES_(Enumerators) \
        DECLARE_ENUMERATION_NAMESPACES


//===============================================
// StringTable base
//-----------------------------------------------------------------------------
#define PROTOTYPE_DMOIB_STRING_TABLE_NO(StringTableName) \
        PROTOTYPE_DMOIB_STRING_TABLE_NO_
#define PROTOTYPE_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        PROTOTYPE_DMOIB_STRING_TABLE_NO_
#define PROTOTYPE_DMOIB_STRING_TABLE_NO_(Entries) 

#define PROTOTYPE_DMOIB_STRING_TABLE_YES(StringTableName) \
        PROTOTYPE_DMOIB_DEFINITIONS_END() \
        void DMOIBInitialize(void); \
        namespace StringTableName \
        { \
            void* StringTable(void); \
        } \
        using namespace StringTableName; \
        namespace StringTableName \
        { \
        PROTOTYPE_DMOIB_STRING_TABLE_YES_
#define PROTOTYPE_DMOIB_STRING_TABLE_YES_(Entries) \
            namespace DefaultLanguage {} \
            using namespace DefaultLanguage; \
            namespace DefaultLanguage \
            Entries

#define PROTOTYPE_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        PROTOTYPE_DMOIB_STRING_TABLE_CONTINUED_YES_
#define PROTOTYPE_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) \
            Entries

//===============================================
#define PROTOTYPE_DMOIB_LANGUAGE_TABLE(Language) \
        { \
            namespace Language##SymbolicLinks {}; \
            using namespace Language##SymbolicLinks; \
            namespace Language##SymbolicLinks \
            { \
                enum Language_e \
                { \
        PROTOTYPE_DMOIB_LANGUAGE_TABLE_
#define PROTOTYPE_DMOIB_LANGUAGE_TABLE_(Strings) \
                    Strings 

#define PROTOTYPE_DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
        PROTOTYPE_DMOIB_LANGUAGE_TABLE_CONTINUED_
#define PROTOTYPE_DMOIB_LANGUAGE_TABLE_CONTINUED_(Strings) \
                    Strings 

#define PROTOTYPE_DMOIB_STRING(SymbolicLink, Text) \
                    SymbolicLink, 

#define PROTOTYPE_DMOIB_STRING_END() \
                    LINK_COUNT \
                }; \
            }; \
        }; \
        namespace MAKE_UNIQUE_NAME(Unused)

//===============================================
// Supported Languages base
//-----------------------------------------------------------------------------
#define PROTOTYPE_DMOIB_SUPPORTED_LANGUAGES(Languages) \
        enum SupportedLanguages_e \
        { \
            Languages \
        };

#define PROTOTYPE_DMOIB_LANGUAGE(LanguageID, Value) \
        LanguageID = Value, 

//===============================================
// LinkTable base
//-----------------------------------------------------------------------------
#define PROTOTYPE_DMOIB_LINK_TABLE_NO(LinkTableName) \
        PROTOTYPE_DMOIB_LINK_TABLE_NO_
#define PROTOTYPE_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        PROTOTYPE_DMOIB_LINK_TABLE_NO_
#define PROTOTYPE_DMOIB_LINK_TABLE_NO_(Entries) 

#define PROTOTYPE_DMOIB_LINK_TABLE_YES(LinkTableName) \
        PROTOTYPE_DMOIB_DEFINITIONS_END() \
        void DMOIBInitialize(void); \
        namespace LinkTableName \
        { \
            void* LinkTable(void); \
        } \
        namespace LinkTableName \
        PROTOTYPE_DMOIB_LINK_TABLE_YES_

#define PROTOTYPE_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        PROTOTYPE_DMOIB_DEFINITIONS_END() \
        void DMOIBInitialize(void); \
        namespace LinkTableName \
        PROTOTYPE_DMOIB_LINK_TABLE_YES_

#define PROTOTYPE_DMOIB_LINK_TABLE_YES_(Entries) \
        { \
            OVERRIDE_OFFSETS    Entries (END) \
        } \
        PROTOTYPE_DMOIB_DEFINITIONS_BEGIN()

//===============================================
#define OVERRIDE_OFFSETS(x)     x##_OVERRIDE_OFFSETS
#define END_OVERRIDE_OFFSETS    DO_NOTHING

//===============================================
#define PROTOTYPE_DMOIB_ENTRY_OVERRIDE_OFFSETS(SymbolicLink) \
        enum SymbolicLink##Overrides_e \
        { \
            SymbolicLink##OverrideValue = SymbolicLink, \
        PROTOTYPE_DMOIB_ENTRY_OVERRIDE_OFFSETS_
#define PROTOTYPE_DMOIB_ENTRY_OVERRIDE_OFFSETS_(Links) \
            OVERRIDE_OFFSETS    Links (END) \
        }; \
        OVERRIDE_OFFSETS

#define PROTOTYPE_DMOIB_LINK_OVERRIDE_OFFSETS(ClassName, ElementID) \
            ClassName##ElementID##EntryValue \
        }; \
        namespace ClassName \
        { \
            namespace LookupConstants \
            { \
                enum ClassName##ElementID##Override_e \
                { \
                    ElementID##SymbolicLink = (ClassName::ElementID <= UINT16_MAX ? (ClassName##ElementID##EntryValue - 1) : 0xFFFF) \
                }; \
            } \
        } \
        enum \
        { \
            ClassName##ElementID##ExitValue = ClassName::LookupConstants::ElementID##SymbolicLink, \
        OVERRIDE_OFFSETS

/*
===============================================================================
    Macros for declaring initialization functions for static tables.
===============================================================================
*/
//===============================================
#define DECLARE_INITIALIZERS_DMOIB_DICTIONARY_NO(DictionaryName) \
        DECLARE_INITIALIZERS_DMOIB_DICTIONARY_NO_
#define DECLARE_INITIALIZERS_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        DECLARE_INITIALIZERS_DMOIB_DICTIONARY_NO_
#define DECLARE_INITIALIZERS_DMOIB_DICTIONARY_YES(DictionaryName) \
        DECLARE_INITIALIZERS_DMOIB_DICTIONARY_NO_
#define DECLARE_INITIALIZERS_DMOIB_DICTIONARY_NO_(Definitions) 

#define DECLARE_INITIALIZERS_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        namespace DictionaryName \
        { \
            void* MAKE_UNIQUE_NAME(Dictionary)(void* PreviousTable); \
        DECLARE_INITIALIZERS_DMOIB_DICTIONARY_CONTINUED_YES_
#define DECLARE_INITIALIZERS_DMOIB_DICTIONARY_CONTINUED_YES_(DMOIBObjects) \
            COMBINE(INCLUDE_, INCLUDE_LINK_TABLE) \
            ( \
                void* MAKE_UNIQUE_NAME(LinkTable)(void* PreviousTable); \
            ) \
        } 

#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_NO(StringTableName) \
        DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_NO_
#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_NO_
#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_NO_(Entries) 

#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_YES(StringTableName) \
        namespace StringTableName \
        { \
        DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_YES_
#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_YES_(Entries) \
            Entries \
        } 

#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        namespace StringTableName \
        { \
            void* MAKE_UNIQUE_NAME(StringTable)(void* PreviousTable); \
        DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_CONTINUED_YES_
#define DECLARE_INITIALIZERS_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) \
            Entries \
        } 

#define DECLARE_INITIALIZERS_DMOIB_LANGUAGE_TABLE(Language) \
        namespace LanguageConstants##Language \
        { \
            void* LanguageTable(void); \
        } \
        DECLARE_INITIALIZERS_DMOIB_LANGUAGE_TABLE_
#define DECLARE_INITIALIZERS_DMOIB_LANGUAGE_TABLE_(Strings) 

#define DECLARE_INITIALIZERS_DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
        DECLARE_INITIALIZERS_DMOIB_LANGUAGE_TABLE_CONTINUED_
#define DECLARE_INITIALIZERS_DMOIB_LANGUAGE_TABLE_CONTINUED_(Strings) 

#define DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_NO(LinkTableName) \
        DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_NO_
#define DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_NO_
#define DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_YES(LinkTableName) \
        DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_NO_
#define DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_NO_
#define DECLARE_INITIALIZERS_DMOIB_LINK_TABLE_NO_(Entries) 

#define DECLARE_INITIALIZERS_DMOIB_SUPPORTED_LANGUAGES(Languages) 

#define DECLARE_INITIALIZERS_DMOIB_DEFINITIONS_BEGIN() 
#define DECLARE_INITIALIZERS_DMOIB_DEFINITIONS_END() 


/*
===============================================================================
    Macros for declaring string constants like array sizes
===============================================================================
*/
//===============================================
#define DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_NO(DictionaryName) \
        DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_YES(DictionaryName) \
        DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_NO_(Definitions) 

#define DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_CONTINUED_YES_
#define DECLARE_STRING_CONSTANTS_DMOIB_DICTIONARY_CONTINUED_YES_(DMOIBObjects) 

#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_NO(StringTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_NO_(Entries) 

#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_YES(StringTableName) \
        } \
        namespace StringTableName \
        { \
        DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_YES_
#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_YES_(Entries) \
            Entries 

#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_CONTINUED_YES_
#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) \
            Entries 

#define DECLARE_STRING_CONSTANTS_DMOIB_LANGUAGE_TABLE(Language) \
            namespace Language##Constants \
            { \
                enum Language##_e \
                { \
        DECLARE_STRING_CONSTANTS_DMOIB_LANGUAGE_TABLE_
#define DECLARE_STRING_CONSTANTS_DMOIB_LANGUAGE_TABLE_(Strings) \
                    LookupTableSize = ALIGN_TO_BOUNDARY(SIZEOF_LANGUAGE_ID + \
                                                        SIZEOF_SYMBOLIC_LINK_COUNT + \
                                                        /*LINK_COUNT + 1 for the extra \
                                                          entry at the end that points to 
                                                          the end of the strings. */ \
                                                        ((LINK_COUNT + 1) * SIZEOF_SYMBOLIC_LINK)), \
                    LanguageTableInit = LookupTableSize - 1, \
                    Strings

#define DECLARE_STRING_CONSTANTS_DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
        DECLARE_STRING_CONSTANTS_DMOIB_LANGUAGE_TABLE_CONTINUED_
#define DECLARE_STRING_CONSTANTS_DMOIB_LANGUAGE_TABLE_CONTINUED_(Strings) \
                    Strings

#define DECLARE_STRING_CONSTANTS_DMOIB_STRING(SymbolicLink, Text) \
                    SymbolicLink##Offset, \
                    SymbolicLink##OffsetExitValue = SymbolicLink##Offset + sizeof(Text) - 1,

#define DECLARE_STRING_CONSTANTS_DMOIB_STRING_END() \
                    STRING_END \
                }; \
            }

#define DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_NO(LinkTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_YES(LinkTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_NO_
#define DECLARE_STRING_CONSTANTS_DMOIB_LINK_TABLE_NO_(Entries) 

#define DECLARE_STRING_CONSTANTS_DMOIB_SUPPORTED_LANGUAGES(Languages) 

#define DECLARE_STRING_CONSTANTS_DMOIB_DEFINITIONS_BEGIN() \
        namespace MAKE_UNIQUE_NAME(Unused) \
        { \

#define DECLARE_STRING_CONSTANTS_DMOIB_DEFINITIONS_END() \
        }


/*
===============================================================================
    Macros for building the actual string tables. 
===============================================================================
*/
//===============================================
#define BUILD_STRING_TABLES_DMOIB_DICTIONARY_NO(DictionaryName) \
        BUILD_STRING_TABLES_DMOIB_DICTIONARY_NO_
#define BUILD_STRING_TABLES_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        BUILD_STRING_TABLES_DMOIB_DICTIONARY_NO_
#define BUILD_STRING_TABLES_DMOIB_DICTIONARY_YES(DictionaryName) \
        BUILD_STRING_TABLES_DMOIB_DICTIONARY_NO_
#define BUILD_STRING_TABLES_DMOIB_DICTIONARY_NO_(Definitions) 

#define BUILD_STRING_TABLES_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        BUILD_STRING_TABLES_DMOIB_DICTIONARY_CONTINUED_YES_
#define BUILD_STRING_TABLES_DMOIB_DICTIONARY_CONTINUED_YES_(DMOIBObjects) 

#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_NO(StringTableName) \
        BUILD_STRING_TABLES_DMOIB_STRING_TABLE_NO_
#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        BUILD_STRING_TABLES_DMOIB_STRING_TABLE_NO_
#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_NO_(Entries) 

#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_YES(StringTableName) \
        } \
        namespace StringTableName \
        { \
        BUILD_STRING_TABLES_DMOIB_STRING_TABLE_YES_
#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_YES_(Entries) \
            Entries 

#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        BUILD_STRING_TABLES_DMOIB_STRING_TABLE_CONTINUED_YES_
#define BUILD_STRING_TABLES_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) \
            Entries 

#define BUILD_STRING_TABLES_DMOIB_LANGUAGE_TABLE(Language) \
            namespace Language##Constants \
            { \
                void* LanguageBase(void) \
                { \
                    static struct Table_s \
                    { \
                        byte        LookupTable[LookupTableSize]; \
                        ansichar    StringBlock[STRING_END - LookupTableSize + 1]; \
                        uint16      CRC; \
                    } Table = \
                    { \
                        {0}, \
        BUILD_STRING_TABLES_DMOIB_LANGUAGE_TABLE_
#define BUILD_STRING_TABLES_DMOIB_LANGUAGE_TABLE_(Strings) \
                        Strings

#define BUILD_STRING_TABLES_DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
        BUILD_STRING_TABLES_DMOIB_LANGUAGE_TABLE_CONTINUED_
#define BUILD_STRING_TABLES_DMOIB_LANGUAGE_TABLE_CONTINUED_(Strings) \
                        Strings

#define BUILD_STRING_TABLES_DMOIB_STRING(SymbolicLink, Text) \
                        Text "\0" \

#define BUILD_STRING_TABLES_DMOIB_STRING_END() \
                        , {0x0000} /*CRC place holder*/ \
                    }; \
                    return &Table; \
                } \
            }

#define BUILD_STRING_TABLES_DMOIB_LINK_TABLE_NO(LinkTableName) \
        BUILD_STRING_TABLES_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_TABLES_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        BUILD_STRING_TABLES_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_TABLES_DMOIB_LINK_TABLE_YES(LinkTableName) \
        BUILD_STRING_TABLES_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_TABLES_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        BUILD_STRING_TABLES_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_TABLES_DMOIB_LINK_TABLE_NO_(Entries) 

#define BUILD_STRING_TABLES_DMOIB_SUPPORTED_LANGUAGES(Languages) 

#define BUILD_STRING_TABLES_DMOIB_DEFINITIONS_BEGIN() \
        namespace MAKE_UNIQUE_NAME(Unused) \
        { \

#define BUILD_STRING_TABLES_DMOIB_DEFINITIONS_END() \
        }

/*
===============================================================================
    Macros for adding lookup information to string tables.
===============================================================================
*/
//===============================================
#define BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_NO(DictionaryName) \
        BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_YES(DictionaryName) \
        BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_NO_(Definitions) 

#define BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_CONTINUED_YES_
#define BUILD_STRING_LOOKUPS_DMOIB_DICTIONARY_CONTINUED_YES_(DMOIBObjects) 

#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_NO(StringTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_NO_(Entries) 

#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_YES(StringTableName) \
        } \
        namespace StringTableName \
        { \
        BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_YES_
#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_YES_(Entries) \
            Entries 

#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_CONTINUED_YES_
#define BUILD_STRING_LOOKUPS_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) \
            Entries 

#define BUILD_STRING_LOOKUPS_DMOIB_LANGUAGE_TABLE(Language) \
            namespace Language##Constants \
            { \
                void* LanguageTable(void) \
                { \
                    static void* pTable = NULL; \
                    if(pTable == NULL) \
                    { \
                        uint32* pTableIter = (uint32*)(pTable = LanguageBase()); \
                        uint32  Value; \
                        uint16  CRC; \
                        LINE_COPY(pTableIter, &(Value = Language), SIZEOF_LANGUAGE_ID); \
                        pTableIter = (uint32*)(((uint8*)pTableIter) + 1); \
                        LINE_COPY(pTableIter, &(Value = LINK_COUNT), SIZEOF_SYMBOLIC_LINK_COUNT); \
                        pTableIter = (uint32*)(((uint16*)pTableIter) + 1); \
        BUILD_STRING_LOOKUPS_DMOIB_LANGUAGE_TABLE_
#define BUILD_STRING_LOOKUPS_DMOIB_LANGUAGE_TABLE_(Strings) \
                        Strings

#define BUILD_STRING_LOOKUPS_DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
        BUILD_STRING_LOOKUPS_DMOIB_LANGUAGE_TABLE_CONTINUED_
#define BUILD_STRING_LOOKUPS_DMOIB_LANGUAGE_TABLE_CONTINUED_(Strings) \
                        Strings

#define BUILD_STRING_LOOKUPS_DMOIB_STRING(SymbolicLink, Text) \
                        LINE_COPY(pTableIter++, &(Value = SymbolicLink##Offset), SIZEOF_SYMBOLIC_LINK);

#define BUILD_STRING_LOOKUPS_DMOIB_STRING_END() \
                        LINE_COPY(pTableIter++, &(Value = STRING_END + 1 + SIZEOF_CRC), SIZEOF_SYMBOLIC_LINK); \
                        CRC = CalculateCRC((uint8*)pTable, STRING_END + 1); \
                        LINE_COPY((uint8*)pTable + STRING_END + 1, &CRC, SIZEOF_CRC); \
                    } \
                    return pTable; \
                } \
            }

#define BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_NO(LinkTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_YES(LinkTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_NO_
#define BUILD_STRING_LOOKUPS_DMOIB_LINK_TABLE_NO_(Entries) 

#define BUILD_STRING_LOOKUPS_DMOIB_SUPPORTED_LANGUAGES(Languages) 

#define BUILD_STRING_LOOKUPS_DMOIB_DEFINITIONS_BEGIN() \
        namespace MAKE_UNIQUE_NAME(Unused) \
        { \

#define BUILD_STRING_LOOKUPS_DMOIB_DEFINITIONS_END() \
        }


/*
===============================================================================
    Macros for DMOIB Initialization function calling.
===============================================================================
*/
//===============================================
#define INITIALIZE_DMOIB_DICTIONARY_NO(DictionaryName) \
        INITIALIZE_DMOIB_DICTIONARY_NO_
#define INITIALIZE_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        INITIALIZE_DMOIB_DICTIONARY_NO_
#define INITIALIZE_DMOIB_DICTIONARY_YES(DictionaryName) \
        INITIALIZE_DMOIB_DICTIONARY_NO_
#define INITIALIZE_DMOIB_DICTIONARY_NO_(Definitions) 

#define INITIALIZE_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        DictionaryName::MAKE_UNIQUE_NAME(Dictionary)(DictionaryName::Dictionary()); \
        INITIALIZE_DMOIB_DICTIONARY_CONTINUED_YES_
#define INITIALIZE_DMOIB_DICTIONARY_CONTINUED_YES_(DMOIBObjects)         

#define INITIALIZE_DMOIB_STRING_TABLE_NO(StringTableName) \
        INITIALIZE_DMOIB_STRING_TABLE_NO_
#define INITIALIZE_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        INITIALIZE_DMOIB_STRING_TABLE_NO_
#define INITIALIZE_DMOIB_STRING_TABLE_YES(StringTableName) \
        INITIALIZE_DMOIB_STRING_TABLE_NO_
#define INITIALIZE_DMOIB_STRING_TABLE_NO_(Entries) 

#define INITIALIZE_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        StringTableName::MAKE_UNIQUE_NAME(StringTable)(StringTableName::StringTable()); \
        INITIALIZE_DMOIB_STRING_TABLE_CONTINUED_YES_
#define INITIALIZE_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) 

#define INITIALIZE_DMOIB_LINK_TABLE_NO(LinkTableName) \
        INITIALIZE_DMOIB_LINK_TABLE_NO_
#define INITIALIZE_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        INITIALIZE_DMOIB_LINK_TABLE_NO_
#define INITIALIZE_DMOIB_LINK_TABLE_YES(LinkTableName) \
        INITIALIZE_DMOIB_LINK_TABLE_NO_
#define INITIALIZE_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        INITIALIZE_DMOIB_LINK_TABLE_NO_
#define INITIALIZE_DMOIB_LINK_TABLE_NO_(Entries) 

#define INITIALIZE_DMOIB_SUPPORTED_LANGUAGES(Languages) 

#define INITIALIZE_DMOIB_DEFINITIONS_BEGIN() 
#define INITIALIZE_DMOIB_DEFINITIONS_END() 

/*
===============================================================================
    Macros for DMOIB Implementation (binary table building)
===============================================================================
*/
//===============================================
#define IMPLEMENT_DMOIB_OBJECT(Family, Genus, Species, Version, ClassName) \
       (IMPLEMENT_DMOIB_OBJECT)(Family, Genus, Species, Version, ClassName)

#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES(ClassName) \
       (IMPLEMENT_DMOIB_AVAILABLE_SPECIES)(ClassName)

#define IMPLEMENT_DMOIB_SPECIES(Name, Value) \
       (IMPLEMENT_DMOIB_SPECIES)(Name, Value)

#define IMPLEMENT_DMOIB_MEMBERS(Members) \
       (IMPLEMENT_DMOIB_MEMBERS)(Members) 

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
       (IMPLEMENT_DMOIB_MEMBER)(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale)
#else
#define IMPLEMENT_DMOIB_MEMBER(Type, MemberName, MinVersion, MaxVersion, Alias) \
       (IMPLEMENT_DMOIB_MEMBER)(Type, MemberName, MinVersion, MaxVersion, Alias)
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED(MemberName) \
       (IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL)(MemberName)

#define IMPLEMENT_DMOIB_ENUMERATION(Name) \
       (IMPLEMENT_DMOIB_ENUMERATION)(Name) 

#define IMPLEMENT_DMOIB_ENUMERATOR(Name, Value) \
       (IMPLEMENT_DMOIB_ENUMERATOR_INTERNAL)(Name, Value)   

#define IMPLEMENT_DMOIB_LANGUAGE_TABLE(Language) \
       (IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL)(Language)

#define IMPLEMENT_DMOIB_LANGUAGE_TABLE_CONTINUED(Language) \
       (IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_CONTINUED)(Language)

#define IMPLEMENT_DMOIB_STRING(SymbolicLink, Text) \
       (IMPLEMENT_DMOIB_STRING_INTERNAL)(SymbolicLink, Text)

#define IMPLEMENT_DMOIB_STRING_END() \
       (IMPLEMENT_DMOIB_STRING_END_INTERNAL)()

#define IMPLEMENT_DMOIB_ENTRY(SymbolicLink) \
       (IMPLEMENT_DMOIB_ENTRY)(SymbolicLink) 

#define IMPLEMENT_DMOIB_LINK(ClassName, ElementID) \
       (IMPLEMENT_DMOIB_LINK)(ClassName, ElementID) 

#define IMPLEMENT_DMOIB_DEFINITIONS_BEGIN() 
#define IMPLEMENT_DMOIB_DEFINITIONS_END() 

//===============================================
#define BUILD_DICTIONARY(x)         x##_BUILD_DICTIONARY
#define END_BUILD_DICTIONARY        DO_NOTHING

#define CREATE_DECLARATIONS(x)      x##_CREATE_DECLARATIONS
#define END_CREATE_DECLARATIONS     DO_NOTHING

#define EVALUATE_DEFINITIONS(x)     x##_EVALUATE_DEFINITIONS
#define END_EVALUATE_DEFINITIONS    DO_NOTHING

#define DECLARE_XML_LOOKUP(x)       x##_DECLARE_XML_LOOKUP
#define END_DECLARE_XML_LOOKUP      DO_NOTHING

#define CREATE_DEFINITIONS(x)       x##_CREATE_DEFINITIONS
#define END_CREATE_DEFINITIONS      DO_NOTHING

#define COUNT_DEFINITIONS(x)        x##_COUNT_DEFINITIONS
#define END_COUNT_DEFINITIONS       DO_NOTHING

#define CREATE_LINK_TABLES(x)       x##_CREATE_LINK_TABLES
#define END_CREATE_LINK_TABLES      DO_NOTHING

#define CREATE_LINK_TABLE(x)        x##_CREATE_LINK_TABLE
#define END_CREATE_LINK_TABLE       DO_NOTHING

#define LINK_LINK_TABLES(x)         x##_LINK_LINK_TABLES
#define END_LINK_LINK_TABLES        DO_NOTHING

//===============================================
// Dictionary base
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_DICTIONARY_NO(DictionaryName) \
        IMPLEMENT_DMOIB_DICTIONARY_NO_
#define IMPLEMENT_DMOIB_DICTIONARY_CONTINUED_NO(DictionaryName) \
        IMPLEMENT_DMOIB_DICTIONARY_NO_
#define IMPLEMENT_DMOIB_DICTIONARY_NO_(Definitions) 

#define IMPLEMENT_DMOIB_DICTIONARY_YES(DictionaryName) \
        namespace DictionaryName \
        { \
            void* Dictionary(void) \
            { \
        IMPLEMENT_DMOIB_DICTIONARY_YES_
#define IMPLEMENT_DMOIB_DICTIONARY_YES_(Definitions) \
                static void* TableBase = NULL; \
                DMOIBInitialize(); \
                if(TableBase == NULL) \
                { \
                    void** TableIter = &TableBase; \
                    BUILD_DICTIONARY    Definitions (END) \
                    *TableIter = NULL; \
                } \
                return TableBase; \
            } \
            CREATE_DECLARATIONS         Definitions (END) \
            EVALUATE_DEFINITIONS        Definitions (END) \
            DECLARE_XML_LOOKUP          Definitions (END) \
            CREATE_DEFINITIONS          Definitions (END) \
            COMBINE(INCLUDE_, INCLUDE_LINK_TABLE) \
            ( \
                CREATE_LINK_TABLES Definitions (END) \
                void* LinkTable(void) \
                { \
                    static void* TableBase = NULL; \
                    DMOIBInitialize(); \
                    if(TableBase == NULL) \
                    { \
                        void** TableIter = &TableBase; \
                        LINK_LINK_TABLES    Definitions (END) \
                        *TableIter = NULL; \
                    } \
                    return TableBase; \
                }; \
            ) \
        }         
       

#define IMPLEMENT_DMOIB_DICTIONARY_CONTINUED_YES(DictionaryName) \
        namespace DictionaryName \
        { \
            void* MAKE_UNIQUE_NAME(Dictionary)(void* PreviousTable) \
            { \
        IMPLEMENT_DMOIB_DICTIONARY_CONTINUED_YES_
#define IMPLEMENT_DMOIB_DICTIONARY_CONTINUED_YES_(Definitions) \
                static void* TableBase = NULL; \
                DMOIBInitialize(); \
                if(TableBase == NULL) \
                { \
                    void** TableIter = &TableBase; \
                    BUILD_DICTIONARY    Definitions (END) \
                    *TableIter = NULL; \
                    if(PreviousTable != NULL) \
                    { \
                        TableIter = &PreviousTable; \
                        while(*(TableIter = (void**)&((*((TableEntry_t**)TableIter))->m_pNext)) != NULL); \
                        *TableIter = TableBase; \
                    } \
                    COMBINE(INCLUDE_, INCLUDE_LINK_TABLE) \
                    ( \
                        MAKE_UNIQUE_NAME(LinkTable)(LinkTable()); \
                    ) \
                } \
                return TableBase; \
            } \
            CREATE_DECLARATIONS         Definitions (END) \
            EVALUATE_DEFINITIONS        Definitions (END) \
            DECLARE_XML_LOOKUP          Definitions (END) \
            CREATE_DEFINITIONS          Definitions (END) \
            COMBINE(INCLUDE_, INCLUDE_LINK_TABLE) \
            ( \
                CREATE_LINK_TABLES Definitions (END) \
                void* MAKE_UNIQUE_NAME(LinkTable)(void* PreviousTable) \
                { \
                    static void* TableBase = NULL; \
                    DMOIBInitialize(); \
                    if(TableBase == NULL) \
                    { \
                        void** TableIter = &TableBase; \
                        LINK_LINK_TABLES    Definitions (END) \
                        *TableIter = NULL; \
                        if(PreviousTable != NULL) \
                        { \
                            TableIter = &PreviousTable; \
                            while(*(TableIter = (void**)&((*((TableEntry_t**)TableIter))->m_pNext)) != NULL); \
                            *TableIter = TableBase; \
                        } \
                    } \
                    return TableBase; \
                }; \
            ) \
        }  

//===============================================
// COUNT_DEFINITIONS
//-----------------------------------------------------------------------------

#define IMPLEMENT_DMOIB_OBJECT_COUNT_DEFINITIONS(Family, Genus, Species, Version, ClassName) \
        + 1 \
        IMPLEMENT_DMOIB_OBJECT_COUNT_DEFINITIONS_
#define IMPLEMENT_DMOIB_OBJECT_COUNT_DEFINITIONS_(Internals) \
        COUNT_DEFINITIONS

//===============================================
// BUILD_DICTIONARY
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_BUILD_DICTIONARY(Family, Genus, Species, Version, ClassName) \
        { \
            static TableEntry_t TableEntry = {(byte*)ClassName::Definition(), NULL}; \
            *TableIter = &TableEntry; \
            TableIter = (void**)&TableEntry.m_pNext; \
        } \
        IMPLEMENT_DMOIB_OBJECT_BUILD_DICTIONARY_
#define IMPLEMENT_DMOIB_OBJECT_BUILD_DICTIONARY_(Internals) \
        BUILD_DICTIONARY


//===============================================
// CREATE_DECLARATIONS
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_CREATE_DECLARATIONS(Family, Genus, Species, Version, ClassName) \
        namespace ClassName \
        IMPLEMENT_DMOIB_OBJECT_CREATE_DECLARATIONS_
#define IMPLEMENT_DMOIB_OBJECT_CREATE_DECLARATIONS_(Internals) \
        { \
            namespace LookupConstants \
            { \
                enum LinkTableLookup_e \
                { \
                    LinkTableLookupBase = SIZEOF_CLASSID + \
                                          SIZEOF_LINK_TABLE_SIZE + \
                                          SIZEOF_MEMBER_COUNT + \
                                          SIZEOF_LINK_TABLE_COUNT, \
                    /*LinkTableLookupInit = LinkTableLookupBase - 1, */\
                    DECLARE_DMOIB_ENUMERATION_LOOKUP  Internals (END) \
                    LinkTableLookupEnumerationEnd, \
                    LinkTableLookupEnumerationCount = ((LinkTableLookupEnumerationEnd - 1) \
                                                        / (SIZEOF_LINK_TABLE_ENTRY*2)) - 1 , \
                    LinkTableLookupEnumeratorStart = LinkTableLookupEnumerationEnd - 1, \
                    DECLARE_DMOIB_ENUMERATOR_LOOKUP   Internals (END) \
                    LinkTableLookupSize \
                }; \
            } \
        } \
        CREATE_DECLARATIONS


//-----------------------------------------------------------------------------
#define DECLARE_DMOIB_MEMBER_LOOKUP(x)         x##_DECLARE_DMOIB_MEMBER_LOOKUP
#define END_DECLARE_DMOIB_MEMBER_LOOKUP        DO_NOTHING

#define DECLARE_DMOIB_ENUMERATION_LOOKUP(x)    x##_DECLARE_DMOIB_ENUMERATION_LOOKUP
#define END_DECLARE_DMOIB_ENUMERATION_LOOKUP   DO_NOTHING

#define DECLARE_DMOIB_ENUMERATOR_LOOKUP(x)     x##_DECLARE_DMOIB_ENUMERATOR_LOOKUP
#define END_DECLARE_DMOIB_ENUMERATOR_LOOKUP    DO_NOTHING

//-----------------------------------------------------------------------------
// DECLARE_DMOIB_ENUMERATION_LOOKUP
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_DMOIB_ENUMERATION_LOOKUP(ClassName) \
        SpeciesTableHeaderEntryValue, \
        SpeciesTableHeaderSpace = SpeciesTableHeaderEntryValue + (SIZEOF_LINK_TABLE_ENTRY*2) - 1, \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_DMOIB_ENUMERATION_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_DMOIB_ENUMERATION_LOOKUP_INTERNALS(Species) \
        DECLARE_DMOIB_ENUMERATION_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_DECLARE_DMOIB_ENUMERATION_LOOKUP(Members) \
        DECLARE_DMOIB_ENUMERATION_LOOKUP Members (END) \
        DECLARE_DMOIB_ENUMERATION_LOOKUP

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_DECLARE_DMOIB_ENUMERATION_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something here for member strings */ \
        Member##MemberName##HeaderEntryValue, \
        Member##MemberName##HeaderSpace = Member##MemberName##HeaderEntryValue + (SIZEOF_LINK_TABLE_ENTRY*2) - 1, \
        DECLARE_DMOIB_ENUMERATION_LOOKUP
#else
#define IMPLEMENT_DMOIB_MEMBER_DECLARE_DMOIB_ENUMERATION_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something here for member strings */ \
        Member##MemberName##HeaderEntryValue, \
        Member##MemberName##HeaderSpace = Member##MemberName##HeaderEntryValue + (SIZEOF_LINK_TABLE_ENTRY*2) - 1, \
        DECLARE_DMOIB_ENUMERATION_LOOKUP
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_DECLARE_DMOIB_ENUMERATION_LOOKUP(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATION_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATION_LOOKUP(Name)  \
        Enumeration##Name##HeaderEntryValue, \
        Enumeration##Name##HeaderSpace = Enumeration##Name##HeaderEntryValue + (SIZEOF_LINK_TABLE_ENTRY*2) - 1, \
        IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATION_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATION_LOOKUP_(Internals) \
        DECLARE_DMOIB_ENUMERATION_LOOKUP


//-----------------------------------------------------------------------------
// DECLARE_DMOIB_ENUMERATOR_LOOKUP
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_DMOIB_ENUMERATOR_LOOKUP(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_DMOIB_ENUMERATOR_LOOKUP_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_DMOIB_ENUMERATOR_LOOKUP_(Species) \
        SpeciesTableEnumEntryValue, \
        SpeciesTableEnumOffset = SpeciesTableEnumEntryValue - 1, \
        SpeciesTableEnumEnd = SpeciesTableEnumOffset \
            + SIZEOF_LINK_TABLE_ENTRY /* Enumeration Size header */ \
            + DECLARE_DMOIB_ENUMERATOR_LOOKUP Species (END) \
            + 0, \
        DECLARE_DMOIB_ENUMERATOR_LOOKUP
#define IMPLEMENT_DMOIB_SPECIES_DECLARE_DMOIB_ENUMERATOR_LOOKUP(Name, Value) \
        /* Do something for species strings */ \
        (SIZEOF_LINK_TABLE_ENTRY*2) + \
        DECLARE_DMOIB_ENUMERATOR_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_DECLARE_DMOIB_ENUMERATOR_LOOKUP(Members) \
        DECLARE_DMOIB_ENUMERATOR_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_DECLARE_DMOIB_ENUMERATOR_LOOKUP(MemberName)  \
        Enumeration##MemberName##EnumEntryValue, \
        MemberName##EnumOffset = Enumeration##MemberName##EnumEntryValue - 1, \
        Enumeration##MemberName##EnumEnd = MemberName##EnumOffset \
        IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATOR_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATOR_LOOKUP(Name)  \
        Enumeration##Name##EnumEntryValue, \
        Enumeration##Name##EnumOffset = Enumeration##Name##EnumEntryValue - 1, \
        Enumeration##Name##EnumEnd = Enumeration##Name##EnumOffset \
        IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATOR_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_DECLARE_DMOIB_ENUMERATOR_LOOKUP_(Internals) \
        + SIZEOF_LINK_TABLE_ENTRY /* Enumeration Size header */ \
        + DECLARE_DMOIB_ENUMERATOR_LOOKUP Internals (END) \
        + 0, \
        DECLARE_DMOIB_ENUMERATOR_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATOR_INTERNAL_DECLARE_DMOIB_ENUMERATOR_LOOKUP(Name, Value) \
        /* Do something for Enumerators */ \
        (SIZEOF_LINK_TABLE_ENTRY*2) + \
        DECLARE_DMOIB_ENUMERATOR_LOOKUP
//-----------------------------------------------------------------------------

//===============================================
// EVALUATE_DEFINITIONS

#define EVALUATE_XML(x)     x##_EVALUATE_XML
#define END_EVALUATE_XML    DO_NOTHING

//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_EVALUATE_DEFINITIONS(Family, Genus, Species, Version, ClassName) \
        namespace ClassName \
        IMPLEMENT_DMOIB_OBJECT_EVALUATE_DEFINITIONS_
#define IMPLEMENT_DMOIB_OBJECT_EVALUATE_DEFINITIONS_(Internals) \
        { \
            enum DefinitionSize_e\
            { \
                EVALUATE_DEFINITIONS  Internals (END) \
                DEFINITION_SIZE , /*Includes number of Species*/\
                TOTAL_DEFINITION_SIZE = DEFINITION_SIZE  \
                                        + SIZEOF_SPECIES_COUNT  /*Species Count*/  \
                                        + SIZEOF_FAMILY         /*Family*/ \
                                        + SIZEOF_GENUS          /*Genus*/ \
                                        + SIZEOF_VERSION        /*Version*/ \
                                        + SIZEOF_DEFINITION     /*DefSize*/ \
                                        + SIZEOF_DEFINITION     /*Full DefSize*/ \
                                        + SIZEOF_XML_CONTENT    /*Base XML Size*/ \
                                        + SIZEOF_MEMBER_COUNT   /*Member Count*/ \
            }; \
            enum XMLSize_e \
            { \
                XML_BASE_SIZE = XML_VALUE \
                ( \
                    + sizeof("<"XML_TAG_DEFINITION" "XML_ATT_CLASS"\"\" "XML_ATT_VERSION"\"\" "XML_ATT_FAMILY"\"\" "XML_ATT_GENUS"\"\" "XML_ATT_SPECIES"\"\">\n") -1 \
                    + STRLEN_OF_INT(VERSION) \
                    + STRLEN_OF_INT(FAMILY) \
                    + STRLEN_OF_INT(GENUS) \
                    + STRLEN_OF_INT(SPECIES) \
                    + sizeof("<"XML_TAG_MEMBERS">\n") -1 \
                    + EVALUATE_XML Internals (END) /* Internal members */ \
                    + sizeof("</"XML_TAG_MEMBERS">\n") -1 \
                    + sizeof("</"XML_TAG_DEFINITION">\n") -1 \
                ) +0 \
            }; \
        } \
        EVALUATE_DEFINITIONS

//-----------------------------------------------------------------------------
// EVALUATE_DEFINITIONS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_EVALUATE_DEFINITIONS(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_EVALUATE_DEFINITIONS_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_EVALUATE_DEFINITIONS_(Species) \
        EVALUATE_DEFINITIONS Species (END) \
        SpeciesCount, \
        SpeciesCountExitValue = SpeciesCount - 1, \
        EVALUATE_DEFINITIONS
#define IMPLEMENT_DMOIB_SPECIES_EVALUATE_DEFINITIONS(Name, Value) \
        /* Do something for species */ \
        Species##Name##DefSpacer, \
        EVALUATE_DEFINITIONS

#define IMPLEMENT_DMOIB_MEMBERS_EVALUATE_DEFINITIONS(Members) \
        EVALUATE_DEFINITIONS Members (END) \
        EVALUATE_DEFINITIONS

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_EVALUATE_DEFINITIONS(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something for members */ \
        Member##MemberName##DefSizeEntryValue, \
        Member##MemberName##DefSize = Member##MemberName##DefSizeEntryValue \
                                      + (SIZEOF_VERSION * 2) \
                                      + DEF_SIZEOF(Type) \
                                      + (SIZEOF_ALIAS + SIZEOF_UNITS + SIZEOF_SCALE), \
        EVALUATE_DEFINITIONS
#else
#define IMPLEMENT_DMOIB_MEMBER_EVALUATE_DEFINITIONS(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something for members */ \
        Member##MemberName##DefSizeEntryValue, \
        Member##MemberName##DefSize = Member##MemberName##DefSizeEntryValue \
                                      + (SIZEOF_VERSION * 2) \
                                      + DEF_SIZEOF(Type) \
                                      + (SIZEOF_ALIAS), \
        EVALUATE_DEFINITIONS
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_EVALUATE_DEFINITIONS(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_DEFINITIONS_
#define IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_DEFINITIONS(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_DEFINITIONS_
#define IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_DEFINITIONS_(Internals) \
        EVALUATE_DEFINITIONS

//-----------------------------------------------------------------------------
// EVALUATE_XML
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_EVALUATE_XML(ClassName) \
        + (sizeof(#ClassName) -1) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_EVALUATE_XML_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_EVALUATE_XML_(Species) \
        EVALUATE_XML Species (END) \
        EVALUATE_XML
#define IMPLEMENT_DMOIB_SPECIES_EVALUATE_XML(Name, Value) \
        EVALUATE_XML

#define IMPLEMENT_DMOIB_MEMBERS_EVALUATE_XML(Members) \
        EVALUATE_XML Members (END) \
        EVALUATE_XML

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_EVALUATE_XML(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something for members */ \
        + (sizeof("<"XML_TAG_DEFINITION" "XML_ATT_TYPE"\"\" "XML_ATT_NAME"\"\" "XML_ATT_MINVERSION"\"\" "XML_ATT_MAXVERSION"\"\"/>\n") -1) \
        + XML_SIZEOF(Type) \
        + (sizeof(#MemberName) -1) \
        + STRLEN_OF_INT(MinVersion) \
        + (MaxVersion == CURRENT_VERSION ? (sizeof(XML_VAL_CURRENT_VERSION) -1) : \
          STRLEN_OF_INT(MaxVersion)) \
        + (Alias == OMITTED ? 0 : \
          sizeof(" "XML_ATT_ALIAS"\"\"") -1 + STRLEN_OF_INT(Alias)) \
        + (Units == OMITTED ? 0 : \
          sizeof(" "XML_ATT_UNITS"\"\"") -1 + STRLEN_OF_INT(Units)) \
        + (Scale == OMITTED ? 0 : \
          sizeof(" "XML_ATT_SCALE"\"\"") -1 + STRLEN_OF_INT(Scale)) \
        EVALUATE_XML
#else
#define IMPLEMENT_DMOIB_MEMBER_EVALUATE_XML(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something for members */ \
        + (sizeof("<"XML_TAG_DEFINITION" "XML_ATT_TYPE"\"\" "XML_ATT_NAME"\"\" "XML_ATT_MINVERSION"\"\" "XML_ATT_MAXVERSION"\"\"/>\n") -1) \
        + XML_SIZEOF(Type) \
        + (sizeof(#MemberName) -1) \
        + STRLEN_OF_INT(MinVersion) \
        + (MaxVersion == CURRENT_VERSION ? (sizeof(XML_VAL_CURRENT_VERSION) -1) : \
          STRLEN_OF_INT(MaxVersion)) \
        + (Alias == OMITTED ? 0 : \
          sizeof(" "XML_ATT_ALIAS"\"\"") -1 + STRLEN_OF_INT(Alias)) \
        EVALUATE_XML
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_EVALUATE_XML(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_XML_
#define IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_XML(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_XML_
#define IMPLEMENT_DMOIB_ENUMERATION_EVALUATE_XML_(Internals) \
        EVALUATE_XML

//-----------------------------------------------------------------------------

//===============================================
// DECLARE_XML_LOOKUP
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_DECLARE_XML_LOOKUP(Family, Genus, Species, Version, ClassName) \
        namespace ClassName \
        IMPLEMENT_DMOIB_OBJECT_DECLARE_XML_LOOKUP_
#define IMPLEMENT_DMOIB_OBJECT_DECLARE_XML_LOOKUP_(Internals) \
        { \
            namespace LookupConstants \
            { \
                enum XMLLookup_e \
                { \
                    XMLLookupBase = ALIGN_TO_BOUNDARY(TOTAL_DEFINITION_SIZE \
                                                        + LinkTableLookupSize \
                                                        - SIZEOF_CLASSID \
                                                        - SIZEOF_LINK_TABLE_SIZE), \
                    XMLLookupEntry = XMLLookupBase - TOTAL_DEFINITION_SIZE -1, \
                    DECLARE_XML_LOOKUP Internals (END) \
                    XMLLookupExit, \
                    XMLLookupEnd = XMLLookupExit + TOTAL_DEFINITION_SIZE, \
                    XMLLookupSize \
                }; \
            } \
        } \
        DECLARE_XML_LOOKUP

//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_XML_LOOKUP(ClassName) \
        XMLSpeciesTableOffset, \
        XMLSpeciesTableOffsetExitValue = XMLSpeciesTableOffset + sizeof(#ClassName) - 1, \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_XML_LOOKUP_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_DECLARE_XML_LOOKUP_(Species) \
        DECLARE_XML_LOOKUP Species (END) \
        DECLARE_XML_LOOKUP
#define IMPLEMENT_DMOIB_SPECIES_DECLARE_XML_LOOKUP(Name, Value) \
        XMLSpecies##Name##Offset, \
        XMLSpecies##Name##OffsetExitValue = XMLSpecies##Name##Offset + sizeof(#Name) - 1, \
        DECLARE_XML_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_DECLARE_XML_LOOKUP(Members) \
        DECLARE_XML_LOOKUP Members (END) \
        DECLARE_XML_LOOKUP

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_DECLARE_XML_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        XMLMember##MemberName##Offset, \
        XMLMember##MemberName##OffsetExitValue = XMLMember##MemberName##Offset + sizeof(#MemberName) - 1, \
        DECLARE_XML_LOOKUP
#else
#define IMPLEMENT_DMOIB_MEMBER_DECLARE_XML_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias) \
        XMLMember##MemberName##Offset, \
        XMLMember##MemberName##OffsetExitValue = XMLMember##MemberName##Offset + sizeof(#MemberName) - 1, \
        DECLARE_XML_LOOKUP
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_DECLARE_XML_LOOKUP(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_DECLARE_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_DECLARE_XML_LOOKUP(Name)  \
        XMLEnumeration##Name##Offset, \
        XMLEnumeration##Name##OffsetExitValue = XMLEnumeration##Name##Offset + sizeof(#Name) - 1, \
        IMPLEMENT_DMOIB_ENUMERATION_DECLARE_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_DECLARE_XML_LOOKUP_(Internals) \
        DECLARE_XML_LOOKUP Internals (END) \
        DECLARE_XML_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATOR_INTERNAL_DECLARE_XML_LOOKUP(Name, Value) \
        XMLEnumerator##Name##Offset, \
        XMLEnumerator##Name##OffsetExitValue = XMLEnumerator##Name##Offset + sizeof(#Name) - 1, \
        DECLARE_XML_LOOKUP

//-----------------------------------------------------------------------------

//===============================================
// CREATE_DEFINITIONS
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_CREATE_DEFINITIONS(Family, Genus, Species, Version, ClassName) \
        namespace ClassName \
        IMPLEMENT_DMOIB_OBJECT_CREATE_DEFINITIONS_
#define IMPLEMENT_DMOIB_OBJECT_CREATE_DEFINITIONS_(Internals) \
        { \
            namespace LookupConstants \
            { \
                void* GetDefinition(void) \
                { \
                    static void* pDefinition = NULL; \
                    struct DefinitionBase_t \
                    { \
                        byte Definition[XML_VALUE(XMLLookupBase - TOTAL_DEFINITION_SIZE) \
                                        + TOTAL_DEFINITION_SIZE]; \
                        XML_VALUE \
                        ( \
                            ansichar XMLStrings[XMLLookupSize - XMLLookupBase]; \
                        ) \
                        uint16 CRC; \
                    } static DefinitionBase = \
                    { \
                        { \
                            SpeciesCount, \
                            UINT16_TO_UINT8(FAMILY), \
                            GENUS, \
                            CREATE_DMOIB_SPECIES Internals (END) \
                            UINT16_TO_UINT8(VERSION), \
                            UINT16_TO_UINT8(TOTAL_DEFINITION_SIZE), \
                            UINT16_TO_UINT8(sizeof(struct DefinitionBase_t)), \
                            UINT16_TO_UINT8(XML_BASE_SIZE), \
                            UINT16_TO_UINT8(DMOIB_MEMBER_COUNT), \
                            CREATE_DMOIB_MEMBERS Internals (END) \
                            CREATE_ALIASES Internals (END) \
                            XML_VALUE \
                            ( \
                                UINT16_TO_UINT8(DMOIB_MEMBER_COUNT) COMMA \
                                UINT16_TO_UINT8(LinkTableLookupEnumerationCount) COMMA \
                                CREATE_DMOIB_MEMBER_XML_LOOKUP        Internals (END) \
                                CREATE_DMOIB_ENUMERATION_XML_LOOKUP   Internals (END) \
                                CREATE_DMOIB_ENUMERATOR_XML_LOOKUP    Internals (END) \
                            ) \
                        } \
                        XML_VALUE \
                        ( \
                            COMMA \
                            { \
                                CREATE_XML_STRINGS Internals (END) \
                            } \
                        ) \
                        , {0x0000} /*CRC place holder*/ \
                    }; \
                    if(pDefinition == NULL) \
                    { \
                        pDefinition = (void*)&DefinitionBase; \
                        uint16 CRC = CalculateCRC((uint8*)&DefinitionBase, sizeof(struct DefinitionBase_t) - SIZEOF_CRC); \
                        LINE_COPY(((uint8*)&DefinitionBase) + sizeof(struct DefinitionBase_t) - SIZEOF_CRC, &CRC, SIZEOF_CRC); \
                    } \
                    DMOIBInitialize(); \
                    return pDefinition; \
                } \
            } \
            void* Definition(void){return LookupConstants::GetDefinition();} \
        } \
        CREATE_DEFINITIONS

//-----------------------------------------------------------------------------
#define CREATE_DMOIB_SPECIES(x)               x##_CREATE_DMOIB_SPECIES
#define END_CREATE_DMOIB_SPECIES              DO_NOTHING

#define CREATE_DMOIB_MEMBERS(x)               x##_CREATE_DMOIB_MEMBERS
#define END_CREATE_DMOIB_MEMBERS              DO_NOTHING

//-----------------------------------------------------------------------------
// CREATE_DMOIB_SPECIES
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_SPECIES(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_SPECIES_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_SPECIES_(Species) \
        CREATE_DMOIB_SPECIES Species (END) \
        CREATE_DMOIB_SPECIES
#define IMPLEMENT_DMOIB_SPECIES_CREATE_DMOIB_SPECIES(Name, Value) \
        /* Do something for species */ \
        Name, \
        CREATE_DMOIB_SPECIES

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_SPECIES(Members) \
        CREATE_DMOIB_SPECIES Members (END) \
        CREATE_DMOIB_SPECIES

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_SPECIES(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do nothing for members in this case */ \
        CREATE_DMOIB_SPECIES
#else
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_SPECIES(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do nothing for members in this case */ \
        CREATE_DMOIB_SPECIES
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_SPECIES(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_SPECIES_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_SPECIES(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_SPECIES_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_SPECIES_(Internals) \
        CREATE_DMOIB_SPECIES

//-----------------------------------------------------------------------------
// CREATE_DMOIB_MEMBERS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBERS(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBERS_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBERS_(Species) \
        CREATE_DMOIB_MEMBERS Species (END) \
        CREATE_DMOIB_MEMBERS
#define IMPLEMENT_DMOIB_SPECIES_CREATE_DMOIB_MEMBERS(Name, Value) \
        /* Do nothing for species in this case */ \
        CREATE_DMOIB_MEMBERS

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_MEMBERS(Members) \
        CREATE_DMOIB_MEMBERS Members (END) \
        CREATE_DMOIB_MEMBERS

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_MEMBERS(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something for members */ \
        UINT16_TO_UINT8(MinVersion), \
        UINT16_TO_UINT8(MaxVersion), \
        BYTE_CODE_FOR(Type), \
        CREATE_DMOIB_MEMBERS
#else
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_MEMBERS(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something for members */ \
        UINT16_TO_UINT8(MinVersion), \
        UINT16_TO_UINT8(MaxVersion), \
        BYTE_CODE_FOR(Type), \
        CREATE_DMOIB_MEMBERS
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_MEMBERS(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBERS_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBERS(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBERS_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBERS_(Internals) \
        CREATE_DMOIB_MEMBERS

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define CREATE_DMOIB_MEMBER_XML_LOOKUP(x)       x##_CREATE_DMOIB_MEMBER_XML_LOOKUP
#define END_CREATE_DMOIB_MEMBER_XML_LOOKUP      DO_NOTHING

#define CREATE_DMOIB_ENUMERATION_XML_LOOKUP(x)  x##_CREATE_DMOIB_ENUMERATION_XML_LOOKUP
#define END_CREATE_DMOIB_ENUMERATION_XML_LOOKUP DO_NOTHING

#define CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(x)   x##_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP
#define END_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP  DO_NOTHING

#define CREATE_XML_STRINGS(x)                   x##_CREATE_XML_STRINGS
#define END_CREATE_XML_STRINGS                  DO_NOTHING

#define CREATE_ALIASES(x)                       x##_CREATE_ALIASES
#define END_CREATE_ALIASES                      DO_NOTHING

//-----------------------------------------------------------------------------
// CREATE_DMOIB_MEMBER_XML_LOOKUP (lookup table for Member information)
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBER_XML_LOOKUP(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBER_XML_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBER_XML_LOOKUP_INTERNALS(Species) \
        CREATE_DMOIB_MEMBER_XML_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_MEMBER_XML_LOOKUP(Members) \
        CREATE_DMOIB_MEMBER_XML_LOOKUP Members (END) \
        CREATE_DMOIB_MEMBER_XML_LOOKUP

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_MEMBER_XML_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something here for member strings */ \
        UINT16_TO_UINT8((MemberName##EnumOffset == 0xFFFF ? 0xFFFF : MemberName##EnumOffset \
                                                                     - SIZEOF_CLASSID \
                                                                     - SIZEOF_LINK_TABLE_SIZE)), \
        UINT16_TO_UINT8(XMLMember##MemberName##Offset), \
        CREATE_DMOIB_MEMBER_XML_LOOKUP
#else
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_MEMBER_XML_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something here for member strings */ \
        UINT16_TO_UINT8((MemberName##EnumOffset == 0xFFFF ? 0xFFFF : MemberName##EnumOffset \
                                                                     - SIZEOF_CLASSID \
                                                                     - SIZEOF_LINK_TABLE_SIZE)), \
        UINT16_TO_UINT8(XMLMember##MemberName##Offset), \
        CREATE_DMOIB_MEMBER_XML_LOOKUP
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_MEMBER_XML_LOOKUP(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_XML_LOOKUP(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_XML_LOOKUP_(Internals) \
        CREATE_DMOIB_MEMBER_XML_LOOKUP
//-----------------------------------------------------------------------------
// CREATE_DMOIB_ENUMERATION_XML_LOOKUP
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATION_XML_LOOKUP(ClassName) \
        UINT16_TO_UINT8((SpeciesTableEnumOffset == 0xFFFF ? 0xFFFF : SpeciesTableEnumOffset \
                                                                     - SIZEOF_CLASSID \
                                                                     - SIZEOF_LINK_TABLE_SIZE)), \
        UINT16_TO_UINT8(XMLSpeciesTableOffset), \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATION_XML_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATION_XML_LOOKUP_INTERNALS(Species) \
        CREATE_DMOIB_ENUMERATION_XML_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_ENUMERATION_XML_LOOKUP(Members) \
        CREATE_DMOIB_ENUMERATION_XML_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_ENUMERATION_XML_LOOKUP(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_XML_LOOKUP(Name)  \
        UINT16_TO_UINT8((Enumeration##Name##EnumOffset == 0xFFFF ? 0xFFFF : Enumeration##Name##EnumOffset \
                                                                     - SIZEOF_CLASSID \
                                                                     - SIZEOF_LINK_TABLE_SIZE)), \
        UINT16_TO_UINT8(XMLEnumeration##Name##Offset), \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_XML_LOOKUP_(Internals) \
        CREATE_DMOIB_ENUMERATION_XML_LOOKUP
//-----------------------------------------------------------------------------
// CREATE_DMOIB_ENUMERATOR_XML_LOOKUP
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP_INTERNALS(Species) \
        UINT16_TO_UINT8(SpeciesTableEnumEnd) - SIZEOF_CLASSID - SIZEOF_LINK_TABLE_SIZE, \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP Species (END) \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP
#define IMPLEMENT_DMOIB_SPECIES_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(Name, Value) \
        /* Do something for species strings */ \
        UINT16_TO_UINT8(Name), \
        UINT16_TO_UINT8(XMLSpecies##Name##Offset), \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(Members) \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(MemberName)  \
        UINT16_TO_UINT8((Enumeration##MemberName##EnumEnd== 0xFFFF ? 0xFFFF : Enumeration##MemberName##EnumEnd \
                                                                     - SIZEOF_CLASSID \
                                                                     - SIZEOF_LINK_TABLE_SIZE)), \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(Name)  \
        UINT16_TO_UINT8((Enumeration##Name##EnumEnd == 0xFFFF ? 0xFFFF : Enumeration##Name##EnumEnd \
                                                                     - SIZEOF_CLASSID \
                                                                     - SIZEOF_LINK_TABLE_SIZE)), \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP_(Internals) \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP Internals (END) \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATOR_INTERNAL_CREATE_DMOIB_ENUMERATOR_XML_LOOKUP(Name, Value) \
        /* Do something for Enumerators */ \
        UINT16_TO_UINT8(Name), \
        UINT16_TO_UINT8(XMLEnumerator##Name##Offset), \
        CREATE_DMOIB_ENUMERATOR_XML_LOOKUP
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// CREATE_XML_STRINGS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_XML_STRINGS(ClassName) \
        #ClassName "\0" \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_XML_STRINGS_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_XML_STRINGS_INTERNALS(Species) \
        CREATE_XML_STRINGS Species (END) \
        CREATE_XML_STRINGS
#define IMPLEMENT_DMOIB_SPECIES_CREATE_XML_STRINGS(Name, Value) \
        #Name "\0" \
        CREATE_XML_STRINGS

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_XML_STRINGS(Members) \
        CREATE_XML_STRINGS Members (END) \
        CREATE_XML_STRINGS

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_CREATE_XML_STRINGS(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        #MemberName "\0" \
        CREATE_XML_STRINGS
#else
#define IMPLEMENT_DMOIB_MEMBER_CREATE_XML_STRINGS(Type, MemberName, MinVersion, MaxVersion, Alias) \
        #MemberName "\0" \
        CREATE_XML_STRINGS
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_XML_STRINGS(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_XML_STRINGS_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_XML_STRINGS(Name)  \
        #Name "\0" \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_XML_STRINGS_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_XML_STRINGS_(Internals) \
        CREATE_XML_STRINGS Internals (END) \
        CREATE_XML_STRINGS

#define IMPLEMENT_DMOIB_ENUMERATOR_INTERNAL_CREATE_XML_STRINGS(Name, Value) \
        #Name "\0" \
        CREATE_XML_STRINGS

//-----------------------------------------------------------------------------
// CREATE_ALIASES
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_ALIASES(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_ALIASES_
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_ALIASES_(Species) \
        CREATE_ALIASES Species (END) \
        CREATE_ALIASES
#define IMPLEMENT_DMOIB_SPECIES_CREATE_ALIASES(Name, Value) \
        /* Do nothing for species in this case */ \
        CREATE_ALIASES

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_ALIASES(Members) \
        CREATE_ALIASES Members (END) \
        CREATE_ALIASES

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_CREATE_ALIASES(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something for members */ \
        UINT32_TO_UINT8(Alias), \
        UINT16_TO_UINT8(Units), \
        UINT16_TO_UINT8(Scale), \
        CREATE_ALIASES
#else
#define IMPLEMENT_DMOIB_MEMBER_CREATE_ALIASES(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something for members */ \
        UINT32_TO_UINT8(Alias), \
        CREATE_ALIASES
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_ALIASES(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_ALIASES_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_ALIASES(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_ALIASES_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_ALIASES_(Internals) \
        CREATE_ALIASES
//-----------------------------------------------------------------------------


//===============================================
// CREATE_LINK_TABLE
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_CREATE_LINK_TABLES(Family, Genus, Species, Version, ClassName) \
        namespace ClassName \
        IMPLEMENT_DMOIB_OBJECT_CREATE_LINK_TABLES_
#define IMPLEMENT_DMOIB_OBJECT_CREATE_LINK_TABLES_(Internals) \
        { \
            namespace LookupConstants \
            { \
                void* LinkTable(void) \
                { \
                    static void* pLinkTable = NULL; \
                    static unsigned char LinkTable[LinkTableLookupSize + SIZEOF_CRC] = \
                    { \
                        UINT32_TO_UINT8(CLASSID), \
                        UINT16_TO_UINT8(LinkTableLookupSize + SIZEOF_CRC), \
                        UINT16_TO_UINT8(DMOIB_MEMBER_COUNT), \
                        UINT16_TO_UINT8(LinkTableLookupEnumerationCount), \
                        CREATE_DMOIB_MEMBER_LOOKUP        Internals (END) \
                        CREATE_DMOIB_ENUMERATION_LOOKUP   Internals (END) \
                        CREATE_DMOIB_ENUMERATOR_LOOKUP    Internals (END) \
                        0x0000 /*CRC place holder*/ \
                    }; \
                    if(pLinkTable == NULL) \
                    { \
                        pLinkTable = LinkTable; \
                        uint16 CRC = CalculateCRC((uint8*)LinkTable, LinkTableLookupSize); \
                        LINE_COPY(((uint8*)LinkTable) + LinkTableLookupSize, &CRC, SIZEOF_CRC); \
                    } \
                    DMOIBInitialize(); \
                    return pLinkTable; \
                } \
            } \
        } \
        CREATE_LINK_TABLES


//-----------------------------------------------------------------------------
#define CREATE_DMOIB_MEMBER_LOOKUP(x)         x##_CREATE_DMOIB_MEMBER_LOOKUP
#define END_CREATE_DMOIB_MEMBER_LOOKUP        DO_NOTHING

#define CREATE_DMOIB_ENUMERATION_LOOKUP(x)    x##_CREATE_DMOIB_ENUMERATION_LOOKUP
#define END_CREATE_DMOIB_ENUMERATION_LOOKUP   DO_NOTHING

#define CREATE_DMOIB_ENUMERATOR_LOOKUP(x)     x##_CREATE_DMOIB_ENUMERATOR_LOOKUP
#define END_CREATE_DMOIB_ENUMERATOR_LOOKUP    DO_NOTHING

//-----------------------------------------------------------------------------
// CREATE_DMOIB_MEMBER_LOOKUP (lookup table for Member information)
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBER_LOOKUP(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBER_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_MEMBER_LOOKUP_INTERNALS(Species) \
        CREATE_DMOIB_MEMBER_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_MEMBER_LOOKUP(Members) \
        CREATE_DMOIB_MEMBER_LOOKUP Members (END) \
        CREATE_DMOIB_MEMBER_LOOKUP

#if defined(CRA383_SUPPORT)
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_MEMBER_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias, Units, Scale) \
        /* Do something here for member strings */ \
        UINT16_TO_UINT8(MemberName##EnumOffset), \
        UINT16_TO_UINT8(MemberName##SymbolicLink), \
        CREATE_DMOIB_MEMBER_LOOKUP
#else
#define IMPLEMENT_DMOIB_MEMBER_CREATE_DMOIB_MEMBER_LOOKUP(Type, MemberName, MinVersion, MaxVersion, Alias) \
        /* Do something here for member strings */ \
        UINT16_TO_UINT8(MemberName##EnumOffset), \
        UINT16_TO_UINT8(MemberName##SymbolicLink), \
        CREATE_DMOIB_MEMBER_LOOKUP
#endif

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_MEMBER_LOOKUP(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_LOOKUP(Name)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_MEMBER_LOOKUP_(Internals) \
        CREATE_DMOIB_MEMBER_LOOKUP
//-----------------------------------------------------------------------------
// CREATE_DMOIB_ENUMERATION_LOOKUP
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATION_LOOKUP(ClassName) \
        UINT16_TO_UINT8(SpeciesTableEnumOffset), \
        UINT16_TO_UINT8(ClassName##SymbolicLink), \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATION_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATION_LOOKUP_INTERNALS(Species) \
        CREATE_DMOIB_ENUMERATION_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_ENUMERATION_LOOKUP(Members) \
        CREATE_DMOIB_ENUMERATION_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_ENUMERATION_LOOKUP(MemberName)  \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_LOOKUP(Name)  \
        UINT16_TO_UINT8(Enumeration##Name##EnumOffset), \
        UINT16_TO_UINT8(Name##SymbolicLink), \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATION_LOOKUP_(Internals) \
        CREATE_DMOIB_ENUMERATION_LOOKUP
//-----------------------------------------------------------------------------
// CREATE_DMOIB_ENUMERATOR_LOOKUP
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATOR_LOOKUP(ClassName) \
        IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATOR_LOOKUP_INTERNALS
#define IMPLEMENT_DMOIB_AVAILABLE_SPECIES_CREATE_DMOIB_ENUMERATOR_LOOKUP_INTERNALS(Species) \
        UINT16_TO_UINT8(SpeciesTableEnumEnd), \
        CREATE_DMOIB_ENUMERATOR_LOOKUP Species (END) \
        CREATE_DMOIB_ENUMERATOR_LOOKUP
#define IMPLEMENT_DMOIB_SPECIES_CREATE_DMOIB_ENUMERATOR_LOOKUP(Name, Value) \
        /* Do something for species strings */ \
        UINT16_TO_UINT8(Name), \
        UINT16_TO_UINT8(Name##SymbolicLink), \
        CREATE_DMOIB_ENUMERATOR_LOOKUP

#define IMPLEMENT_DMOIB_MEMBERS_CREATE_DMOIB_ENUMERATOR_LOOKUP(Members) \
        CREATE_DMOIB_ENUMERATOR_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATION_LINKED_INTERNAL_CREATE_DMOIB_ENUMERATOR_LOOKUP(MemberName)  \
        UINT16_TO_UINT8(Enumeration##MemberName##EnumEnd), \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_LOOKUP(Name)  \
        UINT16_TO_UINT8(Enumeration##Name##EnumEnd), \
        IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_LOOKUP_
#define IMPLEMENT_DMOIB_ENUMERATION_CREATE_DMOIB_ENUMERATOR_LOOKUP_(Internals) \
        CREATE_DMOIB_ENUMERATOR_LOOKUP Internals (END) \
        CREATE_DMOIB_ENUMERATOR_LOOKUP

#define IMPLEMENT_DMOIB_ENUMERATOR_INTERNAL_CREATE_DMOIB_ENUMERATOR_LOOKUP(Name, Value) \
        /* Do something for Enumerators */ \
        UINT16_TO_UINT8(Name), \
        UINT16_TO_UINT8(Name##SymbolicLink), \
        CREATE_DMOIB_ENUMERATOR_LOOKUP
//-----------------------------------------------------------------------------


//===============================================
// LINK_LINK_TABLES
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_OBJECT_LINK_LINK_TABLES(Family, Genus, Species, Version, ClassName) \
        { \
            static TableEntry_t TableEntry = {(byte*)ClassName::LookupConstants::LinkTable(), NULL}; \
            *TableIter = &TableEntry; \
            TableIter = (void**)&TableEntry.m_pNext; \
        } \
        IMPLEMENT_DMOIB_OBJECT_LINK_LINK_TABLES_
#define IMPLEMENT_DMOIB_OBJECT_LINK_LINK_TABLES_(Internals) \
        LINK_LINK_TABLES

//===============================================
// StringTable base
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_STRING_TABLE_NO(StringTableName) \
        IMPLEMENT_DMOIB_STRING_TABLE_NO_
#define IMPLEMENT_DMOIB_STRING_TABLE_CONTINUED_NO(StringTableName) \
        IMPLEMENT_DMOIB_STRING_TABLE_NO_
#define IMPLEMENT_DMOIB_STRING_TABLE_NO_(Entries) 

#define IMPLEMENT_DMOIB_STRING_TABLE_YES(StringTableName) \
        namespace StringTableName \
        { \
            void* StringTable(void) \
            { \
        IMPLEMENT_DMOIB_STRING_TABLE_YES_
#define IMPLEMENT_DMOIB_STRING_TABLE_YES_(Entries) \
                static void* TableBase = NULL; \
                DMOIBInitialize(); \
                if(TableBase == NULL) \
                { \
                    static void** TableIter = &TableBase; \
                    LINK_DMOIB_LANGUAGE_TABLES    Entries (END) \
                    *TableIter = NULL; \
                } \
                return TableBase; \
            } \
        }

#define IMPLEMENT_DMOIB_STRING_TABLE_CONTINUED_YES(StringTableName) \
        namespace StringTableName \
        { \
            void* MAKE_UNIQUE_NAME(StringTable)(void* PreviousTable) \
            { \
        IMPLEMENT_DMOIB_STRING_TABLE_CONTINUED_YES_
#define IMPLEMENT_DMOIB_STRING_TABLE_CONTINUED_YES_(Entries) \
                static void* TableBase = NULL; \
                DMOIBInitialize(); \
                if(TableBase == NULL) \
                { \
                    static void** TableIter = &TableBase; \
                    LINK_DMOIB_LANGUAGE_TABLES    Entries (END) \
                    *TableIter = NULL; \
                    if(PreviousTable != NULL) \
                    { \
                        TableIter = &PreviousTable; \
                        while(*(TableIter = (void**)&((*((TableEntry_t**)TableIter))->m_pNext)) != NULL); \
                        *TableIter = TableBase; \
                    } \
                } \
                return TableBase; \
            } \
        }

//===============================================
#define LINK_DMOIB_LANGUAGE_TABLES(x)                 x##_LINK_DMOIB_LANGUAGE_TABLES
#define END_LINK_DMOIB_LANGUAGE_TABLES                DO_NOTHING

//===============================================
#define IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_LINK_DMOIB_LANGUAGE_TABLES(Language) \
        { \
            static TableEntry_t TableEntry = {(byte*)Language##Constants::LanguageTable(), NULL}; \
            *TableIter = &TableEntry; \
            TableIter = (void**)&TableEntry.m_pNext; \
        } \
        IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_LINK_DMOIB_LANGUAGE_TABLES_
#define IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_LINK_DMOIB_LANGUAGE_TABLES_(Strings) \
        LINK_DMOIB_LANGUAGE_TABLES

#define IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_CONTINUED_LINK_DMOIB_LANGUAGE_TABLES(Language) \
        IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_CONTINUED_LINK_DMOIB_LANGUAGE_TABLES_
#define IMPLEMENT_DMOIB_LANGUAGE_TABLE_INTERNAL_CONTINUED_LINK_DMOIB_LANGUAGE_TABLES_(Strings) \
        LINK_DMOIB_LANGUAGE_TABLES

#define IMPLEMENT_DMOIB_STRING_INTERNAL_LINK_DMOIB_LANGUAGE_TABLES(SymbolicLink, Text) \
        LINK_DMOIB_LANGUAGE_TABLES

#define IMPLEMENT_DMOIB_STRING_END_INTERNAL_LINK_DMOIB_LANGUAGE_TABLES() \
        LINK_DMOIB_LANGUAGE_TABLES

//===============================================
// Supported Languages base
//-----------------------------------------------------------------------------
// No mistake here, we simply don't do anything for them in the implementation
// phase.
#define IMPLEMENT_DMOIB_SUPPORTED_LANGUAGES(Languages) 


//===============================================
// LinkTable base
//-----------------------------------------------------------------------------
#define IMPLEMENT_DMOIB_LINK_TABLE_NO(LinkTableName) \
        IMPLEMENT_DMOIB_LINK_TABLE_NO_
#define IMPLEMENT_DMOIB_LINK_TABLE_CONTINUED_NO(LinkTableName) \
        IMPLEMENT_DMOIB_LINK_TABLE_NO_
#define IMPLEMENT_DMOIB_LINK_TABLE_YES(LinkTableName) \
        IMPLEMENT_DMOIB_LINK_TABLE_NO_
#define IMPLEMENT_DMOIB_LINK_TABLE_CONTINUED_YES(LinkTableName) \
        IMPLEMENT_DMOIB_LINK_TABLE_NO_
#define IMPLEMENT_DMOIB_LINK_TABLE_NO_(Entries) 

//===============================================

#endif /* #define DMOIBDEFINER_H */
