/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2005-2006 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// DMOIBBase.cpp  :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "DMOIBBase.h"

/*
===============================================================================
    Function to copy a buffer, byte-for-byte based on the system
    endianess.
===============================================================================
*/
void ByteCopy(byte* pDest, uint16 DestSize, byte* pSource, uint16 SourceSize)
{
    if(pDest != NULL && pSource != NULL)
    {
        while(DestSize-- > 0 && SourceSize-- > 0)
        {
#if (TARGET_ENDIANNESS == LITTLE_ENDIAN)
            *pDest++ = *pSource++;
#else
            pDest[DestSize] = pSource[SourceSize];
#endif
        }
    }
}
/*
===============================================================================
    Function to copy a buffer, byte-for-byte from a system-endian buffer to
    a media-endian buffer. This function is only used if the 2 endianesses
    are different, as it will swap the order of the bytes copied. If the 2
    endianesses are the same, then this function is no generated, and instead
    bytecopy is used.
===============================================================================
*/
#if(((TARGET_ENDIANNESS==LITTLE_ENDIAN) && (MEDIA_ENDIANNES==BIG_ENDIAN)) || \
    ((TARGET_ENDIANNESS==BIG_ENDIAN) && (MEDIA_ENDIANNES==LITTLE_ENDIAN)))
void LineCopy(byte* pDest, uint16 DestSize, byte* pSource, uint16 SourceSize)
{
    if(pDest != NULL && pSource != NULL)
    {
        while(DestSize-- > 0 && SourceSize-- > 0)
        {
            *pDest++ = pSource[SourceSize];
        }
    }
}
#endif

/*
===============================================================================
    Function to compare 2 buffers, byte-for-byte from a system-endian buffer to
    a media-endian buffer. This function will perform the proper byte swapping
    for the 2 endianesses, if necessary. The comparison starts at the MSBs and
    progresses towards the LSBs.
===============================================================================
*/
uint16 LineCheck(byte* pMediaSource, byte* pTargetSource, uint16 Size)
{
    if(pMediaSource != NULL && pTargetSource != NULL)
    {
        while(Size > 0) 
        {
            Size--;
#if (MEDIA_ENDIANNES==LITTLE_ENDIAN)
    #if (TARGET_ENDIANNESS==LITTLE_ENDIAN)
            if(*pMediaSource[Size] != pTargetSource[Size])
    #else // (TARGET_ENDIANNESS==BIG_ENDIAN)
            if(*pMediaSource[Size] != *pTargetSource++))
    #endif
#else // (MEDIA_ENDIANNES==BIG_ENDIAN)
    #if (TARGET_ENDIANNESS==LITTLE_ENDIAN)
            if(*pMediaSource++ != pTargetSource[Size])
    #else // (TARGET_ENDIANNESS==BIG_ENDIAN)
            if(*pMediaSource++ != *pTargetSource++)
    #endif
#endif
            {
                Size++;
                break;
            }
        }
    }
    return Size;
}