<?xml version="1.0" encoding="UTF-8"?>
<!--************************************************************************* 
    '*   This program source file is confidential and proprietary property of
    '*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
    '*                         Copyright (c) 2006 by
    '*                              Welch Allyn Inc.
    '*                            All rights reserved
    '****************************************************************************
    '****************************************************************************
    '**
    '****************************************************************************
    
    '**   XSLT_LINKTABLE.xsl:                                                  
    '**   Version        :   1.00                                           
    '**   Language       :   XSL                                             
    **  Description:  Transforms a WACP (1.8 format) DDS or Family xml file into
    **  a WACP (1.8 format) Link Table xml file.
    
    '************************************************************************* -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="no"/>
    <xsl:template match="/">

            <xsl:choose>
                <xsl:when test="DDS">
                  <xsl:element name="LINK_TABLE">
                    <xsl:for-each select="DDS">
                      <xsl:attribute name="name">
                        <xsl:value-of select="@name"/>
                      </xsl:attribute>
                      <xsl:for-each select="FAMILY">
                        <xsl:for-each select="DEFINITION">
                          <!--Don't create links to CRNDZDStd because it is not produced
                          in the DMOIB Dictionary-->
                          <xsl:if test="@class != 'CRNDZDStd'">
                            <xsl:variable name="genus" select="@genus"/>
                            <xsl:variable name="class" select="@class"/>
                            <xsl:element name="ENTRY">
                              <xsl:attribute name="symboliclink">
                                <xsl:text>STR_</xsl:text>
                                <xsl:value-of select="$class"/>
                              </xsl:attribute>
                              <xsl:element name="LINK">
                                <xsl:attribute name="class">
                                  <xsl:value-of select="$class"/>
                                </xsl:attribute>
                              </xsl:element>
                            </xsl:element>
                            <xsl:for-each select="ENUMERATION">
                              <xsl:variable name="enumeration" select="@name"/>
                              <xsl:for-each select="ENUM">
                                <xsl:variable name="enumerator" select="@name"/>
                                <xsl:element name="ENTRY">
                                  <xsl:attribute name="symboliclink">
                                    <xsl:text>STR_</xsl:text>
                                    <xsl:value-of select="$class"/>
                                    <xsl:text>_</xsl:text>
                                    <xsl:value-of select="$enumeration"/>
                                    <xsl:text>_</xsl:text>
                                    <xsl:value-of select="$enumerator"/>
                                  </xsl:attribute>
                                  <xsl:element name="LINK">
                                    <xsl:attribute name="class">
                                      <xsl:value-of select="$class"/>
                                    </xsl:attribute>
                                    <xsl:attribute name="enumeration">
                                      <xsl:value-of select="$enumeration"/>
                                    </xsl:attribute>
                                    <xsl:attribute name="enumerator">
                                      <xsl:value-of select="$enumerator"/>
                                    </xsl:attribute>
                                  </xsl:element>
                                </xsl:element>
                              </xsl:for-each>
                            </xsl:for-each>
                            <xsl:for-each select="SPECIES_TABLE">
                              <xsl:for-each select="SPECIES">
                                <xsl:variable name="species" select="@name"/>
                                <xsl:element name="ENTRY">
                                  <xsl:attribute name="symboliclink">
                                    <xsl:text>STR_</xsl:text>
                                    <xsl:value-of select="$class"/>
                                    <xsl:text>_</xsl:text>
                                    <xsl:value-of select="$genus"/>
                                    <xsl:text>_</xsl:text>
                                    <xsl:value-of select="@name"/>
                                  </xsl:attribute>
                                  <xsl:element name="LINK">
                                    <xsl:attribute name="class">
                                      <xsl:value-of select="$class"/>
                                    </xsl:attribute>
                                    <xsl:attribute name="species">
                                      <xsl:value-of select="$species"/>
                                    </xsl:attribute>
                                  </xsl:element>
                                </xsl:element>
                              </xsl:for-each>
                            </xsl:for-each>
                            <xsl:for-each select="MEMBERS/MEMBER">
                              <xsl:variable name="member" select="@name"/>
                              <xsl:element name="ENTRY">
                                <xsl:attribute name="symboliclink">
                                  <xsl:text>STR_</xsl:text>
                                  <xsl:value-of select="$class"/>
                                  <xsl:text>_</xsl:text>
                                  <xsl:value-of select="@name"/>
                                </xsl:attribute>
                                <xsl:element name="LINK">
                                  <xsl:attribute name="class">
                                    <xsl:value-of select="$class"/>
                                  </xsl:attribute>
                                  <xsl:attribute name="member">
                                    <xsl:value-of select="$member"/>
                                  </xsl:attribute>
                                </xsl:element>
                              </xsl:element>
                            </xsl:for-each>
                          </xsl:if>
                        </xsl:for-each>
                      </xsl:for-each>
                    </xsl:for-each>
                  </xsl:element>
                </xsl:when>
                <xsl:when test="FAMILY">
                  <xsl:element name="LINK_TABLE">
                    <xsl:attribute name="name">
                      <xsl:value-of select="@name"/>
                    </xsl:attribute>
                    <xsl:for-each select="FAMILY">
                      <xsl:for-each select="DEFINITION">
                        <!--Don't create links to CRNDZDStd because it is not produced
                          in the DMOIB Dictionary-->
                        <xsl:if test="@class != 'CRNDZDStd'">

                          <xsl:variable name="genus" select="@genus"/>
                          <xsl:variable name="class" select="@class"/>
                          <xsl:element name="ENTRY">
                            <xsl:attribute name="symboliclink">
                              <xsl:text>STR_</xsl:text>
                              <xsl:value-of select="$class"/>
                            </xsl:attribute>
                            <xsl:element name="LINK">
                              <xsl:attribute name="class">
                                <xsl:value-of select="$class"/>
                              </xsl:attribute>
                            </xsl:element>
                          </xsl:element>
                          <xsl:for-each select="ENUMERATION">
                            <xsl:variable name="enumeration" select="@name"/>
                            <xsl:for-each select="ENUM">
                              <xsl:variable name="enumerator" select="@name"/>
                              <xsl:element name="ENTRY">
                                <xsl:attribute name="symboliclink">
                                  <xsl:text>STR_</xsl:text>
                                  <xsl:value-of select="$class"/>
                                  <xsl:text>_</xsl:text>
                                  <xsl:value-of select="$enumeration"/>
                                  <xsl:text>_</xsl:text>
                                  <xsl:value-of select="$enumerator"/>
                                </xsl:attribute>
                                <xsl:element name="LINK">
                                  <xsl:attribute name="class">
                                    <xsl:value-of select="$class"/>
                                  </xsl:attribute>
                                  <xsl:attribute name="enumeration">
                                    <xsl:value-of select="$enumeration"/>
                                  </xsl:attribute>
                                  <xsl:attribute name="enumerator">
                                    <xsl:value-of select="$enumerator"/>
                                  </xsl:attribute>
                                </xsl:element>
                              </xsl:element>
                            </xsl:for-each>
                          </xsl:for-each>
                          <xsl:for-each select="SPECIES_TABLE">
                            <xsl:for-each select="SPECIES">
                              <xsl:variable name="species" select="@name"/>
                              <xsl:element name="ENTRY">
                                <xsl:attribute name="symboliclink">
                                  <xsl:value-of select="$class"/>
                                    <xsl:text>_</xsl:text>
                                  <xsl:text>STR_</xsl:text>
                                  <xsl:value-of select="$genus"/>
                                  <xsl:text>_</xsl:text>
                                  <xsl:value-of select="@name"/>
                                </xsl:attribute>
                                <xsl:element name="LINK">
                                  <xsl:attribute name="class">
                                    <xsl:value-of select="$class"/>
                                  </xsl:attribute>
                                  <xsl:attribute name="species">
                                    <xsl:value-of select="$species"/>
                                  </xsl:attribute>
                                </xsl:element>
                              </xsl:element>
                            </xsl:for-each>
                          </xsl:for-each>
                          <xsl:for-each select="MEMBERS/MEMBER">
                            <xsl:variable name="member" select="@name"/>
                            <xsl:element name="ENTRY">
                              <xsl:attribute name="symboliclink">
                                <xsl:text>STR_</xsl:text>
                                <xsl:value-of select="$class"/>
                                <xsl:text>_</xsl:text>
                                <xsl:value-of select="@name"/>
                              </xsl:attribute>
                              <xsl:element name="LINK">
                                <xsl:attribute name="class">
                                  <xsl:value-of select="$class"/>
                                </xsl:attribute>
                                <xsl:attribute name="member">
                                  <xsl:value-of select="$member"/>
                                </xsl:attribute>
                              </xsl:element>
                            </xsl:element>
                          </xsl:for-each>
                        </xsl:if>
                      </xsl:for-each>
                    </xsl:for-each>
                  </xsl:element>
                </xsl:when>
            </xsl:choose>

    </xsl:template>
</xsl:stylesheet>
