/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Main.cpp                                                          //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include <TCHAR.h>
#include "CWACPStack.h"
#include "CFamilyStub.h"
#include "CWACPEvent.h"

#if defined(STACK_SERIAL)
#include "CSerialCommSvr.h"
#endif

#if defined(STACK_ETHERNET)
#include "CUDPCommSvr.h"
#include "CTCPCommSvr.h"
#endif

#if defined(STACK_USB)
#include "CUSBCommSvr.h"
#endif

#include "CFmBODYMASSINDEXStub.h"
#include "CFmDATATRANSFERStub.h"
#include "CFmDEVICEStub.h"
#include "CFmERRORStub.h"
#include "CFmFILESYSStub.h"
#include "CFmFLASHStub.h"
#include "CFmHEIGHTStub.h"
#include "CFmMODULEStub.h"
#include "CFmNUMERICStub.h"
#include "CFmRECORDStub.h"
#include "CFmWEIGHTStub.h"

void TestClientStub();

/* Global Instances */
CWACPStack *pStack = NULL;
CFmBODYMASSINDEXStub *pFmBODYMASSINDEX = NULL;
CFmDATATRANSFERStub *pFmDATATRANSFER = NULL;
CFmDEVICEStub *pFmDEVICE = NULL;
CFmERRORStub *pFmERROR = NULL;
CFmFILESYSStub *pFmFILESYS = NULL;
CFmFLASHStub *pFmFLASH = NULL;
CFmHEIGHTStub *pFmHEIGHT = NULL;
CFmMODULEStub *pFmMODULE = NULL;
CFmNUMERICStub *pFmNUMERIC = NULL;
CFmRECORDStub *pFmRECORD = NULL;
CFmWEIGHTStub *pFmWEIGHT = NULL;

CFamilyStub *pClientStubClass = new CFamilyStub();

/*********************************************************************************
main:
*********************************************************************************/
#if defined(_WIN32_WCE)
#if defined(_WCE_CONSOLE)

int WinEntry (void *Args)
{
    _newprintf = ((formatter*)Args)[0];
    _newscanf = ((formatter*)Args)[1];

#else
int WINAPI WinMain( HINSTANCE   hInstance,
               HINSTANCE   hPrevInstance,
                LPTSTR  lpCmdLine,
                int nCmdShow)
{
#endif
#else
int main(void)
{
#endif /* _WIN32_WCE */

    int TestID = 1;

    while (TestID != 0)
    {
        SYS_PRINTF("WACP Version = %s\n", SYS_VERSION);
        if (TestID == 0)
        {
            break;
        }
        else if (TestID == 1)
        {
            if (pStack == NULL)
                pStack = new CWACPStack(RNDZ_PORT_DEFAULT, WACP_PORT_DEFAULT, THREAD_PRIORITY_HIGHEST, pClientStubClass);

            if (pStack == NULL)
            {
                break;
            }

            pStack->StartClient();

            TestClientStub();

            delete(pStack);
            pStack = NULL;

            pClientStubClass->Reset();
        }
        else
        {
            SYS_PRINTF("\nInvalid Selection\n\n");
        }

        SYS_PRINTF("0. Exit\n");
        SYS_PRINTF("1. Start\n");
        SYS_PRINTF("Enter Selection:");

        scanf("%d", &TestID);
    }

    /*Clean up*/
    if (pClientStubClass)
        delete(pClientStubClass);

    return (0);
}
/*********************************************************************************
TestClientStub:
*********************************************************************************/
void TestClientStub()
{
    uint32   TestID = 1;
    uint32   CmdRqst = 0;
    uint32   Result = TRUE;

    while (pClientStubClass->bExitFlag != TRUE)
    {
        /* Wait until AppInit callback is received */
        if(pClientStubClass->bAppInit)
        {
#if defined(PC)
            /* Create new instances of stub classes once we receive AppInit */
            pFmBODYMASSINDEX = new CFmBODYMASSINDEXStub(pClientStubClass);
            pFmDATATRANSFER = new CFmDATATRANSFERStub(pClientStubClass);
            pFmDEVICE = new CFmDEVICEStub(pClientStubClass);
            pFmERROR = new CFmERRORStub(pClientStubClass);
            pFmFILESYS = new CFmFILESYSStub(pClientStubClass);
            pFmFLASH = new CFmFLASHStub(pClientStubClass);
            pFmHEIGHT = new CFmHEIGHTStub(pClientStubClass);
            pFmMODULE = new CFmMODULEStub(pClientStubClass);
            pFmNUMERIC = new CFmNUMERICStub(pClientStubClass);
            pFmRECORD = new CFmRECORDStub(pClientStubClass);
            pFmWEIGHT = new CFmWEIGHTStub(pClientStubClass);

            uint16 i;
            uint32 ConnectionCount;
            uint32 Data;
            char   *pSourceID = NULL; // Fill this in with the current connection SourceID
            ConnArgs **Config = NULL;

            SYS_PRINTF("Enter number of connections: 0..255 (0 to exit)");
            Result = (uint16)scanf("%d", &ConnectionCount);
            ConnectionCount %= 256;
            Config = (ConnArgs**)SYS_MALLOC(ConnectionCount * sizeof(ConnArgs*));
            for (i=0; i < ConnectionCount; i++)
            {
                int StackType;
                SYS_PRINTF("\n0. Serial\n");
                SYS_PRINTF("1. TCP/UDP\n");
                SYS_PRINTF("2. USB\n");
                Result = (uint16)scanf("%d", &StackType);
                StackType++;

                if (StackType == SERIAL)
                {
                   /* Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->ConfigSize = sizeof(SerialAddr);
                    Config[i]->pConfig = (void*)SYS_CALLOC(Config[i]->ConfigSize, 1);
                    Config[i]->StackType = SERIAL;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;

                    SYS_PRINTF("1. Start up Host\n");
                    SYS_PRINTF("2. Start up Client\n");
                    SYS_PRINTF("Enter Selection:");
                    scanf("%d", &Data);

                    if (Data == 1)
                    {
                        Config[i]->Timeout = INT32_MAX;
                        SYS_PRINTF("\n0. Disable DTR\n");
                        SYS_PRINTF("1. Enable DTR\n");
                        SYS_PRINTF("Enter Selection:");
                        scanf("%d", &Data);
                        ((SerialAddr*)(Config[i]->pConfig))->bAutoDiscovery = (boolean)Data;
                    }
                    else
                    {
                        Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    }

                    SYS_PRINTF("\nCOM[Baudrate,Paritybit,Databit,Stopbit] \n(ex. \"COM1\" Defaults to COM1:115200,N,8,1)");
                    SYS_PRINTF("\nEnter COM Source:");
                    scanf("%s", Config[i]->SourceID);*/

                }
                else if (StackType == ETHERNET)
                {
                    uint16 Protocol = 0;
                    char   SourceID[ADDR_BUF_SIZE];

                    memset(SourceID, 0, sizeof(SourceID));

                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->ConfigSize = 0;
                    Config[i]->pConfig = NULL;
                    Config[i]->StackType = ETHERNET;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    SYS_PRINTF("\n1. TCP\n");
                    SYS_PRINTF("2. UDP\n");
                    SYS_PRINTF("Enter Protocol:");
                    scanf("%d", &Data);

                    if (Data == 2)
                    {
                        Protocol = IPPROTO_UDP;
                    }
                    else
                    {
                        Protocol = IPPROTO_TCP;
                    }

                    SYS_PRINTF("\n1. Server\n");
                    SYS_PRINTF("2. Client\n");
                    SYS_PRINTF("Enter Type:");
                    scanf("%d", &Data);

                    /* Default to Client unless Server is specified */
                    if (Data == 1)
                    {
                        Config[i]->bSvrFlag = TRUE;

                        SYS_PRINTF("\nEnter IP:Port (ex. 10.11.12.13:281) (0 for default *:%d):", RNDZ_PORT_DEFAULT);
                        scanf("%s", SourceID);

                        /* if "0" default to loopback*/
                        if (strcmp(SourceID,"0") == 0)
                        {
                            if (Protocol == IPPROTO_UDP)
                            {
                                /* ETHERNET UDP Server */
                                sprintf(Config[i]->SourceID, "UDP:*:%d", RNDZ_PORT_DEFAULT);
                            }
                            else
                            {
                                /* ETHERNET TCP Server */
                                sprintf(Config[i]->SourceID, "TCP:*:%d", RNDZ_PORT_DEFAULT);
                            }
                        }
                        else
                        {
                            if (Protocol == IPPROTO_UDP)
                            {
                                /* ETHERNET UDP Server */
                                sprintf(Config[i]->SourceID, "UDP:%s", SourceID);
                            }
                            else
                            {
                                /* ETHERNET TCP Server */
                                sprintf(Config[i]->SourceID, "TCP:%s", SourceID);
                            }
                        }
                    }
                    else
                    {
                        Config[i]->bSvrFlag = FALSE;
                        SYS_PRINTF("\nEnter IP:Port (ex. 10.11.12.13:281) (0 for default 127.0.0.1:%d):", RNDZ_PORT_DEFAULT);
                        scanf("%s", SourceID);

                        /* if "0" default to loopback*/
                        if (strcmp(SourceID,"0") == 0)
                        {
                            /* ETHERNET TEST */
                            if (Protocol == IPPROTO_UDP)
                            {
                                /* ETHERNET UDP Server */
                                sprintf(Config[i]->SourceID, "UDP:127.0.0.%d:%d", i + 1, RNDZ_PORT_DEFAULT);
                            }
                            else
                            {
                                /* ETHERNET TCP Server */
                                sprintf(Config[i]->SourceID, "TCP:127.0.0.%d:%d", i + 1, RNDZ_PORT_DEFAULT);
                            }
                        }
                        else
                        {
                            if (Protocol == IPPROTO_UDP)
                            {
                                /* ETHERNET UDP Server */
                                sprintf(Config[i]->SourceID, "UDP:%s", SourceID);
                            }
                            else
                            {
                                /* ETHERNET TCP Server */
                                sprintf(Config[i]->SourceID, "TCP:%s", SourceID);
                            }
                        }
                    }
                }/* else if (StackType == ETHERNET) */
                else if (StackType == USB)
                {
                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->ConfigSize = 0;
                    Config[i]->pConfig = NULL;          /* No configuration structure */
                    Config[i]->StackType = USB;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->bSvrFlag = TRUE;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    Config[i]->bSvrFlag = TRUE;

                    /* USB TEST */
                    sprintf(Config[i]->SourceID, "%s", "USB");
                }
            }/* for (i=0; i < ConnectionCount; i++) */

            /* Open up the communications channels */
            Result = pClientStubClass->Open(Config, (uint8)ConnectionCount);
            if (Result)
            {
                /* Cycle through the connection list and connect eac client */
                for (i=0; i<ConnectionCount; i++)
                {
                    if (Config[i]->bSvrFlag == FALSE)
                    {
                        Result = pClientStubClass->Connect(Config[i]->SourceID);
                        if (!Result)
                        {
                            pClientStubClass->bExitFlag = TRUE;
                            SYS_PRINTF("Unable to connect to %s\n", Config[i]->SourceID);

                        }
                        /* Set the context via the SourceID */
                        pSourceID = Config[0]->SourceID;
                    }
                }
            }
            else
            {
                pClientStubClass->bExitFlag = TRUE;
            }


            while (TestID != 0)
            {
                if(!pClientStubClass->bExitFlag)
                {
                    SYS_PRINTF("\n0. Exit\n");
                    SYS_PRINTF("1. Show Connections\n");
                    SYS_PRINTF("2. Set Current Connection\n");
                    SYS_PRINTF("3. Set Error Configuration\n");
                    SYS_PRINTF("4. Set KeepAlive Configuration\n");
                    SYS_PRINTF("5. Get GUIDObj\n");
                    SYS_PRINTF("6. Get GUID\n");
                    SYS_PRINTF("7. Get ByPass Connection List\n");
                    SYS_PRINTF("8. Send ByPass Start Command\n");
                    SYS_PRINTF("9. Send ByPass End Command\n");
                    SYS_PRINTF("10. Custom Send\n");
                    SYS_PRINTF("11. Disconnect connection\n");
                    SYS_PRINTF("12. FmBODYMASSINDEX Test\n");
                    SYS_PRINTF("13. FmDATATRANSFER Test\n");
                    SYS_PRINTF("14. FmDEVICE Test\n");
                    SYS_PRINTF("15. FmERROR Test\n");
                    SYS_PRINTF("16. FmFILESYS Test\n");
                    SYS_PRINTF("17. FmFLASH Test\n");
                    SYS_PRINTF("18. FmHEIGHT Test\n");
                    SYS_PRINTF("19. FmMODULE Test\n");
                    SYS_PRINTF("20. FmNUMERIC Test\n");
                    SYS_PRINTF("21. FmRECORD Test\n");
                    SYS_PRINTF("22. FmWEIGHT Test\n");
                    SYS_PRINTF("Enter selection: ");

                    Result = (uint16)scanf("%d", &TestID);
                    if(pClientStubClass->bExitFlag)
                        TestID = 0;

                    if (pSourceID == NULL)
                    {
                        if (pClientStubClass->GetNumConnections())
                        {
                            CONNECTION_LIST::iterator pList;

                            pList = pClientStubClass->SourceList.begin();
                            pSourceID = (*pList)->SourceID;
                        }
                    }
                }
                else
                {
                    Result = 1;
                    TestID = 0;
                }

                if(Result == 1)
                {
                    switch(TestID)
                    {
                        case 0: /* Exit */
                        {
                            CONNECTION_LIST::iterator pList;
                            uint32 i = 0;

                            /* Close server created connections */
                            pClientStubClass->TerminateAll();

                            /* Close User created connections */
                            for (i=0; i < ConnectionCount; i++)
                            {
                                pSourceID = Config[i]->SourceID;
                                SYS_PRINTF("Disconnecting Server Connections %s\n", pSourceID);
                                pClientStubClass->Close(pSourceID);
                                pClientStubClass->RemoveDevice(pSourceID);
                            }

                            SYS_PRINTF("0. Exit\n");
                            SYS_PRINTF("1. Connect\n");
                            Result = (uint16)scanf("%d", &TestID);

                            if(TestID == 1)
                            {
                                pClientStubClass->bExitFlag = FALSE;

                                /* Open up the communications channels */
                                Result = pClientStubClass->Open(Config, (uint8)ConnectionCount);

                                if (Result)
                                {
                                    /* Cycle through the connection list and connect eac client */
                                    for (i=0; i<ConnectionCount; i++)
                                    {
                                        /* Only connect if we are a client. A host device will not get into AppInit until
                                        * the connection is established
                                        */
                                        if(Config[i]->bSvrFlag == FALSE)
                                        {
                                            Result = pClientStubClass->Connect(Config[i]->SourceID);
                                            if (!Result)
                                            {
                                                pClientStubClass->bExitFlag = TRUE;
                                                SYS_PRINTF("Unable to connect to %s\n", Config[i]->SourceID);
                                            }
                                        }
                                    }
                                }
                                else
                                    break;
                            }
                            else
                            {
                                pClientStubClass->bExitFlag = TRUE;
                                break;
                            }
                            break;
                        }
                        case 1:
                        {
                            pClientStubClass->ShowDevices();
                            break;
                        }
                        case 2:
                        {
                            int CurrentConfig;

                            SYS_PRINTF("\t0. Exit\n");
                            pClientStubClass->ShowDevices();
                            SYS_PRINTF("\nEnter Selection: ");
                            scanf("%d", &CurrentConfig);

                            if (CurrentConfig-- == 0)
                                break;

                            if ((uint8)CurrentConfig < pClientStubClass->GetNumConnections())
                            {
                                CONNECTION_LIST::iterator pList;
                                int i = 0;

                                /* Close server created connections */
                                for (pList = pClientStubClass->SourceList.begin(), i=0; 
                                     pList != pClientStubClass->SourceList.end(); 
                                     pList++, i++)
                                {
                                    if (CurrentConfig == i)
                                    {
                                        pSourceID = (*pList)->SourceID;
                                    }
                                }

                                SYS_PRINTF( "Current session set to %s\n", pSourceID);
                            }
                            else
                                SYS_PRINTF("Invalid selection\n");
                            break;
                        }
                        case 3:
                        {
                            SYS_PRINTF("\t1. Get Last Error\n");
                            SYS_PRINTF("\t2. Send CERR_SpAPPINTERNAL error message\n");
                            SYS_PRINTF("\nEnter Configuration: ");

                            Result = (uint16)scanf("%d", &CmdRqst);
                            if(pClientStubClass->bExitFlag)
                                continue;

                            if(Result == 1)
                            {
                                switch(CmdRqst)
                                {
                                    case 1:
                                    {
                                        pClientStubClass->SendMsg(pSourceID, FmTRAP, GnREQUEST, TRAP_CRST_SpGET_LASTERROR, NULL, 0, FALSE);
                                        break;
                                    }
                                    case 2:
                                    {
/*
                                        uint32  ClassID = MakeClassID(FmARCH, GnDATA, 0);
                                        char    TempString[] = "Testing AppInternal Error";

                                        pClientStubClass->SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRAPPINTERNAL, CERR_APP_NULL, (uint8*)TempString, sizeof(TempString));
*/
                                        break;
                                    }
                                }
                            }
                            break;
                        }
                        case 4:
                        {
                            time_t Interval, TimeOut;
                            CONN_CFG_SPECIES Options;

                            SYS_PRINTF("\n\t0. Exit\n");
                            SYS_PRINTF("\t1. SET options\n");
                            SYS_PRINTF("\t2. GET options\n");
                            SYS_PRINTF("\nEnter Configuration: ");

                            scanf("%d", &Result);
                            switch(Result)
                            {
                                case 0:
                                    break;
                                case 1:
                                {
                                    SYS_PRINTF("\n\t0. CONN_CFG_SpKEEPALIVEON\n");
                                    SYS_PRINTF("\t1. CONN_CFG_SpKEEPALIVEOFF\n");
                                    SYS_PRINTF("\nEnter Configuration: ");

                                    Result = (uint16)scanf("%d", &CmdRqst);
                                    Options = (CONN_CFG_SPECIES)CmdRqst;
                                    if(pClientStubClass->bExitFlag)
                                        continue;

                                    if(Result == 1)
                                    {
                                        if(CmdRqst <= 1)
                                        {
                                            SYS_PRINTF("\nEnter KeepAlive Interval (0 for default): ");
                                            Result = (uint16)scanf("%d", &CmdRqst);
                                            Interval = CmdRqst;
                                            if(pClientStubClass->bExitFlag)
                                                continue;

                                            SYS_PRINTF("\nEnter KeepAlive Timeout (0 for default): ");
                                            Result = (uint16)scanf("%d", &CmdRqst);
                                            TimeOut = CmdRqst;
                                            if(pClientStubClass->bExitFlag)
                                                continue;

                                            if (Interval == 0)
                                                Interval = DEFAULT_KEEPALIVE_INTERVAL;

                                            if (TimeOut == 0)
                                                TimeOut = DEFAULT_TIMEOUT_INTERVAL;

                                            pClientStubClass->SetConnOptions(pSourceID, Options, TRUE, Interval, TimeOut);
                                        }
                                    }
                                    break;
                                }
                                case 2:
                                {
                                    pClientStubClass->GetConnOptions(pSourceID, &Options, &Interval, &TimeOut);
                                    SYS_PRINTF("Option = %d\n", Options);
                                    SYS_PRINTF("Timeout = %d\n", TimeOut);
                                    SYS_PRINTF("Interval = %d\n", Interval);
                                    break;
                                }
                                default:
                                    SYS_PRINTF("Invalid Selection\n");
                                    break;
                            }
                            break;
                        }
                        case 5:
                        {
#if((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
                            CRNDZDStd* pGUIDObj = pClientStubClass->GetGUIDObj(pSourceID);
                            uint32 XMLSize = pGUIDObj->size_XML();
                            char* pTemp = (char*)SYS_MALLOC(XMLSize);
                            if (pTemp != NULL)
                            {
                                pGUIDObj->serialize_to_XMLbuf(pTemp);
                                SYS_PRINTF("\n%s\n", pTemp);
                                SYS_FREE(pTemp);
                            }
#else
                            SYS_PRINTF("XML is not supported\n");
#endif
                            break;
                        }
                        case 6:
                        {
                            uint16 Size = 0;
                            uint8* pGUID = pClientStubClass->GetGUID(pSourceID, &Size);
                            char   GUIDBuf[64];

                            if(pGUID)
                            {
                                GUIDToStr(pGUID, GUIDBuf, 32);
                                SYS_PRINTF("\n%s\n\n", GUIDBuf);
                            }
                            else
                            {
                                SYS_PRINTF("NO GUID available\n");
                            }
                            break;
                        }
                        case 7:
                        {
                            CWACPEvent* pWaitEvent = new CWACPEvent(_T("ByPass"), FALSE);
                            if (pWaitEvent)
                            {
                                pClientStubClass->SendMsg(pSourceID, FmCONNECTION, GnREQUEST, CONN_CRST_SpGETCONNLIST, NULL, 0);

                                if (pWaitEvent->Wait(3000) == WAIT_SUCCESS)
                                {
                                    SYS_PRINTF("\n\n");
                                    if (pClientStubClass->ConnCount > 0)
                                    {
                                        for (uint16 i=0; i < pClientStubClass->ConnCount; i++)
                                        {
                                            uint8*    pGUID = NULL;
                                            char      GUIDBuf[64];
                                            uint16    GUIDSize = 0;
                                            pGUID = pClientStubClass->ConnDArray[i]->GetDeviceGUID(&GUIDSize);
                                            GUIDToStr(pGUID, GUIDBuf, GUIDSize);

                                            SYS_PRINTF("%d. SourceID: %s\nGUID: %s\n", i, pClientStubClass->ConnDArray[i]->GetSourceID(NULL), GUIDBuf);
                                        }
                                    }

                                    SYS_PRINTF("\n\n");
                                }
                                else
                                {
                                    SYS_PRINTF("Unable to get connection list in time\n");
                                }
                                delete(pWaitEvent);
                            }
                            break;
                        }
                        case 8:
                        {
                            if (pClientStubClass->ConnCount)
                            {
                                for (uint16 i=0; i < pClientStubClass->ConnCount; i++)
                                {
                                    uint8*    pGUID = NULL;
                                    char      GUIDBuf[64];
                                    uint16    GUIDSize = 0;
                                    pGUID = pClientStubClass->ConnDArray[i]->GetDeviceGUID(&GUIDSize);
                                    GUIDToStr(pGUID, GUIDBuf, GUIDSize);

                                    SYS_PRINTF("%d. SourceID: %s\nGUID: %s\n", i, pClientStubClass->ConnDArray[i]->GetSourceID(NULL), GUIDBuf);
                                }

                                /* Get user input */
                                SYS_PRINTF("\nSelect Device to create a ByPass connection:");
                                Result = (uint16)scanf("%d", &CmdRqst);

                                if (CmdRqst < pClientStubClass->ConnCount)
                                {
                                    uint32 Size = 0;
                                    uint8* pTemp = NULL;

                                    Size = pClientStubClass->ConnDArray[CmdRqst]->size_all();
                                    pTemp = (uint8*)SYS_CALLOC(Size, 1);
                                    if (pTemp)
                                    {
                                        Result = pClientStubClass->ConnDArray[CmdRqst]->serialize_all(pTemp);
                                        if (Result)
                                        {
                                            /* Send the Start ByPass Message */
                                            pClientStubClass->SendMsg(pSourceID, FmCONNECTION, GnCOMMAND, CONN_CMD_SpSTARTBYPASS, 
                                                (uint8*)pTemp, Size);
                                        }
                                        SYS_FREE(pTemp);
                                    }
                                }
                            }
                            else
                                SYS_PRINTF("\nNo connection list.\n");
                            break;
                        }
                        case 9:
                        {
                            pClientStubClass->SendMsg(pSourceID, FmCONNECTION, GnCOMMAND, CONN_CMD_SpENDBYPASS, NULL, 0);
                            break;
                        }
                        case 10:
                        {
                            int Family;
                            int Genus;
                            int Species;

                            SYS_PRINTF("Family: ");
                            scanf("%d", &Family);

                            SYS_PRINTF("Genus: ");
                            scanf("%d", &Genus);

                            SYS_PRINTF("Species: ");
                            scanf("%d", &Species);

                            pClientStubClass->SendMsg(pSourceID, (uint16)Family, (uint8)Genus, (uint8)Species, NULL, 0);
                            break;
                        }
                        case 11:
                        {
                            int CurrentConfig;

                            SYS_PRINTF("\t0. Exit\n");
                            pClientStubClass->ShowDevices();
                            SYS_PRINTF("\nEnter Connection to close: ");
                            scanf("%d", &CurrentConfig);

                            if (CurrentConfig-- == 0)
                                break;

                            if ((uint8)CurrentConfig < pClientStubClass->GetNumConnections())
                            {
                                CONNECTION_LIST::iterator pList;
                                int i = 0;

                                /* Close server created connections */
                                for (pList = pClientStubClass->SourceList.begin(), i=0; 
                                     pList != pClientStubClass->SourceList.end(); 
                                     pList++, i++)
                                {
                                    if (CurrentConfig == i)
                                    {
                                        pSourceID = (*pList)->SourceID;
                                        pClientStubClass->Close(pSourceID);
                                        pClientStubClass->RemoveDevice(pSourceID);
                                        break;
                                    }
                                }
                            }
                            else
                                SYS_PRINTF("Invalid selection\n");

                            break;
                        }
                        case 12:
                        {
                            pFmBODYMASSINDEX->AppInit(pSourceID);
                            break;
                        }
                        case 13:
                        {
                            pFmDATATRANSFER->AppInit(pSourceID);
                            break;
                        }
                        case 14:
                        {
                            pFmDEVICE->AppInit(pSourceID);
                            break;
                        }
                        case 15:
                        {
                            pFmERROR->AppInit(pSourceID);
                            break;
                        }
                        case 16:
                        {
                            pFmFILESYS->AppInit(pSourceID);
                            break;
                        }
                        case 17:
                        {
                            pFmFLASH->AppInit(pSourceID);
                            break;
                        }
                        case 18:
                        {
                            pFmHEIGHT->AppInit(pSourceID);
                            break;
                        }
                        case 19:
                        {
                            pFmMODULE->AppInit(pSourceID);
                            break;
                        }
                        case 20:
                        {
                            pFmNUMERIC->AppInit(pSourceID);
                            break;
                        }
                        case 21:
                        {
                            pFmRECORD->AppInit(pSourceID);
                            break;
                        }
                        case 22:
                        {
                            pFmWEIGHT->AppInit(pSourceID);
                            break;
                        }
                        default:
                        {
                            SYS_PRINTF("\n\nInvalid Selection.\n\n");
                            break;
                        }
                        break;
                    }/* switch(TestID) */
                }/* if(Result == 1) */
            }/* while (TestID != 0) */

            SYS_SLEEP(1);

            for (i=0; i<ConnectionCount; i++)
            {
                SYS_FREE(Config[i]->pConfig);
                SYS_FREE(Config[i]);
            }

            SYS_FREE(Config);

            delete(pFmBODYMASSINDEX);
            delete(pFmDATATRANSFER);
            delete(pFmDEVICE);
            delete(pFmERROR);
            delete(pFmFILESYS);
            delete(pFmFLASH);
            delete(pFmHEIGHT);
            delete(pFmMODULE);
            delete(pFmNUMERIC);
            delete(pFmRECORD);
            delete(pFmWEIGHT);

#else
            pClientStubClass->bExitFlag = FALSE;

            /* Loop until the connection is closed */
            if(Result)
            {
                while(pClientStubClass->bExitFlag == FALSE)
                    SYS_SLEEP(WAIT_COUNT);

                /* Close the WACP session(s) */
                /* pWACPApp->Close(pSourceID); */
            }
#endif  /*  #if defined(PC) */
        
        }/* if(pClientStubClass->bAppInit) */
        else
        {
            SYS_SLEEP(100);
        }
    }/* while (pClientStubClass->bExitFlag != TRUE) */
}

/* End of File */

