/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CBaseCommSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CBASECOMMSVR_H)
    #define   CBASECOMMSVR_H

#include "Target.h"
#include "Common.h"
#include "CCommSvr.h"

#define PARSEREX_MIN_BUFSIZE 512

class CBaseCommSvr
{
/*================================================================================
CBaseCommSvr Members: Public, Private, and Protected
================================================================================*/

    public:
    /* Constructor - Takes in two function pointers for pushing new connections 
     * and new messages up to the calling application 
     */
    CBaseCommSvr();
    virtual ~CBaseCommSvr();
    
    /* Initializes the callback functions and semaphore names */
    void Init(SetRecvBufPtr RecvMsgFunc,                   
              DevArrPtr DeviceArrivalFunc,
              DevRemPtr DeviceRemovalFunc,
              char* pAppName);

    /* Sets the SourceID of the CBaseCommSvr. */
    boolean  SetSourceID(char* pSourceID);
    
    /* Returns the SourceID of the CBaseCommSvr. The SourceID is the unique
     * identifier for the communications session. In Ethernet the SourceID 
     * would hold the destination IP Address of the connection peer. In 
     * RS232 this would hold the Name of the com Port through which */
    char* GetSourceID();

    /* Sets the communication port that distinguishes between WACP and RNDZ messages */
    boolean SetPortNum(uint16 NewPort);

    /* Returns the Port which represents either a RNDZ or WACP session */
    uint16 GetPortNum();

    /* Returns the type of stack that is being used (i.e. Ethernet, 
     * RS232, Bluetooth, Zigby, etc
     */
    STACK_TYPE GetStackType();

    /* Creates a connection */
    virtual boolean StartReader(uint32 (WACPAPI* StartAddress)(void*), void* pArgList, 
                                 int32 ThreadPriority, char* pName);

    /* Handles incoming connection requests */
    virtual uint32 StartServer() {return 1;}

    /* Sets up the configuration of the stack based on the data received 
     * via pAddr.
     */
    virtual void SetConfig(void* pAddr) = 0;

    /* Sends a binary Buffer */
    virtual boolean SendMsg(uint8* pBuf, uint32 BufSize) = 0;

    /* Creates a connection */
    virtual boolean Connect(uint8* pSourceID, uint16 PortNum) = 0;

    protected:
        uint32          WACPParser(uint8* pBuf, uint32 BufSize, uint32* pPacketLength);
        uint32          WACPParserEx(uint8* pBuf, uint32 BufSize, uint32* pPacketLengthHint);
        boolean         Arrival(char* pSourceID, STACK_TYPE StackType, void* pConfig, uint16 ConfigSize);
        boolean         Removal();

        SetRecvBufPtr   pSetRecvMsg;
        DevArrPtr       pDeviceArrival;
        DevRemPtr       pDeviceRemoval;
        char            SourceID[ADDR_BUF_SIZE];
        char*           pSemName;
        uint16          Port;
        STACK_TYPE      StackType;
        CWACPThread     RThread;
private:
    uint8* pParseBuf;
    uint32 ParseBufSize;
    uint32 ParseState;
};


#endif /* #if !defined (CBASECOMMSVR_H) */

/*---------------------------------------------------*/
/* End of File */





