/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmRouter.cpp  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CFmRouter.h"
#include "SysUtil.h"
#if defined(TRACE_SERVER_ON)
#include "CTraceSvr.h"
#endif
/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
*********************************************************************************/
CFmRouter::CFmRouter()
{      
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    /* Initialize the MsgCounts */
    MsgInCount = 0;
    MsgOutCount = 0;
#endif
}

CFmRouter::~CFmRouter()
{
    /* Initialize the array list to NULL (0) */
    memset(ObjSvrList, 0, sizeof(ObjSvrList));
}

void CFmRouter::Init(CTrapObjSvr *pCTrapObjSvr,
                     void *pCWACPApp)
{       
    /* Initialize the array list to NULL (0) */
    memset(ObjSvrList, 0, sizeof(ObjSvrList));

    /* Get a reference to the CTrapObjSvr instance for sending error messages */
    pTrapSvr = pCTrapObjSvr;

    /* Get a reference to the CWACPApp instance */
    pWACPApp = pCWACPApp;
}

/*================================================================================
FmRouter Register object servers - The valid object servers are listed in the
    enumerated type OBJSVR.
================================================================================*/
boolean CFmRouter::RegisterServer(FAMILY Family, CObjSvr* pObjSvr)
{
    boolean Result=TRUE;
    OBJSVR  Index;

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

    ObjSvrList[Index] = pObjSvr;

    return Result;
}

/*================================================================================
FmRouter Command, Response, and Request Routing for outgoing messages
================================================================================*/
boolean CFmRouter::SendMsg(char *pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8 *pBuffer, uint32 Size, boolean EncryptionOn)
{
    boolean RetVal = FALSE;
    OBJSVR  Index;

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    char tempString[] = {"MsgOutCount %d\nMessage Data:\n%A"};
    SYS_TRACE(TRACE_Type_SEND,
        "CFmRouter", "SendMsg",
        Family, Genus, Species, 
        tempString, ++MsgOutCount, pBuffer, Size);
#endif

    if (ObjSvrList[Index])
    {
        /* Call the messagehandler for the object server */
        RetVal = (ObjSvrList[Index])->SendMsgHandler(pSourceID, Family, Genus, Species, pBuffer, Size, EncryptionOn);
    }

    return RetVal;
}
/*================================================================================
FmRouter Command, Response, and Request Routing for incoming messages
================================================================================*/
boolean CFmRouter::RecvMsg(char *pSourceID, uint8 *pBuffer, uint32 Size)
{
    uint16  Family;
    boolean RetVal = TRUE;
    OBJSVR  Index;

    TOUCH(Size);

    /* Extract the Family */
    Family = GetFamily(pBuffer);

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

    if (ObjSvrList[Index])
    {
        /* Call the messagehandler for the object server */
        (ObjSvrList[Index])->RecvMsgHandler(pSourceID, pBuffer, pWACPApp);
    }

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    uint8   Genus;
    uint8   Species;
    
    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);

    char tempString[] = {"MsgInCount %d\nMessage Data:\n%A"};
    SYS_TRACE(TRACE_Type_RECEIVE,
        "CFmRouter", "RecvMsg",
        Family, Genus, Species, 
        tempString, ++MsgInCount, pBuffer, Size);
#endif

    return RetVal;
}

/* End of File */







