/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CInterchange.cpp:                                                 //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CInterchange.h"

#if defined(TRACE_SERVER_ON)
#include "CTraceSvr.h"
#endif

DLLStruct* CInterchange::ExcgTable[MAX_DLL_COUNT];
uint16    CInterchange::DLLCount;

/* Faked Termiante message */
const uint8 WacpTerminate[] = 
{   
    0x00,0x1E,0x01,0x00,0x00,
    0x00,0x00,0x07,0x00,0x00,
    0x00,0x00,0x00,0x61,0xAD
};

/* Faked CRC Error message */
const uint8 CRCError[] = 
{   
    0x00, 0x1c, 0x03, 0x04, /* CErrSvr ClassID */
    0x00, 0x35,             /* CErrSvr Payload Size */
    0x00, 0x04,             /* CErrSvr Static Struct Size */
    0x00, 0x00, 0x00, 0x00, /* CErrSvr Error ClassID */ 
    0x00, 0x00, 0x00, 0x29, /* CErrSvr Data Size */
          
        0x00, 0x72, 0x03, 0x01, /* CErrEStd ClassID */
        0x00, 0x23,             /* CErrEStd Payload Size */
        0x00, 0x64,             /* CErrEStd Version */
        0xc0,                   /* CErrEStd DataByte */
        0x00, 0x1a,             /* CErrEStd Static Size */
        0x00, 0x00, 0x00, 0x00, /* CErrEStd Error ClassID */
        0x00, 0x05,             /* CErrEStd Errorcode */
        0x43, 0x52, 0x43, 0x20, 0x4d, 0x69, 0x73, 0x6d, 0x61, 0x74, 
        0x63, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
        0x00, 0x00,             /* CErrEStd Error Description */
        0x2b, 0x06,             /* CErrEStd CRC*/

    0x46, 0x29              /* CErrSvr CRC*/
};


/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
CInterchange Start up: 
*********************************************************************************/

CInterchange::CInterchange()
{
}

CInterchange::~CInterchange()
{
    MESSAGE       MsgStruct;

    Close();

    /* Cleanup any remaining messages from the pINQMgr */
    while (pINQMgr->get_num_used() > 0)
    {
        /* Get the first packet Stream in the queue */
        pINQMgr->dequeue_array((uint8*)&MsgStruct, sizeof(MESSAGE));

        /* Free the DataBuffer allocated by RecvMsg()*/
        SYS_FREE(MsgStruct.pDataBuf);

    } /* if (pINQMgr->get_num_elements() > 0) */     

    /* Destroy the queue manager */
    delete(pINQMgr);

    if (pCommSvr != NULL)
    {
        delete(pCommSvr);
        pCommSvr = NULL;
    }

    /* Free the Queue buffers */
    SYS_FREE(pINQueueBuf);

#if defined(DEBUG_PRINT)
    SYS_PRINTF("Removing %p from ExcgTable DLLCount = %d\n", ExcgTable[DLLCount], DLLCount);
#endif

    /* Decrement the DLLCount to track the number of CInterchange class instances */
    DLLCount--;

#if defined(DEBUG_PRINT)
    for (uint16 i=0; i < DLLCount; i++)
    {
        SYS_PRINTF("ExcgTable[%d]->pExchange = %p\n", i, ExcgTable[i]->pExchange);
        SYS_PRINTF("ExcgTable[%d]->pAppName = %s\n", i, ExcgTable[i]->pAppName);
    }
#endif

    /* Delete the WACP Application Layer. This deletes 
     * the WACP Presentation Layer as well.
     */
    delete(pCWACPApp);

    pCWACPApp = NULL;

    /* Delete the CCommSvr if it still exists */
    if (pCommSvr)
    {
        delete(pCommSvr);
        pCommSvr =  NULL;
    }

}

void CInterchange::Init(uint16 RNDZPortNum, uint16 WACPPortNum, int32 Priority, void* pCallBackObj)
{
    /* Initialize the CWACPApp pointer */
    pCWACPApp = NULL;

    /* Set the Port values for RNDZ and WACP */
    RNDZPort = RNDZPortNum;
    WACPPort = WACPPortNum;

    /* Allocate space for the IN queue */
    pINQueueBuf = (uint8*)SYS_MALLOC(MAX_RECV_QUEUE_ITEMS * sizeof(MESSAGE));

    /* Create the queue managers for the IN queue */
    pINQMgr   = new uint8QManager(pINQueueBuf, MAX_RECV_QUEUE_ITEMS * sizeof(MESSAGE));

    /* Create the semaphores for signaling data in the IN queue */
    pRecvSemaphore = new CWACPSemaphore(NULL, 0, MAX_RECV_QUEUE_ITEMS);

    /* Open the named handle and signal the Semaphore */
    memset(AppSemName, 0, sizeof(AppSemName));
    sprintf(AppSemName, WACP_SEM_NAME, SYS_GETPID(), this);

    /* Set this to TRUE before starting the Recv Threads */
    ThreadStatus = TRUE;

    /* Store stack settings */
    Args.ThreadPriority = Priority;
    Args.pCallBackObj = pCallBackObj;

    /* Start the Receiving Thread */
    RecvThread.StartThread(this->ReceivingThread, this, Args.ThreadPriority, "CInterchange::ReceivingThread");

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    MsgInCount = 0;
    MsgOutCount = 0;
#endif

    /* Setup the ExcgStruct structure with the information on this CInterchange instance */
    ExcgStruct.pAppName = AppSemName;
    ExcgStruct.pExchange = this;

    /* Initialize this to NULL until StartThreads fills it in */
    ExcgStruct.pSourceID = NULL;

#if defined(DEBUG_PRINT)
    SYS_PRINTF("Adding %p from ExcgTable DLLCount = %d\n", this, DLLCount);
#endif

    /* Add this instance to the ExcgTable to track each CInterchange class instance */
    ExcgTable[DLLCount] = &ExcgStruct;

    /* Increment the DLLCount to track the number of CInterchange class instances */
    DLLCount++;

#if defined(DEBUG_PRINT)
    for (uint16 i=0; i < DLLCount; i++)
    {
        SYS_PRINTF("ExcgTable[%d]->pExchange = %p\n", i, ExcgTable[i]->pExchange);
        SYS_PRINTF("ExcgTable[%d]->pAppName = %s\n", i, ExcgTable[i]->pAppName);
    }
#endif

    /* Initialize the CommSvr pointer */
    pCommSvr = NULL;
}

/*================================================================================
STATIC CInterchange Receive Thread : Processes messages in the IN queue.
================================================================================*/
uint32 WACPAPI CInterchange::ReceivingThread(void* pData)
{
    /* Recreate the CInterchange class from the pointer sent in pData. */
    CInterchange* pExchange = (CInterchange*)pData;

    /* Lock the Mutex until the thread is terminated */
    pExchange->RecvThreadMutex.Lock(INFINITE);

    /* Wait until we get a notification of data in the Recv queue */
    pExchange->pRecvSemaphore->Wait(INFINITE);

    while(pExchange->GetThreadStatus() == (uint16)TRUE)
    {
        /* Check for new messages */
        pExchange->EvaluateReceive();        

        /* Wait until we get a notification of data in the Recv queue */
        pExchange->pRecvSemaphore->Wait(INFINITE);
    }

    /* Release the mutex to allow CInterchange to terminate */
    pExchange->RecvThreadMutex.Release();
    
    return TRUE;
}

/*================================================================================
STATIC SendPacketWrapper: Wrapper function for SendPacket().
================================================================================*/
boolean CInterchange::SendPacketWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 Size)
{
    boolean RetVal = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp((char*)pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)ExcgTable[i]->pExchange;
    }

    if (pThis != NULL)
    {
        RetVal = pThis->SendPacket(pSourceID, pBuf, Size);
    }

    return (RetVal);
}

/*================================================================================
STATIC MsgFuncWrapper: Wrapper function for MsgFunc().
================================================================================*/
boolean CInterchange::MsgFuncWrapper(char* pSemName, uint16 Option, void* pData, uint16 DataVal)
{
    boolean RetVal = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp(pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)ExcgTable[i]->pExchange;
    }

    if (pThis != NULL)
    {
        RetVal = pThis->MsgFunc(Option, pData, DataVal);
    }

    return (RetVal);
}

/*================================================================================
STATIC RecvMsgWrapper: Wrapper function for RecvMsg(). pNewConnection 
       are only used in Server configurations
================================================================================*/
boolean CInterchange::RecvMsgWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 MsgSize)
{
    boolean Result = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp(pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)(ExcgTable[i]->pExchange);
    }

    if ((pThis != NULL) && (pBuf != NULL))
    {
        Result = pThis->RecvMsg(pSourceID, pBuf, MsgSize);
    }
    
    return (Result);
}

/*================================================================================
STATIC DeviceArrivalWrapper: Wrapper function for DeviceArrival().
================================================================================*/
char* CInterchange::DeviceArrivalWrapper(char* pSemName, char* pSourceID, 
                                           ConnArgs* pConnArgs)
{
    char* pNewSourceID = NULL;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp((char*)pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)(ExcgTable[i]->pExchange);
    }

    if (pThis != NULL) 
    {
        pNewSourceID = pThis->DeviceArrival(pSourceID, pConnArgs);
    }
    
    return (pNewSourceID);
}

/*================================================================================
STATIC DeviceRemovalWrapper: Wrapper function for DeviceRemoval().
================================================================================*/
boolean CInterchange::DeviceRemovalWrapper(char* pSemName, char* pSourceID)
{
    boolean Result = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp((char*)pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)(ExcgTable[i]->pExchange);
    }

    if (pThis != NULL) 
    {
        Result = pThis->DeviceRemoval(pSourceID);
    }
    
    return (Result);
}
/*================================================================================
CInterchange Send Packet: Wraps the passed in Buffer into a PACKET structure and
    sends the message via the CommSvr.
================================================================================*/
boolean CInterchange::SendPacket(char* pSourceID, uint8* pBuf, uint32 Size)
{
    boolean RetVal = FALSE;
    uint8* OutBuffer;
    uint32 PacketLen;
    ConnArgs* pConnArgs = LinkTable.GetEntry(pSourceID);

    /* Set the Size of the entire packet */
    PacketLen = PACKET_HEADER_SIZE + Size + CRCSIZE;

    if ((pCommSvr != NULL) && (pConnArgs != NULL) && (pConnArgs->bConnected != FALSE))
    {
        /* Allocate room for the serialize packet Stream freed below */
        OutBuffer = GeneratePacket(&PacketLen, pCommSvr->GetPortNum(pConnArgs),0, 0, pBuf, Size);
        if (OutBuffer)
        {
            /* Send the message */
            RetVal = pCommSvr->SendMsg(pConnArgs, (uint8*)OutBuffer, PacketLen);

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
            {
                char tempString[] = {"SourceID %s\nMsgOutCount %d\nMessage Data:\n%A"};
                SYS_TRACE(TRACE_Type_SEND,
                    "CInterchange", "SendPacket", 0, 0, 0, 
                    tempString, pSourceID, ++MsgOutCount, OutBuffer, PacketLen);
            }
#endif
            /* Free the buffer */
            SYS_FREE(OutBuffer);

        }/* if (OutBuffer) */
#if defined(TRACE_SERVER_ON)
        else
        {
            SYS_TRACE(TRACE_Type_ERROR,
                "CInterchange", "SendPacket", 0, 0, 0, 
                "MEMORY ERROR Count %d!!! SendPacket: Unable to allocate memory\n",
                ++ErrCount);
        }
#endif

    }/* if (pCommSvr != NULL) */

    return(RetVal );
}

/*================================================================================
CInterchange MsgFunc: 
    WACP_CONNECT: Opens a set of connections based on the parameters passed in.
    WACP_STATUS: Updated the RNDZ status of the currrent connection.
    WACP_DISCONNECT: Closes a single connection based on the SourceID in pData
================================================================================*/
boolean CInterchange::MsgFunc(uint16 Option, void* pData, uint16 DataVal)
{
    boolean RetVal = FALSE;

    if (ThreadStatus)
    {
        switch (Option)
        {
            case WACP_CONNECT:
            {
                RetVal = Connect((ConnArgs**)pData, (uint8)DataVal);
                break;
            }
            case WACP_STATUS:
            {
                if (pCommSvr)
                {
                    ConnArgs* pConnArgs = LinkTable.GetEntry((char*)pData);
                    if (pConnArgs)
                    {
                        RetVal = pCommSvr->SetPortNum(pConnArgs);
                    }
                }
                break;
            }
            case WACP_DISCONNECT:
            {
                RetVal = Disconnect((char*)pData);
                break;
            }
#if defined(TRACE_SERVER_ON)
            default:
                SYS_TRACE(TRACE_Type_ERROR,
                    "CInterchange", "MsgFunc", 0, 0, 0, 
                    "ERROR Count %d!!! CInterchange MsgFunc: Error Invalid Option\n", ++ErrCount);
                break;
#endif
        }
    }

    return (RetVal);
}

/*================================================================================
CInterchange Evaluate Receive Thread 
================================================================================*/
void CInterchange::EvaluateReceive()
{
    PACKET      Pack;
    MESSAGE     Msg;
    ConnArgs*   pConnArgs = NULL;

    /* Process the message if the queue is not empty */
    if (pINQMgr->get_num_used() > 0)
    {
        uint8*  pTemp;
        uint8*  pBuf = NULL;
        int     Port;
        uint32  DataLength;

        /* Wait for the pINQMgr mutex */
        RecvMutex.Lock(INFINITE);

        /* Get the first packet Stream in the queue */
        pINQMgr->dequeue_array((uint8*)&Msg, sizeof(MESSAGE));

        /* Release the RecvBuf Mutex */
        RecvMutex.Release();

        /* Set pBuf to the actual data buffer */
        pBuf = Msg.pDataBuf;
        
        /* Set the Temp Pointer beyond the PREAMBLE of the byte stream */
        pTemp = (uint8*)pBuf + PREAMBLE_SIZE;

        /* Read the Packet Length */
        Pack.Header.PacketLen = ReadInt32(pTemp);
        pTemp += sizeof(int32);

        /* Read the Port */
        Pack.Header.Port = ReadInt16(pTemp);
        pTemp += sizeof(int16);

        /* Check the last Preamble Character...
            * CTL_C is the full packetization support mode.
            * CTL_L is the light mode that does not contain the SequenceNum and UUID
            */
        if (pBuf[2] == CTL_C)
        {
            /* Read the SequenceNum */
            Pack.Header.SequenceNum = ReadInt16(pTemp);
            pTemp += sizeof(int16);

            /* Read the Packet_UUDID */
            Pack.Header.Packet_UUID = ReadInt32(pTemp);
            pTemp += sizeof(int32);
            
            DataLength = Pack.Header.PacketLen - PACKET_HEADER_SIZE  - 2;
        }
        else
        {
            DataLength = Pack.Header.PacketLen - PACKET_LITE_HEADER_SIZE  - 2;
        }

        /* Set the data length */
        Msg.DataSize = DataLength;

        /* Set the data buffer pointer */
        Msg.pDataBuf = pTemp;

        /* Increment past the data */
        pTemp += DataLength;

        /* Get the CRC */
        Pack.CRC = ReadInt16(pTemp);

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
        {
            char tempString[] = {"SourceID %s\n\nMessage Data:\n%A"};
            SYS_TRACE(TRACE_Type_RECEIVE,
                "CInterchange", "EvaluateReceive", 0, 0, 0, 
                tempString, Msg.pSourceID, pBuf, Pack.Header.PacketLen);
        }
#endif

        /* Get the Port */
        Port = Pack.Header.Port;

        /* Check the SourceID in the LinkTable */
        pConnArgs = LinkTable.GetEntry(Msg.pSourceID);

        /* Exit the function and drop the message if the entry is not found in the LinkTable */
        if (!pConnArgs)
        {
            if (Msg.bAllocated == TRUE)
            {
                SYS_FREE(Msg.pSourceID);
            }

#if defined(TRACE_SERVER_ON_STATUS)
            char tempString[] = {"Unable to find LinkTable Entry. SourceID %s\nMessage Data:\n%A"};
            SYS_TRACE(TRACE_Type_ERROR,
                "CInterchange", "EvaluateReceive", 0, 0, 0, 
                tempString, NULL, pBuf, Pack.Header.PacketLen);
#endif
            SYS_FREE(pBuf);
            return;
        }

        /* Verify the CRC */
        if (Pack.CRC == CalculateCRC((uint8*)pBuf, Pack.Header.PacketLen - CRCSIZE))
        {
            boolean bRejectFlag = FALSE;

            /* Test that we have a valid Port number and, if RNDZ is enabled,
            * that RNDZ is complete before allowing any WACP messages.
            */
            if (pConnArgs->bRNDZDisabled == FALSE)
            {
                bRejectFlag = (!(Port == RNDZPort || Port == WACPPort) ||
                            (((Port == WACPPort) && (pConnArgs->bRNDZStatus == FALSE))));
            }

            if (bRejectFlag)
            {
#if defined(TRACE_SERVER_ON)
                char tempString[] = {"Unauthenticated Message. SourceID %s\nMessage Data:\n%A"};
                SYS_TRACE(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    tempString, Msg.pSourceID, pBuf, Pack.Header.PacketLen);
#endif
                SYS_FREE(pBuf);
                if (Msg.bAllocated == TRUE)
                {
                    SYS_FREE(Msg.pSourceID);
                }
                return;
            }

            if (ThreadStatus)
            {
                pCWACPApp->ProcessMsg(Msg.pSourceID, Msg.pDataBuf, Msg.DataSize);
            }

        }/* if (Pack.CRC == CalculateCRC(Pack)) */
        else
        {
            /* CRC mismatch! Pass message up to presentation layer for 
             * notification to the application
             */

            /* Set the data length */
            Msg.DataSize = sizeof(CRCError);

            Msg.pDataBuf = (uint8*)SYS_MALLOC(Msg.DataSize);
            if (Msg.pDataBuf != NULL)
            {
                /* Set the CRCError in the message data buffer*/
                memcpy(Msg.pDataBuf, CRCError, Msg.DataSize);
            } 

#if defined(TRACE_SERVER_ON)
            {
                char tempString[] = {"CRC failure Stored CRC %d Calculated CRC %d\nErrCount %d\nMessage Data:\n%A"};
                SYS_TRACE(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    tempString, Pack.CRC, 
                    CalculateCRC((uint8*)pBuf, Pack.Header.PacketLen - CRCSIZE),
                    ++ErrCount, pBuf, Pack.Header.PacketLen);
            }
#endif
            if (ThreadStatus)
            {
                pCWACPApp->ProcessMsg(Msg.pSourceID, Msg.pDataBuf, Msg.DataSize);
            }

            SYS_FREE(Msg.pDataBuf);
        }

        if (Msg.bAllocated == TRUE)
        {
            SYS_FREE(Msg.pSourceID);
        }

        /* Free the message Buffer allocated in RecvMsg() */
        SYS_FREE(pBuf);

    }/*if (IN_queue.Size() > 0)*/
}

/*================================================================================
CInterchange StartThreads
================================================================================*/
boolean CInterchange::StartClient(void)
{       
    boolean Result = FALSE;

    /* Set the semaphore name used to retrieve data */
    LnkStruct.pSemName = AppSemName;

    /* Update the bServer flag  */
    LnkStruct.bServer = FALSE;
    
    /* Set the ThreadPriority */
    LnkStruct.ThreadPriority = Args.ThreadPriority;

    /* Set the DLL reference */
    LnkStruct.pCallBackObj = Args.pCallBackObj;

    /* Set the CLinkTable instance */
    LnkStruct.pCLinkTable = (void*)&(LinkTable);

    pCommSvr = new CCommSvr(RecvMsgWrapper, DeviceArrivalWrapper, DeviceRemovalWrapper, AppSemName);
    if (pCommSvr)
    {
        /* Create the application layer */  
        pCWACPApp = new CWACPApp(SendPacketWrapper,
                            MsgFuncWrapper,
                            &(LinkTable),
                            &LnkStruct);
        if (pCWACPApp)
        {
            /* Start up the App */
            pCWACPApp->AppStart();
            Result = TRUE;
        }
    }    

    return (Result);
}

/*================================================================================
CInterchange Thread Status
================================================================================*/
boolean CInterchange::GetThreadStatus()
{   
    return ThreadStatus;
}

/*================================================================================
CInterchange Close: Closes all Mutex handles and sets the ThreadStatus to FALSE
================================================================================*/
uint16 CInterchange::Close()
{
    ThreadStatus = FALSE;
    uint16 i;

    char* pSourceList[MAX_CONNECTIONS];
    
    /* Get a list of all entries in Linktable and shut down all of them individually */
    LinkTable.GetSourceList(pSourceList);
    for (i=0; i<MAX_CONNECTIONS; i++)
    {
        if (pSourceList[i] == NULL)
            break;

        Disconnect(pSourceList[i]);
    }

    /* Raise the Semaphore to allow the recv thread to terminate */
    pRecvSemaphore->Raise();

    /* Wait for the Recv Thread to terminate */
    RecvThreadMutex.Lock(INFINITE);
    RecvThreadMutex.Release();

    if(pRecvSemaphore != NULL)
        delete(pRecvSemaphore);

    return TRUE;
}

/*================================================================================
CInterchange RecvMsg: Adds a message to the pINQMgr
================================================================================*/
boolean CInterchange::RecvMsg(char* pSourceID, uint8* pBuf, uint32 MsgSize)
{
    boolean Result = FALSE;
    MESSAGE Msg;

    /* Check for a valid Buffer and Size */
    if (pBuf != NULL)
    {
        ConnArgs* pCurrConnArgs = NULL;

        /* Check for an existing entry */
        pCurrConnArgs = LinkTable.GetEntry(pSourceID);
        if (pCurrConnArgs != NULL)
        {
            /* Allocate room for the new message, this is released in EvaluateReceive() */
            Msg.pDataBuf = (uint8*)SYS_MALLOC(MsgSize);
            if (Msg.pDataBuf != NULL)
            {
                /* Create a copy of the message */
                memcpy(Msg.pDataBuf, pBuf, MsgSize);

                /* Set the Data size */
                Msg.DataSize = MsgSize;

                /* Set the SourceID */
                Msg.pSourceID = pCurrConnArgs->SourceID;
                Msg.bAllocated = FALSE;

                /* Wait for the Recv Mutex */
                RecvMutex.Lock(INFINITE);
        
                /* Queue up the MESSAGE */
                Result = pINQMgr->enqueue_array((uint8*)&Msg, sizeof(MESSAGE));

#if defined(TRACE_SERVER_ON)
                if (!Result)
                {
                    SYS_TRACE(TRACE_Type_ERROR,
                        "CInterchange", "RecvMsg", 0, 0, 0,
                        "pINQMgr is full %s. Unable to queue up message for %s\n", pCurrConnArgs->SourceID);
                }
#endif
          
                /* Release the Mutex */
                RecvMutex.Release();

                /* Notify the Receiving thread to process the message */
                pRecvSemaphore->Raise();

                Result = TRUE;
            }
        }
    }

    return (Result);
}

/*================================================================================
DeviceArrival: Creates a new client connection
================================================================================*/
char* CInterchange::DeviceArrival(char* pSourceID, ConnArgs* pConnArgs)
{
    char* pNewSourceID = NULL;
  
    if ((pSourceID != NULL) && (pConnArgs != NULL) && (ThreadStatus == TRUE))
    {
        ConnArgs* pCurrConnArgs = NULL;

        /* Check for an existing entry */
        pCurrConnArgs = LinkTable.GetEntry(pSourceID);
        if (pCurrConnArgs == NULL)
        {
            boolean Result = pCWACPApp->Open(&pConnArgs, 1, TRUE);

            /* Check for a valid pointer */
            if (Result)
            {
                pConnArgs = LinkTable.GetEntry(pSourceID);
                if (pConnArgs)
                {
                    /* Start StartClient */
                    Result = pCommSvr->Connect(pConnArgs);
                    pNewSourceID = pConnArgs->SourceID;
                }
            }
        }
    }
    return pNewSourceID;
}

/*================================================================================
DeviceRemoval: Terminates the client connection
================================================================================*/
boolean CInterchange::DeviceRemoval(char* pSourceID)
{
    boolean Result = FALSE;
    MESSAGE Msg;
    uint32 PacketLen = 0;

    /* Open the named handle and signal the Semaphore */
    if ((AppSemName != NULL) && (pSourceID != NULL))
    {
        ConnArgs* pConnArgs;

        /* Set the data length */
        Msg.DataSize = sizeof(WacpTerminate);

        /* Set the SourceID */
        Msg.pSourceID = (char*)SYS_MALLOC(ADDR_BUF_SIZE);
        if (Msg.pSourceID)
        {
            memcpy(Msg.pSourceID, pSourceID, ADDR_BUF_SIZE);

            /* Set the Allocated flag to insure we release this memory allocation */
            Msg.bAllocated = TRUE;
        }

        /* Set the Size of the entire packet */
        PacketLen = PACKET_HEADER_SIZE + Msg.DataSize + CRCSIZE;

        /* Grab the new ConnArgs pointer */
        pConnArgs = LinkTable.GetEntry(pSourceID);
        if (pConnArgs)
        {
            Msg.pDataBuf = GeneratePacket(&PacketLen, pCommSvr->GetPortNum(pConnArgs),0, 0, (uint8*)WacpTerminate, Msg.DataSize);

            /* Wait for the Recv Mutex */
            RecvMutex.Lock(INFINITE);
            
            /* Queue up the MESSAGE */
            pINQMgr->enqueue_array((uint8*)&Msg, sizeof(MESSAGE));
        
            /* Release the Mutex */
            RecvMutex.Release();

            /* Notify the Receiving thread to process the message */
            pRecvSemaphore->Raise();

            Result = TRUE;
        }

    }/* if (AppSemName != NULL) */

    return Result;
}


/* End of File */


/*================================================================================
GeneratePacket: Wraps the passed in Buffer into a PACKET structure and
    adds header information based on parameters. Returns a newly malloced 
    uint8 buffer filled with the new packet.
================================================================================*/
uint8* GeneratePacket(uint32* OutSize, uint16 Port, int16 SequenceNum, int32 UUID, uint8* pBuf, uint32 Size)
{
    uint8* OutBuffer = NULL;
    boolean Packetized = FALSE;

    /* Set the Size of the entire packet */
    if (Packetized)
    {
        *OutSize = PACKET_HEADER_SIZE + Size + CRCSIZE;
    }
    else
    {
        *OutSize = PACKET_LITE_HEADER_SIZE + Size + CRCSIZE;
    }

    /* Allocate room for the serialize packet Stream freed in EvaluateSend*/
    OutBuffer = (uint8*)SYS_MALLOC(*OutSize);
    
    if (OutBuffer)
    {
        uint8* pTemp = OutBuffer; 

        /* Serialize the packet header struct */
        /* Set the PREAMBLE */    
        *pTemp++ = CTL_W;
        *pTemp++ = CTL_A;

        /* Check to see if this is a piece of a packetized message */
        if (Packetized)
            *pTemp++ = CTL_C;
        else
            *pTemp++ = CTL_L;

        /* Write the Packet Length */
        WriteInt32(pTemp, *OutSize);
        pTemp += sizeof(int32);

        /* Write the Port */
        WriteInt16(pTemp, Port);
        pTemp += sizeof(int16);

        if (Packetized)
        {
            /* Write the SequenceNum */
            WriteInt16(pTemp, SequenceNum);
            pTemp += sizeof(int16);

            /* Write the Packet_UUDID */
            WriteInt32(pTemp, UUID);
            pTemp += sizeof(int32);
        }

        /* Copy the data Stream */
        memcpy((char*)pTemp, pBuf, Size);
        pTemp += Size;
        
        /* Write the CRC */
        WriteInt16(pTemp, CalculateCRC(OutBuffer, *OutSize - CRCSIZE));

    }/* if (OutBuffer) */

    return(OutBuffer);
}

/*================================================================================
Connect: Creates the connections specified in the ConnArgs array passed in. The
    status boolean bConnected is updated with each individual connection status.
    
    Returns TRUE if all connections are opened successfully
    Returns FALSE if any one connection fails to open.
================================================================================*/
boolean CInterchange::Connect(ConnArgs** pConfig, uint8 NumConnections)
{
    boolean Result = FALSE;
    uint16 i = 0;
    ConnArgs* pConnArgs = NULL;

    if (pCommSvr != NULL)
    {
        for (i=0; i < NumConnections; i++)
        {
            pConnArgs = LinkTable.GetEntry(pConfig[i]->SourceID);
            if (pConnArgs)
            {
                if (pConnArgs->bConnected == FALSE)
                {
                    /* Client Connections */
                    if (pConnArgs->bSvrFlag == FALSE)
                    {
                        /* Start the connection */
                        Result = pCommSvr->Connect(pConnArgs);
                        if (Result)
                        {
                            /* Update the connection status */
                            pConnArgs->bConnected = TRUE;
                        }
#if defined(TRACE_SERVER_ON)
                        else
                        {
                            SYS_TRACE(TRACE_Type_ERROR,
                                "CInterchange", "Connect", 0, 0, 0,
                                "Failed to connect to %s\n", pConfig[i]->SourceID);
                        }
#endif
                    }
                    else /* Server connections */
                    {
                        /* Start the connection */
                        Result = pCommSvr->StartServer(&pConnArgs,  1);
                        if (Result)
                        {
                            /* Update the connection status */
                            pConnArgs->bConnected = TRUE;
                        }
#if defined(TRACE_SERVER_ON)
                        else
                        {
                            SYS_TRACE(TRACE_Type_ERROR,
                                "CInterchange", "Connect", 0, 0, 0,
                                "Failed to connect to %s\n", pConfig[i]->SourceID);
                        }
#endif
                    }
                }
#if defined(TRACE_SERVER_ON)
                else
                {
                    SYS_TRACE(TRACE_Type_ERROR,
                        "CInterchange", "Connect", 0, 0, 0,
                        "%s is already connected\n", pConfig[i]->SourceID);
                }
#endif
            }/* if (pConnArgs) */
        }/* for (i=0; i < NumConnections; i++) */
    }/* if (pCommSvr != NULL) */

    return (Result);
}

/*================================================================================
Disconnect: Closes down a single connection based on the SourceID.

    Returns TRUE if the connection is successfully closed.
    Returns FALSE if the SourceID is not found or if the connection was not closed.
================================================================================*/
boolean CInterchange::Disconnect(char* pSourceID)
{
    boolean Result = FALSE;
    ConnArgs* pConnArgs = NULL;

    /* Find the entry in the linktable */
    pConnArgs = LinkTable.GetEntry(pSourceID);
    if (pConnArgs) 
    {
        if (pConnArgs->bSvrFlag == FALSE)
        {
            if (pConnArgs->bConnected == TRUE)
            {
                Result = pCommSvr->Disconnect(pConnArgs);
                if (Result == TRUE)
                {
                    /* Reset the RNDZ authentication */
                    pConnArgs->bRNDZStatus = FALSE;

                    /* Reset the connection status */
                    pConnArgs->bConnected = FALSE;

                    /* Free up the memory allocated in connection sequence */
                    SYS_FREE(pConnArgs->pKey);
                    pConnArgs->pKey = NULL;

                    if (pConnArgs->pGUIDStd)
                    {
                        delete(pConnArgs->pGUIDStd);
                        pConnArgs->pGUIDStd = NULL;
                    }
                }
            }
#if defined(TRACE_SERVER_ON)
            else
            {
                SYS_TRACE(TRACE_Type_ERROR,
                    "CInterchange", "Disconnect", 0, 0, 0,
                    "Failed to disconnect %s\n", pSourceID);
            }
#endif
        }/* if (pConnArgs->bSvrFlag == FALSE) */
        else
        {
            Result = pCommSvr->Disconnect(pConnArgs);

            /* Free up the memory allocated in connection sequence */
            SYS_FREE(pConnArgs->pKey);
            pConnArgs->pKey = NULL;

            if (pConnArgs->pGUIDStd)
            {
                delete(pConnArgs->pGUIDStd);
                pConnArgs->pGUIDStd = NULL;
            }
        }

        /* Remove the entry from the LinkTable */
        LinkTable.RemoveEntry(pConnArgs->SourceID);

    }/* if (pConnArgs) */
    else
    {
        Result = FALSE;
    }

    return (Result);
}

