/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CInterchang.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CINTERCHANGE_H)
    #define  CINTERCHANGE_H

#include "Common.h"
#include "SysUtil.h"
#include "CCommSvr.h"
#include "CWACPApp.h"
#include "CWACPMutex.h"
#include "CWACPThread.h"
#include "CWACPSemaphore.h"


#define WACP_SEM_NAME "WACP_%d_%p"
#define EXIT_EXCHANGE "EXIT_EXCHANGE_%d"
#define CRCSIZE (2)

struct PACKET_HDR
{
    uint8   Preamble[PREAMBLE_SIZE];
    uint32  PacketLen;
    uint16  Port;
    uint16  SequenceNum;
    uint32  Packet_UUID;
};

/* PACKET_LITE_HEADER_SIZE = PREAMBLE + PacketLen + Port
 *      PREAMBLE  = sizeof(uint8) * PREAMBLE_SIZE
 *      PacketLen = sizeof(uint32)
 *      Port      = sizeof(uint16) 
 */
#define PACKET_LITE_HEADER_SIZE 9

/* PACKET_HEADER_SIZE = PREAMBLE + PacketLen + Port + SequenceNum + Packet_UUID
 *      PREAMBLE    = sizeof(uint8) * PREAMBLE_SIZE
 *      PacketLen   = sizeof(uint32)
 *      Port        = sizeof(uint16) 
 *      SequenceNum = sizeof(uint16) 
 *      Packet_UUID = sizeof(uint32)
 */
#define PACKET_HEADER_SIZE 15

struct PACKET
{
    PACKET_HDR  Header;
    uint8*      pDataBuffer;
    uint16      CRC;
};

struct MESSAGE
{
    char*  pSourceID;
    uint8* pDataBuf;
    uint32 DataSize;
    boolean bAllocated;
};

struct ThreadArgs
{
    void* pExchange;
    int32 ThreadPriority;
    void* pCallBackObj;
};

struct DLLStruct
{
    void* pExchange;
    char* pSourceID;
    char* pAppName;
};

class CInterchange
{
public:
    CInterchange();
    ~CInterchange();

    void Init(uint16 RNDZPortNum, uint16 WACPPortNum, int32 Priority, void* pCallBackObj);

    boolean StartClient(void);

    uint16 Close();

    boolean GetThreadStatus();

    char* DeviceArrival(char* pSourceID, ConnArgs* pConnArgs);
    boolean DeviceRemoval(char* pSourceID);

private:
    static DLLStruct*   ExcgTable[MAX_DLL_COUNT];
    static uint16       DLLCount;
    DLLStruct           ExcgStruct;
    LinkStruct          LnkStruct;

    /* Static Callback Function Wrappers */
    static boolean SendPacketWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 Size);
    static boolean MsgFuncWrapper(char* pSemName, uint16 Option, void* pData, uint16 DataVal);
    static boolean RecvMsgWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 MsgSize);
    static boolean TerminateClientWrapper(char* pSemName, char* pSourceID, uint16 Port);
    static char* DeviceArrivalWrapper(char* pSemName, char* pSourceID, ConnArgs* pConnArgs);
    static boolean DeviceRemovalWrapper(char* pSemName, char* pSourceID);

    /* Callback Functions */
    boolean SendPacket(char* pSourceID, uint8* pBuf, uint32 Size);
    boolean MsgFunc(uint16 Option, void* pData, uint16 DataVal);
    boolean RecvMsg(char* pSourceID, uint8* pBuf, uint32 MsgSize);
    boolean Connect(ConnArgs** pConnArgs, uint8 NumConnections);
    boolean Disconnect(char* pSourceID);

    /* Thread Functions */
    static uint32 WACPAPI ReceivingThread(void* pData);

    /* Receive Thread function */
    void EvaluateReceive();

    /* Mutex Pointers */
    CWACPMutex  WACPRecvMutex;             /* WACP Recv Queue mutex */
    CWACPMutex  RecvMutex;                 /* IN_queue mutex */
    CWACPMutex  RecvThreadMutex;           /* Exit RecvThread sync mutex */
    
    /* Thread Status var */
    boolean ThreadStatus;

    /* Threads */
    CWACPThread RecvThread;

    /* Queue Buffer */
    uint8* pINQueueBuf;
    uint8QManager* pINQMgr;

    /* Thread arguements*/
    ThreadArgs Args;
    CLinkTable LinkTable;

    /* WACP and RNDZ Ports */
    uint16 RNDZPort;
    uint16 WACPPort;

    /* Pointer to the CCommSvr */
    CCommSvr* pCommSvr;

    /* Pointer to the CWACPApp */
    CWACPApp* pCWACPApp;

    /* Semaphore used for message processing */
    CWACPSemaphore* pRecvSemaphore;

    /* Semaphore name used for DLL usage */
    char AppSemName[EVENT_BUF_SIZE];
    
#if defined(TRACE_SERVER_ON)
    uint32 ErrCount;
#endif

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    uint32 MsgInCount;
    uint32 MsgOutCount;
#endif

};

#endif /* #if !defined(CINTERCHANGE_H) */

/*---------------------------------------------------*/
/* End of File */





