/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CTrapObjSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CTRAPOBJSVR_H)
    #define  CTRAPOBJSVR_H

#include "CObjSvr.h"
#include "CErrEStd.h"
#include "CErrSvr.h"
#include "CMsgSvr.h"
#include "Common.h"

typedef uint32 (*ErrorFuncPtr)(char*, uint8, uint32, uint8*, void*);

typedef  enum TRAPCRSPtypSPECIES {
    TRAP_CRSP_SpNONE = 0,
    TRAP_CRSP_SpPUT_LASTERROR,
    TRAP_CRSP_SpMAX
}CRSP_SPECIES;

typedef  enum TRAPCRSttypSPECIES {
    TRAP_CRST_SpNONE = 0,
    TRAP_CRST_SpGET_LASTERROR,
    TRAP_CRST_SpMAX
}CRST_SPECIES;

class CTrapObjSvr : public CObjSvr
{
/*================================================================================
CTrapObjSvr Members: Public, Private, and Protected
================================================================================*/
private:

    CErrEStd    ErrObj;
    CErrSvr     ErrSvr;           
    CMsgSvr     MsgSvr;         
    
    char*       pSemName;

    /* declaration for callback function */
    ErrorFuncPtr        pErrorHandler;
    SendPacketFuncPtr   pSendPacket;

public:
    CTrapObjSvr();
    ~CTrapObjSvr();

    void Init(boolean (*SendPacket)(char*, char*, uint8*, uint32), 
              uint32 (*RxError)(char*, uint8, uint32, uint8*, void*),
              LinkStruct* pLinkStruct);

    /*================================================================================
    CTrapObjSvr Transmit Error Message
    ================================================================================*/
    boolean TxError(char* pSourceID, uint32 ClassID, uint8 ErrorType, uint8* pBuf, uint32 Size);
    boolean TxErrMsg(char* pSourceID, CErrSvr* pCErrSvr, uint8 Genus, uint8 Species);
    boolean TxMsg(char* pSourceID, CMsgSvr* pMsgSvr, uint8 Genus, uint8 Species);

    /*================================================================================
    CTrapObjSvr Error Routing  
    ================================================================================*/
    boolean SendError(char* pSourceID, uint32 ClassID, uint16 ErrorCode, void* pWACPApp);
    boolean SendResponse(char* pSourceID, uint8 RespID, uint8* Payload, uint32 Size );
    boolean SendRequest(char* pSourceID, uint8 RqstID, uint8* Payload, uint32 Size );
    boolean RecvMsgHandler(char* pSourceID, uint8* pBuffer, void* pRouter );
    boolean SendMsgHandler(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8* pBuffer, uint32 Size, boolean EncryptionOn);
};
#endif /* #if !defined(CTRAPOBJSVR_H) */

/*---------------------------------------------------*/
/* End of File */



