/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// WAUSBDLL.h     :                                                  //
'**  '// Language       :   C/C++                                          //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   USB                                            //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#ifndef __WAUSBDLL_H__
#define __WAUSBDLL_H__

#include "StdVarTyp.h"

#if defined(__cplusplus)
extern "C" {
#endif
    
// Qualifying storage-class information
#define WAUSB_DLLSC     // Implied by explicit DEF file use

// Calling convention for function calls into DLL
#define WAUSB_API       APIENTRY

// Calling convention for callback functions into the application
#define WAUSB_CALLBACK  CALLBACK


#define WAUSB_S_DLL_NAME                            "WaUsb.dll"

#define WAUSB_S_ENTRY_WA_USB_CONNECT                "WA_USB_Connect"
#define WAUSB_S_ENTRY_WA_USB_CONFIGURE              "WA_USB_Configure"
#define WAUSB_S_ENTRY_WA_USB_DISCONNECT             "WA_USB_Disconnect"
#define WAUSB_S_ENTRY_WA_USB_GET_STRING_DESRIPTOR   "WA_USB_GetStringDescriptor"
#define WAUSB_S_ENTRY_WA_USB_OPEN_SESSION           "WA_USB_OpenSession"
#define WAUSB_S_ENTRY_WA_USB_CLOSE_SESSION          "WA_USB_CloseSession"
#define WAUSB_S_ENTRY_WA_USB_RECV                   "WA_USB_Recv"
#define WAUSB_S_ENTRY_WA_USB_RESET                  "WA_USB_Reset"
#define WAUSB_S_ENTRY_WA_USB_SEND                   "WA_USB_Send"
#define WAUSB_S_ENTRY_WA_CONFIG                     "WA_USB_Config"
#define WAUSB_S_ENTRY_WA_GET_DIAG_COUNTERS          "WA_USB_GetDiagCounters"
#define WAUSB_S_ENTRY_WA_CLR_DIAG_COUNTERS          "WA_USB_ClrDiagCounters"

#define WAUSB_U_MAX_DEVICENAME  ((uint16)32)

// Data types
typedef enum
{
    BULK,
    INTERRUPT,
    ISOCHRONOUS,
    UNKNOWN = -1
}
ENDPOINT_TYPE;

typedef enum
{
    LOW_SPEED,
    FULL_SPEED,
    HIGH_SPEED
}
DEVICE_SPEED;

typedef enum
{
    ENDPOINT_IN,
    ENDPOINT_OUT
}
ENDPOINT_DIR;

typedef enum
{
    MANUFACTURER_STR = 1,   // Value predefined by USB Spec
    PRODUCT_STR      = 2,   // Value predefined by USB Spec
    SERIAL_NUMBER    = 3    // Value predefined by USB Spec
}
STR_DESC_ID;

typedef struct
{
    ENDPOINT_TYPE   EndPointType;
    int32           EndPointID;
    ENDPOINT_DIR    Direction;
}
ENDPOINT_DESC, *PENDPOINT_DESC;

typedef struct
{
    char            szDevName[WAUSB_U_MAX_DEVICENAME];
    uint32          DevNumber;
    int32           VendorID;
    int32           ProductID;
    int32           ProductVersion;
    DEVICE_SPEED    DeviceSpeed;
    int32           NumEndPoints;
    ENDPOINT_DESC   EndPoints[1];
}
DEVICE_DESC, *PDEVICE_DESC;

typedef struct
{
    ENDPOINT_DESC   EndPointDesc;
    int32           NumRecRequests;
    int32           NumReceived;
    int32           NumSendRequests;
    int32           NumSent;
    int32           NumTimeout;
    int32           NumCancelled;
    int32           NumErrors;
}
ENDPOINT_DIAG, *PENDPOINT_DIAG;

typedef struct
{
    int32           NumEndPoints;
    int32           NumPendingIoReq;
    int32           NumPausedIoReq;
    ENDPOINT_DIAG   EndPointDiag[1];
}
DIAG_COUNTERS, *PDIAG_COUNTERS;

typedef struct
{
    uint32          ReceiveTimeOut; // In msesconds
}
WAUSBCONFIG, *PWAUSBCONFIG;


#ifndef KERNEL

typedef void
(WAUSB_CALLBACK* DEVICEARRIVALPROC)(
    const PDEVICE_DESC  pDeviceDesc,
    void*               pUserArrivalData);

typedef void
(WAUSB_CALLBACK* DEVICEREMOVALPROC)(
    const char*         pDevName, 
    void*               pUserRemovalData);

typedef void
(WAUSB_CALLBACK* DEVICEFREEPROC)(
    const PDEVICE_DESC  pDeviceDesc,
    void*               pUserFreeData);

typedef void
(WAUSB_CALLBACK* RECVDATAPROC)(
    uint32              uiStatus,
    const char*         pDevName, 
    int32               EndPointID, 
    void*               pRecvData, 
    uint32              RevcLength, 
    void*               pUserRecvData);

// Prototypes
typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_OPEN_SESSION)(
    DEVICEARRIVALPROC   procDeviceArrival, 
    void*               pUserArrivalData, 
    DEVICEREMOVALPROC   procDeviceRemoval, 
    void*               pUserRemovalData,
    DEVICEFREEPROC      procDeviceFree,
    void*               pUserFreeData);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_OpenSession(
    DEVICEARRIVALPROC   procDeviceArrival, 
    void*               pUserArrivalData, 
    DEVICEREMOVALPROC   procDeviceRemoval, 
    void*               pUserRemovalData,
    DEVICEFREEPROC      procDeviceFree,
    void*               pUserFreeData);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_CLOSE_SESSION)();

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_CloseSession();


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_CONNECT)(
    const char*         pDevName, 
    RECVDATAPROC        procRecvData,
    void*               pUserRecvData);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_Connect(
    const char*         pDevName, 
    RECVDATAPROC        procRecvData,
    void*               pUserRecvData);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_CONFIGURE)(
    const char*         pDevName, 
    const PWAUSBCONFIG  pConfigData);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_Configure(
    const char*         pDevName, 
    const PWAUSBCONFIG  pConfigData);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_DISCONNECT)(
    const char*         pDevName);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_Disconnect(
    const char*         pDevName); 


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_GETSTRINGDESCRIPTOR)(
    const char*         pDevName, 
    int32               DescriptorID,   // Identifies the string descriptor. 
    char*               pString,
    int32*              pStringLength);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_GetStringDescriptor(
    const char*         pDevName, 
    int32               DescriptorID,   // Identifies the string descriptor. 
    char*               pString,
    int32*              pStringLength);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_RECV)(
    const char*         pDevName, 
    int32               EndPointID,
    uint8*              pBuf, 
    int32               BufLen,
    uint32*             pBytesReceived);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_Recv(
    const char*         pDevName, 
    int32               EndPointID,
    uint8*              pBuf, 
    int32               BufLen,
    uint32*             pBytesReceived);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_RECV_ASYNC)(
    const char*         pDevName, 
    int32               EndPointID,
    uint8*              pBuf, 
    int32               BufLen);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_RecvAsync(
    const char*         pDevName, 
    int32               EndPointID,
    uint8*              pBuf, 
    int32               BufLen);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_SEND)(
    const char*         pDevName, 
    int32               EndPointID,
    uint8*              pBuf, 
    int32               BufLen);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_Send(
    const char*         pDevName, 
    int32               EndPointID,
    uint8*              pBuf, 
    int32               BufLen);


// Resets all connections
typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_SOFT_RESET)(
    const char*         pDevName);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_SoftReset(
    const char*         pDevName);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_HARD_RESET)(
    const char*         pDevName);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_HardReset(
    const char*         pDevName);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_DEVICE_RESTART)(
    const char*         pDevName);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_DeviceRestart(
    const char*         pDevName);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_GET_DIAG_COUNTERS)(
    const char*         pDevName,
    uint8**             pDiagCounters);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_GetDiagCounters(
    const char*         pDevName,
    PDIAG_COUNTERS*     pDiagCounters);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_CLR_DIAG_COUNTERS)(
    const char*         pDevName);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_ClrDiagCounters(
    const char*         pDevName);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_SET_VID)(
    const char*			pDevName,
	 uint16             Vid);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_SetVid(
    const char*         pDevName,
    uint16              Vid);


typedef uint32
(WAUSB_API* WAUSB_PROC_WA_USB_SET_PID)(
    const char*         pDevName,
	uint16              Pid);

WAUSB_DLLSC uint32 WAUSB_API
WA_USB_SetPid(
    const char*         pDevName,
    uint16              Pid);

#endif // KERNEL

#if defined(__cplusplus)
}
#endif

#endif // __WAUSB1DLL_H__
