/*-D*************************************************************************
*
*  Copyright 2008 Welch Allyn, Inc - All rights reserved.
*
*  FILENAME: CIOCP.h
*
*  PURPOSE:  Declaration of the CIOCP. Provides a simple wrapper for
*            NT I/O completion ports.
*
*  NOTES:  Using IOCPs, a server application can service multiple clients by
*          using multiple threads, but not in a thread-per-client fashion. An
*          IOCP is an object that can be associated with a number of I/O objects,
*          such as files, named pipes, sockets, or remote procedure calls (RPCs).
*          When using IOCPs, the OS manages the data queues using a fixed number
*          of threads. After all worker threads are setup, the system kernel
*          waits in each thread to process data asynchronously with
*          GetQueuedCompletionStatus(). The I/O completion port places the
*          completion packet in a first-in-first-out (FIFO) queue. Also, the
*          actual thread usage is based on a LIFO stack, where the last thread
*          used will be re-used first, if available. This functionality for
*          NT I/O completion ports yields extremely high thread performance
*          and minimizes context switches.
**-E*************************************************************************/
#pragma once
#include <windows.h>

class CIOCP
{
private:
	HANDLE m_hIOcp;
public:
	CIOCP(DWORD NumberOfConcurrentThreads=0); //Constructor (default to allow as many threads as processors)
	~CIOCP(); //Destructor
	DWORD Associate(HANDLE FileHandle, ULONG_PTR CompletionKey);
	HANDLE GetHandle(); //Read-only accessor to internal completion port handle
	//Posts an I/O completion packet to an I/O completion port.
	DWORD Enqueue(DWORD NumberOfBytesTransferred, ULONG_PTR CompletionKey, LPOVERLAPPED pOverlapped);
	//Dequeue an I/O completion packet from the specified I/O completion port.
	DWORD Dequeue(LPDWORD pNumberOfBytesTransferred, PULONG_PTR pCompletionKey, LPOVERLAPPED* ppOverlapped, DWORD Timeout);
};
