/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// COverlappedTransport.h  :                                         //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '// Notes:  Main overlapped architect class keeps a reference-counted
'**              completion port for any number of derived classes to use.
'**              This keeps the thread count static. Instances of this class,
'**              such as transports, provide implementation of overlapped
'**              I/O for any type of file handle. To avoid exceptions calling
'**              into a derived class that has been destroyed, instances
'**              must wait for all their overlapped operations to complete
'**              before their destruction.
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#pragma once

#include <windows.h>
#include "target.h"
#include "CIOCP.h"
#include "CBaseCommSvr.h"
#include "SysUtil.h"

#define OVERLAPPED_TRANSPORT_STOP -1 //special key to shutdown overlapped queue loop
#define NUM_OVERLAPPED_THREADS 0 //specify threads or 0 for one per processor
#define MAX_OVERLAPPED_READS 2 //not used in abstract base class, derived classes can choose to use this or some other number
// Overlapped State Codes
#define OVERLAPPED_INIT  0
#define OVERLAPPED_WRITE 1
#define OVERLAPPED_READ  2

extern const HANDLE g_hHeap; //Global declaration for memory heap

class COverlappedTransport : public CBaseCommSvr
{
public:
	COverlappedTransport();
	virtual ~COverlappedTransport();
	DWORD Assign(HANDLE OverlappedFileHandle, ULONG_PTR Cookie);
	virtual void ProcessCP(DWORD dwBytes, LPOVERLAPPED lpol) = 0;
	virtual void RecvCPError(DWORD LastError, LPOVERLAPPED lpol) = 0;

private:
	static unsigned int WINAPI CompletionPortThreadProc(void *);
};
