/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CSerialCommSvr.h:                                                 //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#pragma once
#include <tchar.h>
#include "Target.h"
#include "Common.h"
#include "CWACPMutex.h"
#include "COverlappedTransport.h"

#define SERIAL_MAX_OVERLAPPED_READS 1
#define SERIAL_RCVBUFSIZE 512

typedef struct SerialAddr
{
    boolean bAutoDiscovery;
}SerialAddr;

//Overlapped Plus structure for network client data
typedef struct SERIAL_PACKET
{
    OVERLAPPED Ol; //must be first element
    DWORD EventMask;
    int State;
    BYTE Buffer[SERIAL_RCVBUFSIZE]; //must be last element
} SERIAL_PACKET, *PSERIAL_PACKET;

class CSerialCommSvr : public COverlappedTransport
{
public:

    //Default Constructor/Destructor
    CSerialCommSvr();
    ~CSerialCommSvr();

    //Sets up the configuration of the stack based on the data received via pAddr.
    void SetConfig(void* pAddr);

    // Sends a binary Buffer
    boolean SendMsg(uint8* pBuf, uint32 BufSize);

    //Creates a connection
    boolean Connect(uint8* pSourceID, uint16 PortNum);

private:
    //Completion port callbacks
    void ProcessCP(DWORD dwBytes, LPOVERLAPPED lpOl);
    void RecvCPError(DWORD LastError, LPOVERLAPPED lpOl);
    //Private helper
    DWORD PostReceive(PSERIAL_PACKET pPacket, UINT Count=1);

    DCB m_DCB, m_DCB2; //original, new settings
    COMMTIMEOUTS m_CTOS; //original
    char m_CommPort[MAX_PATH]; //port string for file open
    bool m_bDCB; //flag indicating original DCB has been backed up
    bool m_bCTOS; //flag indicating original CommTimeOuts has been backed up
    bool m_bSetConfig;
    bool m_bAutoDiscovery;
    HANDLE m_hPort;
    volatile LONG m_NumOverlappedReads;
    volatile LONG m_NumOverlappedWrites;
    CRITICAL_SECTION m_CS;
};
