/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CTCPCommSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '// Notes:  This class provides client, server, or session instance.
'**              When providing server functionality, the server may be
'**              paused from the application by closing the listener instance.
'**              In this case, the server will no longer accept new connections.
'**              However, existing connections will be allowed to continue,
'**              providing a graceful switch-over. After the listener is closed
'**              another application could immediately start servicing new
'**              sessions.
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#pragma once
#include <winsock2.h> //link with ws2_32.lib
#include <Mswsock.h>
#include "COverlappedTransport.h"

#define TCP_MAX_OVERLAPPED_READS 1
#define TCP_RCVBUFSIZE USHRT_MAX
#define TCP_ACCEPT_SOCKETS 10  //look ahead sockets for accepting connections
#define TCP_SOURCE_PREFIX "TCP"

typedef struct TCPAddr
{
    SOCKET hSocket;
} TCPAddr;

//Overlapped Plus structure for network client data
typedef struct TCP_Packet
{
    WSAOVERLAPPED Ol; //must be first element
    SOCKADDR_IN Sin;
    INT SinLen;
    SOCKET hSocket;
    int State;
    WSABUF Wsa;
} TCP_PACKET, *PTCP_PACKET;

class CTCPCommSvr : public COverlappedTransport
{
public:
    CTCPCommSvr();
    CTCPCommSvr(bool Server);
    ~CTCPCommSvr();

    /* Handles incoming connection requests */
    uint32 StartServer();

    /* Sets up the configuration of the stack based on the data received via pAddr. */
    void SetConfig(void* pAddr);

    /* Sends a binary Buffer */
    boolean SendMsg(uint8* pBuf, uint32 BufSize);

    /* Creates a connection */
    boolean Connect(uint8* pSourceID, uint16 PortNum);

private:
    /* Completion port callbacks */
    void ProcessCP(DWORD dwBytes, LPOVERLAPPED lpOl);
    void RecvCPError(DWORD LastError, LPOVERLAPPED lpOl);

    void Construct(bool Server=false);
    int BuildAccepts(SOCKET ListenSocket, BYTE Count);
    DWORD PostReceive(PTCP_PACKET pPacket, UINT Count=1);

    SOCKET m_Socket;
    bool m_bServer;
    bool m_bClientMode;
    bool m_bSetConfig;
    SOCKADDR_IN m_addr;
    volatile LONG m_NumOverlappedReads;
    volatile LONG m_NumOverlappedWrites;
    volatile LONG m_SocketClosed; /* signal to stop processing Removal */
    CRITICAL_SECTION m_CS;
};
