/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CUDPCommSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '// Notes:  This class provides client, server, or session instance.
'**              The server instance will create a new socket for each session.
'**              Clients are expected to respond back on the new port after
'**              the first message has gone out, similar to TFTP.
'**              When providing server functionality, the server may be
'**              paused from the application by closing the listener instance.
'**              In this case, the server will no longer accept new sessions.
'**              However, existing connections will be allowed to continue,
'**              providing a graceful switch-over. After the listener is closed
'**              another application could immediately start servicing new
'**              sessions.
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#pragma once
#include <winsock2.h> /* link with ws2_32.lib */
#include "COverlappedTransport.h"

#define UDP_MAX_OVERLAPPED_READS MAX_OVERLAPPED_READS
#define UDP_RCVBUFSIZE USHRT_MAX
#define UDP_SOURCE_PREFIX "UDP"

typedef struct UDPAddr
{
    SOCKADDR_IN ClntAddr;
} UDPAddr;

/* Overlapped Plus structure for network client data */
typedef struct UDP_Packet
{
    WSAOVERLAPPED Ol; /* must be first element */
    SOCKADDR_IN Sin;
    INT SinLen;
    int State;
    WSABUF Wsa;
} UDP_PACKET, *PUDP_PACKET;

class CUDPCommSvr : public COverlappedTransport
{
public:
    /* Constructors */
    CUDPCommSvr();
    CUDPCommSvr(bool Server);
    /* Destructor */
    ~CUDPCommSvr();

    /* Sets up the configuration of the stack based on the data received via pAddr. */
    void SetConfig(void* pAddr);

    /* Sends a binary Buffer */
    boolean SendMsg(uint8* pBuf, uint32 BufSize);

    /* Creates a connection */
    boolean Connect(uint8* pSourceID, uint16 PortNum);

    /* Handles incoming connection requests */
    uint32 StartServer();

private:
    /* Completion port callbacks */
    void ProcessCP(DWORD dwBytes, LPOVERLAPPED lpOl);
    void RecvCPError(DWORD LastError, LPOVERLAPPED lpOl);

    void Construct(bool Server=false);
    BOOL SetupUDP(SOCKET *hSocket, LPSOCKADDR_IN Addr);
    DWORD PostReceive(PUDP_PACKET pPacket, UINT Count=1);

    bool m_bServer;
    bool m_bClientMode;
    bool m_bSetConfig;
    LONG m_PastFirstSend;
    LONG m_PastFirstRecv;
    SOCKET m_Socket;
    SOCKADDR_IN m_Addr;
    volatile LONG m_NumOverlappedReads;
    volatile LONG m_NumOverlappedWrites;
    CRITICAL_SECTION m_CS;
};
