/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CUSBCommSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CUSBCOMMSVR_H)
    #define   CUSBCOMMSVR_H

#include "Target.h"
#include "Common.h"
#include "CWACPMutex.h"
#include "CWACPEvent.h"
#include "CWACPSemaphore.h"
#include "CBaseCommSvr.h"

#include "WAUSBDLL.h"

typedef struct USBAddr
{
    DEVICE_DESC* pDevice;
}USBAddr;

typedef struct AsyncRequest
{
    uint32 EndpointID;
    uint8* pDataBuf;
}AsyncRequest;

class uint8QManager;

class CUSBCommSvr : public CBaseCommSvr
{
    /*================================================================================
    CUSBCommSvr Members: Public, Private, and Protected
    ================================================================================*/

    public:
        CUSBCommSvr();
        ~CUSBCommSvr();
        
        /* Sets up the configuration of the stack based on the data received 
        * via pAddr.
        */
        void SetConfig(void* pAddr);
        
        /* Sends a binary Buffer */
        boolean SendMsg(uint8* pBuf, uint32 BufSize);

        /* Creates a connection */
        boolean Connect(uint8* pSourceID, uint16 PortNum);

        /* Handles the start of a new client connection thread */
        static uint32 WACPAPI StartClient(void* pData);

        /* Handles the startup of server instances */
        uint32 StartServer();

    private:
        USBAddr          Addr;
        volatile boolean bServer;          
        CWACPMutex*      pClientExitMutex;
        CWACPMutex*      pDeviceMutex;
        CWACPMutex*      pDisconnectMutex;
        
        CWACPEvent*      pThreadEvent;

        CWACPSemaphore*  pReadSem;

        volatile boolean bActive;
        volatile boolean bConnected;

	    int32	         INEndPointNum;
	    int32	         OUTEndPoint;

        CWACPMutex*      pCacheMutex;
        uint8QManager*   pCacheQMgr;

        uint8            Cache[USB_CACHE_SIZE];
    public:
        static uint32 WAUSB_CALLBACK RecvData(uint32 Status, char* pDevName, int32 EndPointID, 
                           void* pRecvData, int32 RecvLength, void*	pUserRecvData);

        static uint32 WAUSB_CALLBACK DeviceArrival(PDEVICE_DESC pDeviceDesc, void* pUserArrivalData);

        static uint32 WAUSB_CALLBACK DeviceRemoval(char* pDevName, void* pUserArrivalData);

        static uint32 WAUSB_CALLBACK DeviceNotify(PDEVICE_DESC pDeviceDesc, void* pUserArrivalData);

        boolean ReadData(char* pBuf, uint32 BytesToRead, uint32* pBytesRead);
};


#endif /* #if !defined (CUSBCOMMSVR_H) */

/*---------------------------------------------------*/
/* End of File */





