/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPEvent.cpp :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CWACPEvent.h"
#include "Common.h"

/*********************************************************************************
  Default constructor
*********************************************************************************/
CWACPEvent::CWACPEvent()
{
    hHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
    bReset = FALSE;
}

/*********************************************************************************
CWACPEvent::CWACPEvent - Constructor for named Events to support Event notification
    across threads. 

Parameters:
    pName - Pointer to string that contains the name of the Event used to open 
            the same Event handle across threads. This can be NULL for non 
            named Events.

    bManualReset - Boolean flag used to specify a manual reset option for the Event.
*********************************************************************************/
CWACPEvent::CWACPEvent(TCHAR* pName, boolean bManualReset)
{
    bReset = bManualReset;
    hHandle = CreateEvent(NULL, bReset, FALSE, pName);
}

/*********************************************************************************
CWACPEvent::~CWACPEvent - Destructor. Closes and resets the Event handle.
*********************************************************************************/
CWACPEvent::~CWACPEvent()
{
    if (hHandle)
    {
        CloseHandle(hHandle);
        hHandle = NULL;
    }
}

/*********************************************************************************
CWACPEvent::Raise - Raises the event if we have a valid event handle. 
*********************************************************************************/
void CWACPEvent::Raise()
{
    if (hHandle)
    {
        SetEvent(hHandle);
    }
}

/*********************************************************************************
CWACPEvent::Reset - Resets the Event. This should only be used for events that 
    have bManualReset set to TRUE.
*********************************************************************************/
void CWACPEvent::Reset()
{
    if (hHandle && bReset)
    {
        ResetEvent(hHandle);
    }
}

/*********************************************************************************
CWACPEvent::Wait - Waits for an Event to fire

Parameters:
    Timeout - Amount of time in milliseconds to wait before returning. This can
              be set to INFINITE, which will stay blocked until the Event is fired.

Returns: 
    WAIT_SUCCESS when the Event is fired.
    WAIT_FAILURE, if the timeout occurs or the handle is invalid.
*********************************************************************************/
uint32 CWACPEvent::Wait(uint32 Timeout)
{
    uint32 Result = WAIT_FAILURE;
    
    if (hHandle)
    {
        Result = WaitForSingleObject(hHandle, Timeout);
        if (Result != WAIT_SUCCESS)
            Result = WAIT_FAILURE;
    }

    return(Result);
}


/* End of File */





