/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPMutex.cpp :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CWACPMutex.h"
#include "Common.h"

/*********************************************************************************
  Default constructor
*********************************************************************************/
CWACPMutex::CWACPMutex()
{
    hMutex = CreateMutex(NULL, FALSE, NULL);
}

/*********************************************************************************
CWACPMutex::CWACPMutex - Constructor for named Mutexes

Parameters:
    pName - Pointer to string that contains the name of the Mutex used to open 
            the same Mutex handle across threads. This can be NULL for non 
            named Mutexes.

*********************************************************************************/
CWACPMutex::CWACPMutex(TCHAR* pName)
{
    hMutex = CreateMutex(NULL, FALSE, pName);
}

/*********************************************************************************
Default Destructor. Closes and resets the Mutex handle.
*********************************************************************************/
CWACPMutex::~CWACPMutex()
{
    if (hMutex)
    {
        CloseHandle(hMutex);
        hMutex = NULL;
    }
}

/*********************************************************************************
CWACPMutex::Lock - Waits for the lock on the mutex

Parameters:
    Timeout - Amount of time in milliseconds to wait before returning. This can
              be set to INFINITE, which will stay blocked until the Mutex is locked.

Returns: 
    WAIT_SUCCESS when the Mutex is locked.
    WAIT_FAILURE, if the timeout occurs or the handle is invalid.
*********************************************************************************/
uint32 CWACPMutex::Lock(uint32 Timeout)
{
    uint32 MutexResult = WAIT_FAILURE;

    if (hMutex)
    {
        MutexResult = WaitForSingleObject(hMutex, Timeout);
        if (MutexResult != WAIT_SUCCESS)
            MutexResult = WAIT_FAILURE;
    }

    return(MutexResult);
}

/*********************************************************************************
CWACPMutex::Release - Releases the lock on the mutex

Returns: TRUE, if the mutex was released.
         FALSE, if the handle is invalid or the release fails.
*********************************************************************************/
boolean CWACPMutex::Release()
{
    boolean Result = FALSE;

    if (hMutex)
    {
        Result = (boolean)ReleaseMutex(hMutex);
    }

    return(Result);
}
    

/* End of File */





