/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFamilyStub.h                                                     //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#if !defined(CFAMILYSTUB_H)
    #define  CFAMILYSTUB_H

#include "Common.h"
#include "CMOMPBaseClass.h"
#include "CFamilyEnum.h"
#include "CConDStd.h"
#include "CBMIDStd.h"
#include "CDeviceDStd.h"
#include "CDevDPOST.h"
#include "CHeightDStd.h"
#include "CModuleDStd.h"
#include "CNumDStd.h"
#include "CRecordDService.h"
#include "CWeightDStd.h"
#include <list>

using namespace std;

    struct ConnStruct
    {
        char SourceID[ADDR_BUF_SIZE];
    };

    typedef list < ConnStruct* > CONNECTION_LIST;


class CFamilyStub : public CMOMPBaseClass
{ 
private:
    void LoadObjects();

    /* DEBUG */
    uint32  AckCount;
    uint32  ErrCount;


public:
    CFamilyStub();
    ~CFamilyStub();

    CMOMPBaseClass* AppInit();
    uint32 AppEnd();
    uint32 RecvMessage(char *pSourceID, uint32 ClassID, uint8 *pBuffer);
    uint32 RecvError(char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer);

    void AddDevice(char* pSourceID);
    void RemoveDevice(char* pSourceID);
    void ShowDevices();
    uint16 GetNumConnections();
    boolean TerminateConn(uint32 Index);
    boolean TerminateAll();

    CBMIDStd  *pCBMIDStd;
    CDeviceDStd  *pCDeviceDStd;
    CDevDPOST  *pCDevDPOST;
    CHeightDStd  *pCHeightDStd;
    CModuleDStd  *pCModuleDStd;
    CNumDStd  *pCNumDStd;
    CRecordDService  *pCRecordDService;
    CWeightDStd  *pCWeightDStd;

   /* RNDZ Status Flag */
   volatile boolean bConnected;

   /* Connection list */
   CONNECTION_LIST   SourceList;

   /* Test ByPass Configuration */
   CConDExtLst ConnList;
   volatile uint32   ConnCount;
   CConDStd*   ConnDArray[MAX_CONNECTIONS];

   /* Test server manager */
   uint16  NumConnections;
   uint16  MaxCount;

   char* pSourceID;

};

#define SEND_COMMAND(pSourceID, Family, CommandID, DataBuffer, DataSize, bEncryptionOn) SendMsg(pSourceID, Family, GnCOMMAND, CommandID, DataBuffer, DataSize, bEncryptionOn)
#define SEND_REQUEST(pSourceID, Family, RequestID, DataBuffer, DataSize, bEncryptionOn) SendMsg(pSourceID, Family, GnREQUEST, RequestID, DataBuffer, DataSize, bEncryptionOn)
#define SEND_RESPONSE(pSourceID, Family, ResponseID, DataBuffer, DataSize, bEncryptionOn) SendMsg(pSourceID, Family, GnRESPONSE, ResponseID, DataBuffer, DataSize, bEncryptionOn)
#define SEND_STATUS(pSourceID, Family, StatusID, DataBuffer, DataSize, bEncryptionOn) SendMsg(pSourceID, Family, GnSTATUS, StatusID, DataBuffer, DataSize, bEncryptionOn)
#define SEND_EVENT(pSourceID, Family, EventMsgID, DataBuffer, DataSize, bEncryptionOn) SendMsg(pSourceID, Family, GnEVENT, EventMsgID, DataBuffer, DataSize, bEncryptionOn)
#define SEND_STREAM(pSourceID, Family, StreamMsgID, DataBuffer, DataSize, bEncryptionOn) SendMsg(pSourceID, Family, GnSTREAM, StreamMsgID, DataBuffer, DataSize, bEncryptionOn)
#define SEND_DATAOBJ(pSourceID, ClassID, DataBuffer, DataSize) SendMsg(pSourceID, GetFamily(ClassID), GnDATA, GetSpecies(ClassID), DataBuffer, DataSize, FALSE)
#define SEND_TRAP(pSourceID, MsgClassID, ErrorClassID, ErrorType, ErrorCode, pData, DataSize) SendTrap(pSourceID, MsgClassID, ErrorClassID, ErrorType, ErrorCode, pData, DataSize)
#define SEND_MSGPROC_TRAP(pSourceID, ClassID, ErrorCode) SendMsgProcTrap(pSourceID, ClassID, ErrorCode)

#endif /* #if !defined(CFAMILYSTUB_H) */

/*---------------------------------------------------*/
/* End of File */

