#include <windows.h>
#include <process.h>
#include <stdio.h>
#include "CSerialCommSvr.h"
#include "CEthCommSvr.h"
#include "CFamilyEnum.h"
#include "WACP.h"
#include "SysUtil.h"
#include "DtData.h"
#include "CErrEStd.h"
#include "CDeviceDStd.h"

#include "CFmTEMPStub.h"


extern uint32 SendMsg(uint32 MsgClassID, uint8* pBuf, uint32 BufSize, uint32 ErrClassID);
extern uint32 SendTrap(uint8 ErrType, uint32 ErrClassID, uint8 ErrorCode);
extern boolean bConnected;

/*State members used for object processing*/
uint16 PostState   = 0;

/*Test members used for sample data.*/
float Temp = 0;

void SendSampleMessages(uint8 bStream);


/*'****************************************************************************
'** Disconnect 
'*****************************************************************************/
const uint8 DISCONNECT_MSG[] =
{   0x17, 0x01, 0x0c, //envelope preamble
	0x00, 0x00, 0x00, 0x1a, //envelope size
	0x01, 0x1a, //envelope port
		0x00, 0x1e, 0x01, 0x00,//message class ID 
		0x00, 0x00, 0x00, 0x07,//message package size
		0x00, //message encComp
		0x00, 0x00, 0x00, 0x00, //Message payload data size
		0x61, 0xad, //Message CRC
	0xb2, 0x78 //Envelope CRC
};

/*Thread proccess is using a C++ method. Need to type cast to WINAPI function*/
typedef unsigned (WINAPI* WACP_THREAD_PROC)
(    LPVOID lpThreadParameter);

/* Globals */

#if defined STACK_SERIAL
CSerialCommSvr  CommSvr;
#else if STACK_ETHERNET
CEthCommSvr     CommSvr;
#endif

uint32			TestID = 1;
uint32			CmdRqst = 0;
uint8			*ObjBuffer = NULL;
uint32			ObjSize = 0;



#if defined STACK_SERIAL
void StartSerial()
{
    boolean Result;

    CSerialCommSvr_Init(&CommSvr);
    Result = CSerialCommSvr_Connect(&CommSvr, (uint8*)"COM1", RNDZ_PORT_DEFAULT);

    /* Open up the communications channel */
    if (Result)
    {
		/* Launch the reader thread in the CommSvr */
        HANDLE hHandle = (HANDLE)_beginthreadex(NULL, 0,(WACP_THREAD_PROC)CSerialCommSvr_StartClient, &CommSvr, 0, NULL);
        if (hHandle)
        {
            Result = (boolean)SetThreadPriority(hHandle, THREAD_PRIORITY_HIGHEST);
            CloseHandle(hHandle);
        }
    }
}
#else if STACK_ETHERNET
/*'****************************************************************************
'** StartEthernet: Initializes a CommSvr and connects to Host
'*****************************************************************************/
void StartEthernet()
{
    boolean Result = FALSE;

    CEthCommSvr_Init(&CommSvr);
    Result = CEthCommSvr_Connect(&CommSvr,				/*Pointer to the CommSvr*/
								 (uint8*)"127.1",		/*Source ID of connection (Source ID is loopback for demo purposes*/
								 RNDZ_PORT_DEFAULT);	/*Port the RNDZ connection is listening on*/

    /* Open up the communications channel */
    if (Result)
    {
		/* Launch the reader thread in the CommSvr */
        HANDLE hHandle = (HANDLE)_beginthreadex(NULL, 0,(WACP_THREAD_PROC)CEthCommSvr_StartClient, &CommSvr, 0, NULL);
        if (hHandle)
        {
            Result = (boolean)SetThreadPriority(hHandle, THREAD_PRIORITY_HIGHEST);
            CloseHandle(hHandle);
        }
    }
}
#endif
void Menu()
{
    int     Data = 0;
    uint32  Result = 0;

	do
	{
		Data = 0;
		SYS_SLEEP(500);
		SYS_PRINTF("\n\n0. Disconnect\n");
		SYS_PRINTF("1. Get Device Description\n");
        SYS_PRINTF("2. Send ACK\n");
		SYS_PRINTF("3. Send NAK\n");
                
		scanf("%d", &Data);
		
		switch(Data)
		{
			case 0: /* Disconnect */
			{
#if defined STACK_SERIAL
				CSerialCommSvr_SendMessage(&CommSvr, DISCONNECT_MSG, sizeof(DISCONNECT_MSG));
#else if STACK_ETHERNET
				CEthCommSvr_SendMessage(&CommSvr, DISCONNECT_MSG, sizeof(DISCONNECT_MSG));
#endif             

				SYS_SLEEP(1000);

#if defined STACK_SERIAL
				CSerialCommSvr_Destroy(&CommSvr);
#else if STACK_ETHERNET
				CEthCommSvr_Destroy(&CommSvr);
#endif
				Data =0;
				break;
			}
			case 1: /* Get Device Description */
			{	
                /*Send out a request to get the device description*/
				Result = SendMsg(MakeClassID(FmDEVICE,								/*Family: FmDEVICE*/
											 GnREQUEST,								/*Genus:  GnREQUEST*/
											 DEVICE_CRST_SpGET_DEVICEDESCRIPTION),	/*Species:DEVICE_CRST_SpGET_DEVICEDESCRIPTION*/
											 NULL,									/*No Object*/
							     0,          										/*No size for object*/
								 0);		             							/*Not used*/
				break;
			}			
			case 2: /* Test ACK */
			{	
                /*Example send an ACK message */
				Result = SendTrap(CERR_SpERRNONE,							   /*Trap ErrorType; Note: Species of CERR_SpERRNONE indicates ACK*/
                                  MakeClassID(FmNIBP, GnSTATUS, 1),			   /*ClassID of the message that is acknowledged*/
								  0);										   /*Error Code: Not used*/
				break;
			}
			case 3: /* Test NAK */
			{	
                /*Example send NAK message*/
				Result = SendTrap(CERR_SpERRAPPINTERNAL,    					/*Trap ErrorType. Note: Species of CERR_SpERRAPPINTERNAL indicates Application Internal NAK*/                                 
                                 MakeClassID(FmNIBP, GnSTATUS, 1),				/*ClassID of the message that is NAK'd*/
								 33);											/*ErrorCode: 33 Application defined error code*/
				break;
			}
			default:
			{
				Data = -1;
				SYS_PRINTF("\n\n---UNSUPPORTED---\n");
			}

		}//End Switch

	}while (Data != 0); //End Do
}


int main()
{

    /* Start up transport and send out Rendezvous Connection*/
#if defined STACK_SERIAL
    StartSerial();
#else if STACK_ETHERNET
    StartEthernet();
#endif


    while(bConnected == FALSE)
    {
        SYS_SLEEP(200);
    }

    Menu();

}

