/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmDATATRANSFERStub.c                                             //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmDATATRANSFERStub.h"
#include "CDataTransferCDesc.h"
#include "CDataTransferDExtStd.h"
#include "CFileSysPSelector.h"
#include "DDS_MERLION.h"

#define BDD_MAX_PACKET_SIZE 64
#define BDD_LENGTH DDS_MERLION_BDD_SIZE
#define bdd_data DDS_MERLION_BDD

static CDataTransferCDesc DataTransferCDesc; //these are strategetically initialized, DO NOT initialize
static CDataTransferDExtStd DataTransferDExtStd; //these are strategetically initialized, DO NOT initialize
static uint8 datatransfer_state = 0;
static const ansichar FileName[] = "\\Device.bdds";
#define FILENAME_SIZE 12

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmDATATRANSFERStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmDATATRANSFER, Genus, Species);
	uint8* ObjBuffer = NULL;
	uint32  ObjSize = 0;
	uint32 sequence_number;
	CFileSysPSelector FileSysPSelector;
	uint8 i;

    TOUCH(Buffer);
	
	CFileSysPSelector_Init(&FileSysPSelector);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case DATATRANSFER_CCMD_SpEXECUTE_DTT:
					/*	Data Transmission Terminate:
						ACK;	
						reset datatransfer_state;
					*/ 
					SendTrap(CERR_SpERRNONE, MsgClassID, 0);		
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 0;
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case DATATRANSFER_CRST_SpGET_CONFIG:
					/*	Get Config:
						Deserialize;
						Validate data;
						Verify role;
						Set BDD length, packet size;
						Set data format, data endianness;
						Send response;
						Set datatransfer_state;
					*/
					CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
					CDataTransferCDesc_Init(&DataTransferCDesc);
					if (CDataTransferCDesc_Deserialize(&DataTransferCDesc, Buffer))
					{
						if (DataTransferCDesc.m_Role == CDTACDSC_Role_DATA_RECIPIENT)
							DataTransferCDesc.m_Role = CDTACDSC_Role_DATA_SOURCE;
						else
						{
							ErrorCode = CERR_PROC_UNSUPPORTED;
							break;
						}
						
						if ((NULL != DataTransferCDesc.m_Descriptor) && (DataTransferCDesc.m_DescriptorSize > 0))
						{
							ObjBuffer = CDataTransferCDesc_GetDescriptorAt(&DataTransferCDesc, 0, &ObjSize);
							if (!CFileSysPSelector_Deserialize(&FileSysPSelector, ObjBuffer))
							{
								ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
								break;
							}
							ObjBuffer = NULL;
							ErrorCode = CERR_PROC_NONE;
							for (i = 0; i < FILENAME_SIZE; i++)
							{
								if (FileName[i] != (FileSysPSelector.m_PathName[i] & 0xFF))
								{
									ErrorCode = CERR_PROC_INVL_PARAM;
									break;
								}
							}
							if (ErrorCode)
							{
								ErrorCode = CERR_PROC_CONTEXT;
								break;
							}
							CFileSysPSelector_Reset(&FileSysPSelector, TRUE, TRUE);
						}
						else
						{
							ErrorCode = CERR_PROC_UNSUPPORTED;
							break;
						}
						DataTransferCDesc.m_InitSeqNumber = 1;
						DataTransferCDesc.m_TotalDataSize = BDD_LENGTH;
						if ((0 == DataTransferCDesc.m_DataPacketSize) ||
						    (DataTransferCDesc.m_DataPacketSize > BDD_MAX_PACKET_SIZE))
							DataTransferCDesc.m_DataPacketSize = BDD_MAX_PACKET_SIZE;

	                    /* Get object size */
	                    ObjSize = CDataTransferCDesc_SizeAll(&DataTransferCDesc);

	                    /* Allocate memory */
	                    if((ObjBuffer = (uint8*)SYS_REALLOC(ObjBuffer,ObjSize)) != NULL)
	                    {
	                        /* Serialize object */
	                        if (CDataTransferCDesc_Serialize(&DataTransferCDesc,ObjBuffer))
	                        {
							/* Send serialized object */
							SendMsg(MakeClassID(FmDATATRANSFER,
												GnRESPONSE,
												DATATRANSFER_CRSP_SpPUT_CONFIG),
												ObjBuffer,									// Serialized CDeviceDStd object
												ObjSize,									// Size of Serialized CDeviceDStd object
												0);									     	// ErrorClsID: 0
								ErrorCode = CERR_PROC_NONE;
								datatransfer_state = 1;
	                        }
	                        else
	                        {
	                            ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
	                        }
	                        SYS_FREE(ObjBuffer);
	                    }
	                    else
	                    {
	                        ErrorCode = CERR_PROC_MEMORY;
	                    }
					}
					else
					{
						ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
					}
                    break;
                case DATATRANSFER_CRST_SpGET_DATA:
					/*	Get Data:
						Verify that DTR and Get config has happened (datatransfer_state = 2);
						Deserialize;
						Validate data;
						Verify Indentifier;
						Validate sequence number;
						Init Object;
						Set Sequence, Idenifier, Data Size and Data pointer;
						Send Response;
					*/
					if (datatransfer_state != 2)
					{
						ErrorCode = CERR_PROC_CONTEXT;
						break;
					}
					CDataTransferDExtStd_Init(&DataTransferDExtStd);
					if (CDataTransferDExtStd_Deserialize(&DataTransferDExtStd, Buffer))
					{
						if (DataTransferDExtStd.m_Identifier != DataTransferCDesc.m_Identifier)
						{
							ErrorCode = CERR_PROC_CONTEXT;
							break;
						}
						if (0 == DataTransferDExtStd.m_SeqNumber || DataTransferDExtStd.m_SeqNumber > BDD_LENGTH)
						{
							ErrorCode = CERR_PROC_CONTEXT;
							//CDataDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
							break;
						}
						sequence_number = DataTransferDExtStd.m_SeqNumber;
						CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
						CDataTransferDExtStd_Init(&DataTransferDExtStd);
						DataTransferDExtStd.m_SeqNumber = sequence_number;
						DataTransferDExtStd.m_Identifier = DataTransferCDesc.m_Identifier;
						DataTransferDExtStd.m_DataSize = (sequence_number + DataTransferCDesc.m_DataPacketSize) > BDD_LENGTH ?
												BDD_LENGTH - sequence_number + 1 :
												DataTransferCDesc.m_DataPacketSize;
						DataTransferDExtStd.m_Data = &(bdd_data[sequence_number - 1]);
	                    /* Get object size */
	                    ObjSize = CDataTransferDExtStd_SizeAll(&DataTransferDExtStd);

	                    /* Allocate memory */
	                    if((ObjBuffer = (uint8*)SYS_REALLOC(ObjBuffer,ObjSize)) != NULL)
	                    {
	                        /* Serialize object */
	                        if (CDataTransferDExtStd_Serialize(&DataTransferDExtStd,ObjBuffer))
	                        {
							/* Send serialized object */
							SendMsg(MakeClassID(FmDATATRANSFER,
												GnRESPONSE,
												DATATRANSFER_CRSP_SpPUT_DATA),
									ObjBuffer,									// Serialized object
									ObjSize,									// Size of Serialized object
									0);									     	// ErrorClsID: 0
								ErrorCode = CERR_PROC_NONE;
	                        }
	                        else
	                        {
	                            ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
	                        }
	                        SYS_FREE(ObjBuffer);
	                    }
	                    else
	                    {
	                        ErrorCode = CERR_PROC_MEMORY;
	                    }
					}
					//CDataDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case DATATRANSFER_CRSP_SpPUT_CONFIG:
                    break;
                case DATATRANSFER_CRSP_SpPUT_DATA:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case DATATRANSFER_CSS_SpREPORT_DTR:
					/* 	Data Transmission Ready:
						Verify that Get config has happened (datatransfer_state = 1);
						Deserialize;
						Validate data;
						Verify that Transmission Identifier matches;
						ACK;
					*/
					if (datatransfer_state != 1)
					{
						ErrorCode = CERR_PROC_CONTEXT;
						break;
					}
					CDataTransferDExtStd_Init(&DataTransferDExtStd);
					if (CDataTransferDExtStd_Deserialize(&DataTransferDExtStd, Buffer))
					{
						if (DataTransferDExtStd.m_Identifier != DataTransferCDesc.m_Identifier)
						{
							ErrorCode = CERR_PROC_CONTEXT;
							CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
							break;
						}
						SendTrap(CERR_SpERRNONE, MsgClassID, 0);		
						ErrorCode = CERR_PROC_NONE;
						datatransfer_state = 2;
					}
					CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
                    break;
                case DATATRANSFER_CSS_SpREPORT_DTC:
					/* 	Data Transmission Complete:
						ACK;
						reset datatransfer_state;
					*/
					SendTrap(CERR_SpERRNONE, MsgClassID, 0);		
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 0;
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

     /* Any Errors? */
    if (ErrorCode == CERR_PROC_NONE)
    {
        MsgResult = TRUE;
    }

//	/* Send a TRAP message if we have an application error */
//	if (APP_INTERNAL_ERROR(ErrorCode))
//	{
//		/* Clear the error offset */
//		CLEAR_APP_INTERNAL_ERROR_OFFSET(ErrorCode);
//
//		/* Return raw error code with CERR_SpERRAPPINTERNAL Species */
//		SendTrap(CERR_SpERRAPPINTERNAL, MsgClassID, ErrorCode);
//
//		MsgResult = TRUE;
//	}

   /* Send a TRAP message if we have a unimplemented or unsupported error */
	if (!MsgResult)
	{
		SendTrap(CERR_SpERRMSGPROC, MsgClassID, ErrorCode);
	}

    return MsgResult;
}
