/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmFLASHStub.c                                                    //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmFLASHStub.h"

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmFLASHStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmFLASH, Genus, Species);

    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case FLASH_CCMD_SpWRITE_SEGMENT:
                    break;
                case FLASH_CCMD_SpERASE_SECTOR:
                    break;
                case FLASH_CCMD_SpENABLE_FLASH:
                    break;
                case FLASH_CCMD_SpDISABLE_FLASH:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case FLASH_CRST_SpGET_CRC:
                    break;
                case FLASH_CRST_SpGET_SEGMENT:
                    break;
                case FLASH_CRST_SpGT_FLASH_CONFIG:
                    break;
                case FLASH_CRST_SpGET_SECTOR_CONFIG:
                    break;
                case FLASH_CRST_SpGET_UPGRADE:
                    break;
                case FLASH_CRST_SpGET_FLASH_PREPARATION:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case FLASH_CRSP_SpPUT_CRC:
                    break;
                case FLASH_CRSP_SpPUT_SEGMENT:
                    break;
                case FLASH_CRSP_SpPUT_FLASH_CONFIG:
                    break;
                case FLASH_CRSP_SpPUT_SECTOR_CONFIG:
                    break;
                case FLASH_CRSP_SpPUT_UPGRADE:
                    break;
                case FLASH_CRSP_SpPUT_FLASH_PREPARATION:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case FLASH_CSS_SpFLASH_READY:
                    break;
                case FLASH_CSS_SpFLASH_COMPLETE:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnDATA:
        {
            switch(Species)
            {
                case CFLSDCHK_SpFLASHCHK:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
	{
		SendTrap(CERR_SpERRMSGPROC, MsgClassID, ErrorCode);
	}

    return MsgResult;
}

