/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmMODULEStub.c                                                   //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmMODULEStub.h"

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmMODULEStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
	uint8   ErrorType  = CERR_SpERRNONE;
    uint32  MsgClassID = MakeClassID(FmMODULE, Genus, Species);
	CModulePSelector Selector;
	uint8 ObjBuffer[128];
	
    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case MODULE_CCMD_SpWRITE_CONFIG:
                    break;
                case MODULE_CCMD_SpWRITE_MODULE_DESC:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case MODULE_CRST_SpGET_MODULE_DESC:
				{				
					CModuleDStd ModuleDStd;
					MsgResult = TRUE;
					ErrorCode = CERR_NONE;

					if(Buffer != NULL)
					{
						CModulePSelector_Init(&Selector);

						if(CModulePSelector_Deserialize(&Selector, Buffer))
						{
							CModuleDStd_Init(&ModuleDStd);

							/* Re-set the species to the selector value */
							SetSpecies(&ModuleDStd.ClassID, (uint8)Selector.m_SpeciesSelector);

							switch (Selector.m_SpeciesSelector)
							{
								case CMODDSTD_SpHOST:
								case CMODDSTD_SpSCALE:
								{
									memcpy(ModuleDStd.m_Description, "BraunPro XXXXX", strlen("BraunPro XXXXX"));
									memcpy(ModuleDStd.m_Version, "ToDo", strlen("ToDo"));
									break;
								}
								case CMODDSTD_SpHARDWARE:
								{
									memcpy(ModuleDStd.m_Description, "BraunPro XXXXX Hardware", strlen("BraunPro XXXXX Hardware"));
									memcpy(ModuleDStd.m_Version, "ToDo", strlen("ToDo"));
									break;
								}								
								case CMODDSTD_SpBOOTLOADER:
								{
									memcpy(ModuleDStd.m_Description, "BraunPro XXXXX Bootloader", strlen("BraunPro XXXXX Bootloader"));
									memcpy(ModuleDStd.m_Version, "ToDo", strlen("ToDo"));
									break;
								}
								default:
								{
									/*Invalid Paramter*/
									ErrorCode = CERR_PROC_INVL_PARAM;
									ErrorType = CERR_SpERRMSGPROC;
									break;
								}
							}

							if(ErrorCode == CERR_NONE)
							{
								 /* Serialize object */
								if (CModuleDStd_Serialize(&ModuleDStd, ObjBuffer))
								{
									/*Send Message*/
									SendMsg(MakeClassID(FmMODULE,						/*Family*/
												 GnRESPONSE,							/*Genus*/
												 MODULE_CRSP_SpPUT_MODULE_DESC),		/*Species*/
												 ObjBuffer,								/*Serialized Object*/
												 CModuleDStd_SizeAll(&ModuleDStd),   	/*Size of object*/
												 0);           							/*Not used*/

								}
								else
								{
									/*Serialization Failure*/
									ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
									ErrorType = CERR_SpERRMSGPROC;
								}
							}							
						}
						else
						{
							/*Deserialize Failure*/
							ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
							ErrorType = CERR_SpERRMSGPROC;
						}
					}
					else
					{
						/*Invalid Paramter*/
						ErrorCode = CERR_PROC_INVL_PARAM;
						ErrorType = CERR_SpERRMSGPROC;
					}

					/*Send a NAK if there is an ErrorCode*/
					if(ErrorCode != CERR_NONE)
					{
						 SendTrap(ErrorType,		/*Trap ErrorType*/
                                  MsgClassID,		/*ClassID of the message that is NAK'd*/
								  ErrorCode);		/*Error Code: Not used*/
					}
                    break;
				}
                case MODULE_CRST_SpGET_CONFIG:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case MODULE_CRSP_SpPUT_MODULE_DESC:
                    break;
                case MODULE_CRSP_SpPUT_CONFIG:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnEVENT:
        {
            switch(Species)
            {
                case MODULE_CEVNT_SpHW_MOD_CONNECT:
                    break;
                case MODULE_CEVNT_SpHW_MOD_DISCONNECT:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnDATA:
        {
            switch(Species)
            {
                case CMODDSTD_SpSTANDARD:
                    break;
                case CMODDSTD_SpHOST:
                    break;
                case CMODDSTD_SpHARDWARE:
                    break;
                case CMODDSTD_SpBOOTLOADER:
                    break;
                case CMODDSTD_SpBOOTRECOVERY:
                    break;
                case CMODDSTD_SpINITBOOTLOADER:
                    break;
                case CMODDSTD_SpOPERATINGSYSTEM:
                    break;
                case CMODDSTD_SpFONT:
                    break;
                case CMODDSTD_SpAUXILIARYHOST:
                    break;
                case CMODDSTD_SpAUXILIARYHARDWARE:
                    break;
                case CMODDSTD_SpAUXILIARYBOOTLOADER:
                    break;
                case CMODDSTD_SpAUXILIARYOPERATINGSYSTEM:
                    break;
                case CMODDSTD_SpNIBPMODULE:
                    break;
                case CMODDSTD_SpNIBPSAFETYALG:
                    break;
                case CMODDSTD_SpNIBPSTEPALG:
                    break;
                case CMODDSTD_SpNIBPFASTALG:
                    break;
                case CMODDSTD_SpNIBPNEONATSTEPALG:
                    break;
                case CMODDSTD_SpNIBPPOEMALG:
                    break;
                case CMODDSTD_SpNIBPSTEPAOALG:
                    break;
                case CMODDSTD_SpTEMPMODULE:
                    break;
                case CMODDSTD_SpSURETEMPALG:
                    break;
                case CMODDSTD_SpSPO2MODULE:
                    break;
                case CMODDSTD_SpECGMODULE:
                    break;
                case CMODDSTD_SpMEANS:
                    break;
                case CMODDSTD_SpSPIROMETRY:
                    break;
                case CMODDSTD_SpFRONTEND:
                    break;
                case CMODDSTD_SpPPP:
                    break;
                case CMODDSTD_SpSECUREDIGITALCARD:
                    break;
                case CMODDSTD_SpPRINTER:
                    break;
                case CMODDSTD_SpCOMMSMODULE:
                    break;
                case CMODDSTD_SpRADIO:
                    break;
                case CMODDSTD_SpBARCODE:
                    break;
                case CMODDSTD_SpSCALE:
                    break;
                case CMODDSTD_SpBATTERY:
                    break;
                case CMODDSTD_SpALL:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
    {
		SendTrap(CERR_SpERRMSGPROC, MsgClassID, ErrorCode);		
    }

    return MsgResult;
}

