/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmRECORDStub.c                                                   //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmRECORDStub.h"
#include "CRecordPSelector.h"

uint16  ErrorCode  = CERR_PROC_UNIMPL;
uint16  ErrorType  = CERR_PROC_UNIMPL;
/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmRECORDStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;

    uint32  MsgClassID = MakeClassID(FmRECORD, Genus, Species);
	uint8   ObjBuffer[128];

    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case RECORD_CCMD_SpWRITE_CONFIG:
                    break;
                case RECORD_CCMD_SpWRITE_RECORD_COUNT:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case RECORD_CRST_SpGET_RECORD_COUNT:
				{				
					CRecordDStd RecordDStd;
					CRecordPSelector SpeciesSelector;
					MsgResult = TRUE;
					ErrorCode = CERR_NONE;

					if(Buffer != NULL)
					{
						CRecordPSelector_Init(&SpeciesSelector);

						if(CRecordDStd_Deserialize(&SpeciesSelector, Buffer))
						{	
							CRecordDStd_Init(&RecordDStd);
							GetCRecordDStd(&RecordDStd, SpeciesSelector.m_SpeciesSelector);
						}
						else
						{
							/*Deserialize Failure*/
							ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
							ErrorType = CERR_SpERRMSGPROC;
						}
					}
					else
					{
						ErrorCode = CERR_PROC_INVL_PARAM;
						ErrorType = CERR_SpERRMSGPROC; 
					}

					/*Send object if there is no error code*/
					if(ErrorCode == CERR_NONE)
					{
						/* Serialize object */
						if (CRecordDStd_Serialize(&RecordDStd, ObjBuffer))
						{
							/*Send Message*/
							SendMsg(MakeClassID(FmRECORD,						   /*Family*/
										 GnRESPONSE,							   /*Genus*/
										 RECORD_CRSP_SpPUT_RECORD_COUNT),		   /*Species*/
										 ObjBuffer,								   /*Serialized Object*/
										 CRecordDStd_SizeAll(&RecordDStd), /*Size of object*/
										 0);           							   /*Not used*/

						}
						else
						{
							/*Serialization Failure*/
							ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
							ErrorType = CERR_SpERRMSGPROC;
						}
					}

					/*Send a NAK if there is an ErrorCode*/
					else
					{
						 SendTrap(ErrorType,		/*Trap ErrorType*/
                                  MsgClassID,		/*ClassID of the message that is NAK'd*/
								  ErrorCode);		/*Error Code: Not used*/
					}
                    break;
				}
                    break;
                case RECORD_CRST_SpGET_CONFIG:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case RECORD_CRSP_SpPUT_RECORD_COUNT:
                    break;
                case RECORD_CRSP_SpPUT_CONFIG:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case RECORD_CSS_SpREPORT_RECORD_COUNT:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnDATA:
        {
            switch(Species)
            {
                case CRCDDSTD_SpSTANDARD:
                    break;
                case CRCDDSTD_SpNIBP:
                    break;
                case CRCDDSTD_SpTEMPERATURE:
                    break;
                case CRCDDSTD_SpSPO2:
                    break;
                case CRCDDSTD_SpHEIGHT:
                    break;
                case CRCDDSTD_SpWEIGHT:
                    break;
                case CRCDDSTD_SpCO2:
                    break;
                case CRCDDSTD_SpRESTING_ECG:
                    break;
                case CRCDDSTD_SpSTRESS_ECG:
                    break;
                case CRCDDSTD_SpSPIROMETRY:
                    break;
                case CRCDDSTD_SpTEMPERATURE_PROBE:
                    break;
                case CRCDDSTD_SpBATTERY:
                    break;
                case CRCDDSTD_SpHOURS:
                    break;
                case CRCDDSTD_SpTEMPERATURE_DOCK:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
    {
        SendTrap(CERR_SpERRMSGPROC, MsgClassID, 0);
    }

    return MsgResult;
}
boolean GetCRecordDStd(CRecordDStd* pObject, uint32 SpeciesSelector)
{
	boolean bResult = FALSE;

	if(pObject)
	{
		bResult = TRUE;

		switch(SpeciesSelector)
		{
			case CRCDDSTD_SpTEMPERATURE_PROBE:
			{
				pObject->m_RecordCount = 0;//To Do
				break;
			}
			case CRCDDSTD_SpTEMPERATURE_DOCK:
			{
				pObject->m_RecordCount = 0;//To Do
				break;
			}
			default:
			{
				/*Invalid Paramter*/
				ErrorCode = CERR_PROC_INVL_PARAM;
				ErrorType = CERR_SpERRMSGPROC;
				break;
			}
		}
	}

	return bResult;
}
