/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmTEMPStub.c                                                     //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmTEMPStub.h"

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmTEMPStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmTEMP, Genus, Species);

    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case TEMP_CCMD_SpWRITE_CONFIG:
                    break;
                case TEMP_CCMD_SpMSG_PUSH_ENABLE:
                    break;
                case TEMP_CCMD_SpMSG_SET_ALG:
                    break;
                case TEMP_CCMD_SpMSG_SET_MODE:
                    break;
                case TEMP_CCMD_SpRESET_DEVICE:
                    break;
                case TEMP_CCMD_SpSET_CALIBRATION:
                    break;
                case TEMP_CCMD_SpSAVE_CALIBRATION:
                    break;
                case TEMP_CCMD_SpLOAD_CALIBRATION:
                    break;
                case TEMP_CCMD_SpRESTORE_DFLT_CALIBRATION:
                    break;
                case TEMP_CCMD_SpSET_SETTINGS:
                    break;
                case TEMP_CCMD_SpSAVE_SETTINGS:
                    break;
                case TEMP_CCMD_SpLOAD_SETTINGS:
                    break;
                case TEMP_CCMD_SpRESTORE_DFLT_SETTINGS:
                    break;
                case TEMP_CCMD_SpSET_HARDWARE_STATE:
                    break;
                case TEMP_CCMD_SpSET_TEST_LOCK:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case TEMP_CRST_SpGET_TEMP:
                    break;
                case TEMP_CRST_SpGET_CONFIG:
                    break;
                case TEMP_CRST_SpGET_ENG_STATUS:
                    break;
                case TEMP_CRST_SpSTATUS:
                    break;
                case TEMP_CRST_SpGET_PROBE_INFO:
                    break;
                case TEMP_CRST_SpGET_CALIBRATION:
                    break;
                case TEMP_CRST_SpGET_SETTINGS:
                    break;
                case TEMP_CRST_SpGET_HARDWARE_STATE:
                    break;
                case TEMP_CRST_SpGET_TEST_LOCK:
                    break;
                case TEMP_CRST_SpGET_RAW_DATA:
                    break;
                case TEMP_CRST_SpGET_SAMPLE:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case TEMP_CRSP_SpPUT_TEMP:
                    break;
                case TEMP_CRSP_SpPUT_CONFIG:
                    break;
                case TEMP_CRSP_SpPUT_ENG_STATUS:
                    break;
                case TEMP_CRSP_SpPUT_STATUS:
                    break;
                case TEMP_CRSP_SpPUT_PROBE_INFO:
                    break;
                case TEMP_CRSP_SpPUT_CALIBRATION:
                    break;
                case TEMP_CRSP_SpPUT_SETTINGS:
                    break;
                case TEMP_CRSP_SpPUT_HARDWARE_STATE:
                    break;
                case TEMP_CRSP_SpPUT_TEST_LOCK:
                    break;
                case TEMP_CRSP_SpPUT_RAW_DATA:
                    break;
                case TEMP_CRSP_SpPUT_SAMPLE:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case TEMP_CSS_SpREPORT_TEMPERATURE:
                    //handle the recieve temp obj
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTREAM:
        {
            switch(Species)
            {
                case TEMP_CSTR_SpREPORT_ENG_STATUS:
                    break;
                case TEMP_CSTR_SpREPORT_STATUS:
                    break;
                case TEMP_CSTR_SpREPORT_SAMPLE:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnDATA:
        {
            switch(Species)
            {
                case CTMPDDTA_SpDATA:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
	{
		SendTrap(CERR_SpERRMSGPROC, MsgClassID, ErrorCode);
	}

    return MsgResult;
}

