/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmTRAPStub.c                                                     //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmTRAPStub.h"

static uint32   AckCount;
static uint32   ErrCount;

/*================================================================================
CFmTRAPStub_RecvMsg : Handles Error messages.
================================================================================*/
boolean CFmTRAPStub_RecvMsg(const char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8* Buffer)
{
    boolean Result  = TRUE;
    uint16  Family  = (uint16)(ClassID >> 16);
    uint8   Genus   = (uint8)(ClassID >> 8);
    uint8   Species = (uint8)ClassID;

    if(ErrorType == CERR_SpERRNONE)
    {
        SYS_PRINTF("\nRxError: %s ACK #%d F:%d G:%d S:%d\n", pSourceID, ++AckCount, Family, Genus, Species);
    }
    else 
    {
        if (Buffer)
        {
            /* Handle descriptive NAK notification */
            CErrEStd ErrObj;
            uint8    ErrSpecies = GetSpecies(Buffer);

            CErrEStd_Init(&ErrObj);
            SetSpecies(&ErrObj.ClassID, ErrSpecies);

            if (ErrSpecies == CERR_SpERRWACPSTACK)
                SYS_PRINTF("\nRxError: %s WACPStack error\n", pSourceID);
            else if (ErrSpecies == CERR_SpERRMSGPROC)
                SYS_PRINTF("\nRxError: %s Message Processing error\n", pSourceID);
            else if (ErrSpecies == CERR_SpERRAPPINTERNAL)
                SYS_PRINTF("\nRxError: %s Application error\n", pSourceID);
            else
                SYS_PRINTF("\nRxError: %s Unknown Error type\n", pSourceID);

            if (CErrEStd_Deserialize(&ErrObj, Buffer))
            {
                Family  = (uint16)(ErrObj.m_ErrClassID >> 16);
                Genus   = (uint8)(ErrObj.m_ErrClassID >> 8);
                Species = (uint8)ErrObj.m_ErrClassID;

                SYS_PRINTF("\nRxError: %s NAK #%d F:%d G:%d S:%d\n", pSourceID, ++ErrCount, Family, Genus, Species);
                SYS_PRINTF("\nRxError: %s ErrorCode: %d, Desc:%s, Data: %s\n", pSourceID, ErrObj.m_ErrorCode,
                    ErrObj.m_Description, ErrObj.m_Data);

                /* Call Reset to free up any dynamic memory created in the deserialize method */
                CErrEStd_Reset(&ErrObj, FALSE, FALSE);

            }
            else
                SYS_PRINTF("\nRxError: %s Failed to Deserialize ErrEStd object\n", pSourceID);
        }
        else
        {
            /* Handle simple NAK notification */
            SYS_PRINTF("\nRxError: %s NAK #%d F:%d G:%d S:%d\n", pSourceID, ++ErrCount, Family, Genus, Species);
        }
    }

    return Result;
}

