#include "..\..\Utils\Utils.h"
#include "..\..\General\CSysSettings.h"
#include "..\..\Utils\TBitArray.h"
#include "..\..\Host_Interface\CHostInterface.h"
#include "..\..\HAL\HAL.h"
#include "CWACPCommProtocol.h"
#include "Wacp.h"

#define DEFAULT_WAVE_MSG_PERIOD (TIMER_TICK * 2)		/*  Default wave msg period */

////////////////////////////////////////////////
///  TEMP CODE
///
/////////////////////////////////////

extern "C"
{
	#include "CWaveSampleDStd.h"
	#include "CWaveSampleAStd.h"
	#include "CFamilyEnum.h"
}

// Operation implementations

UINT ReadSerialData (uint8* pBuffer, UINT nMaxSize)
{
	UINT nActualSize = 0;
	while (USB_DRIVER.IsReadReady() && (nActualSize < nMaxSize))
	{
		*pBuffer++ = USB_DRIVER.Read();
		 nActualSize++;
	}

	return nActualSize;
}

extern "C"
{

void WriteSerialData (uint8 const* pBuf, uint32 BufSize)
{
	while (BufSize--)
	{
		USB_DRIVER.Write (*pBuf++);
	}
}

void WacpStubEnterDownloadMode()
{
	USB_DRIVER.StartTx();
	
	while (USB_DRIVER.GetLeftToSend() != 0)
	{
		RESET_HW_WATCHDOG();
	}

	HI.SwitchToDwlMode();
}

}

CWACPCommProtocol::CWACPCommProtocol()
{

}

// This is a template method invoked when the system starts the task.
void CWACPCommProtocol::OnCreate()
{
	CTask::OnCreate();

	// Set setup defaults
	m_bWasBTPSTouched = false;
	m_bWasPeakPickingTouched = false;

	m_bIsBTPSOn = true;

	DEFINE_HI;
	hi.ApplyBTPS ();	// BTPS default is on

	hi.SetPeakPicking (CHostInterface::e_PEAK_PICKING_20SEC);
	hi.SetPatientStatus (CHostInterface::e_PATIENT_ADULT);
	hi.SetCO2Units (CHostInterface::e_UNITS_MMHG);

	m_bIsDistinguishableBreathMsg = false;
	m_byBreathPeriodicMsgPeriod = 0;		// No periodic breath message

	m_byAutoStandbyDelay = 0;			// Automatic standby disabled
	m_wLastConfigItemValue = 0;

	m_nWaveMsgindex = 0;
	
	// Init opmode to measurement mode
	m_bIsCalibMode = false;

	// Create the low-level comm. driver

	m_bIsCommEstablished = false;

	USB_DRIVER.GetInstance();				// Create USB driver object

	USB_DRIVER.StartRx();
	USB_DRIVER.StartTx();
}


void CWACPCommProtocol::DoActivities()
{
	if (IsCommEstablished() == false)
	{
		if (USB_DRIVER.IsCommReady()
			  &&
			  m_tmFastPeriodicMsg.IsTimeout())
		{
			m_bIsCommEstablished = true;
		}
		else
		{
			#ifdef USB_COMM
			if (&USB_DRIVER != 0)			// If USB comm mode
			{
				USB_DRIVER.StartRx();		// Start RX process in case it wasn't initiated
			}
			#endif

			return;
		}
	}

	DEFINE_HI;


	//////////////////////////////////////////////////////////////////
	//   Send Periodic Messages
	//////////////////////////////////////////////////////////////////

	///////////////////////////////////////////
	// CO2 Wave

	m_fCO2ForComm = hi.GetCO2();

	if (m_tmFastPeriodicMsg.HasPassed (TIME_MSECONDS (100)))
	{
		m_tmFastPeriodicMsg.ResetToNow();

		CWaveSampleDStd WaveSampleDStd;
		uint8		ObjBuffer [128];
		uint32		ObjSize = 0;

		/*Reset object fields*/
		CWaveSampleDStd_Init(&WaveSampleDStd);

		WaveSampleDStd.m_SequenceNumber = m_nWaveMsgindex++; /* The sequence number of the wave sample set. A sequence number of zero represents that the data within the sample represents a complete wave. */
		WaveSampleDStd.m_Quality = (hi.IsCO2Valid() ?
										(hi.IsCO2OverRange() ?
											CWSPDSTD_Quality_DUBIOUS
											:
											CWSPDSTD_Quality_VALID)
										:
										CWSPDSTD_Quality_INVALID
								   );

		WORD wWave[2];
		wWave [0] = WORD (m_fCO2ForComm * 10.0f);
		wWave [1] = WORD (hi.GetCO2() * 10.0f);
		WaveSampleDStd.m_Sample = ((uint8*) &wWave);
		WaveSampleDStd.m_SampleSize = sizeof (wWave);
		
		CWaveSampleDStd_Serialize(&WaveSampleDStd ,ObjBuffer);	

		/*Calculate the size of the object*/
		ObjSize = CWaveSampleDStd_SizeAll(&WaveSampleDStd);

		/*Sending request message*/
		SendMsg(MakeClassID(FmWAVESAMPLE,
							GnSTREAM,
							WAVESAMPLE_CSTR_SpSTREAM_WAVE),
				ObjBuffer,									/*Serialized CWaveSampleDStd object*/
				ObjSize,									/*Size of Serialized CWaveSampleDStd object*/
				0);									     	/*ErrorClsID: 0*/
	}
	else
	{
		m_fCO2Prev = hi.GetCO2();
	}

	///////////////////////////////////////////
	// Annotations

	if (hi.IsEndOfBreath())
	{
		CWaveSampleAStd WaveSampleAStd;
		uint8		ObjBuffer [128];
		uint32		ObjSize = 0;

		/*Reset object fields*/
		CWaveSampleAStd_Init(&WaveSampleAStd);
		WaveSampleAStd.ClassID = (uint32)MakeClassID(FmWAVESAMPLE, GnANNOTATION, CWVSPASTD_SpCO2);

		WaveSampleAStd.m_SequenceNumber = m_nWaveMsgindex; /* The sequence number of the wave sample set. A sequence number of zero represents that the data within the sample represents a complete wave. */
		WaveSampleAStd.m_SampleIndex = m_nWaveIndex; /* The sequence number of the wave sample set. A sequence number of zero represents that the data within the sample represents a complete wave. */
		WaveSampleAStd.m_Annotation = 55; /* The sequence number of the wave sample set. A sequence number of zero represents that the data within the sample represents a complete wave. */

		CWaveSampleAStd_Serialize(&WaveSampleAStd ,ObjBuffer);	

		/*Calculate the size of the object*/
		ObjSize = CWaveSampleAStd_SizeAll(&WaveSampleAStd);

		/*Sending request message*/
		SendMsg(MakeClassID(FmWAVESAMPLE,
							GnSTREAM,
							WAVESAMPLE_CSTR_SpSTREAM_WAVE),
				ObjBuffer,									/*Serialized CWaveSampleAStd object*/
				ObjSize,									/*Size of Serialized CWaveSampleAStd object*/
				0);									     	/*ErrorClsID: 0*/
	}

	///////////////////////////////////////////
	// Breath and Smart Capnography Numerics
	if (m_tmFastPeriodicMsg.HasPassed (TIME_SECONDS (1)))
	{
		
	}

	m_nWaveIndex = 	(m_nWaveIndex + 1) % 2;

	//////////////////////////////////////////////////////////////////
	//   Process received commands
	//////////////////////////////////////////////////////////////////

	static UINT Index = 0;
	static uint8 Buffer[MAX_PACKET_SIZE];
	
	while (true)
	{
		UINT MsgSize = ReadSerialData (Buffer + Index, (MAX_PACKET_SIZE - Index));

		if (MsgSize > 0)
		{
			/* Pass the data to the WACP Parser */
			static char pszSourceID[] = "USB";
			uint32 PacketLen;

			UINT BytesProcessed = WACPParser (pszSourceID, Buffer, MsgSize + Index, &PacketLen);

			if (BytesProcessed == 0)
			{
				/* Partial packet, increment the Index and continue to read data */
				Index += MsgSize;
			}
			else if (BytesProcessed == (MsgSize + Index))
			{
				/* Full packet received, reset Index to 0 */
				Index = 0;
				memset (Buffer, 0, MAX_PACKET_SIZE);
			}
			else if (BytesProcessed < (uint32)MsgSize + Index)
			{
				/* Multiple packets, with partial packet found, copy over processed packet 
				 * and set Index to the first unprocessed packet byte
				 */
				Index = (MsgSize + Index - BytesProcessed);
				memcpy (Buffer, Buffer + BytesProcessed, Index);
			}
		}
		else
		{
			break;		// No available data
		}
	}

	USB_DRIVER.StartRx();		// Start RX process in case it wasn't initiated
	USB_DRIVER.StartTx();		// Start sending in case TX was complete
}

bool CWACPCommProtocol::IsCommEstablished() const
{
	return m_bIsCommEstablished;
}

e_COMM_PROTOCOL CWACPCommProtocol::GetType() const
{
	return e_COMM_PROTOCOL_WACP;
}

void CWACPCommProtocol::SetBTPS (const bool bSet)
{
	m_bWasBTPSTouched = true;
	m_bIsBTPSOn = bSet;
	HI.ApplyBTPS (bSet);
}

void CWACPCommProtocol::SetPeakPicking (const CHostInterface::e_PEAK_PICKING pp)
{
	m_bWasPeakPickingTouched = true;
	HI.SetPeakPicking (pp);
}

bool CWACPCommProtocol::IsBreathMessageRelevant()
{
	if (HI.IsCalibInProgress() )
		return false;
	else
		return true;
}

