// Use this define to use as forward declaration only 
#ifdef CWACPCommProtocol_Forward
	class CWACPCommProtocol;
#else

#ifndef __CWACPCommProtocol
#define __CWACPCommProtocol

#include "..\..\Types\Types.h"
#include "..\ICommProtocol.h"
#include "..\..\System\RTOS\CTimer.h"
//#include "..\..\Host_Interface\CHostInterface.h"


#define WACP_PROTOCOL			(*(CWACPCommProtocol*) &COMM_PROTOCOL )
#define DEFINE_WACP_PROTOCOL	CWACPCommProtocol& wacpProtocol = WACP_PROTOCOL

class CWACPCommProtocol : public ICommProtocol
{
public:
	// public operations
	
	//
	//Modified by the State Machine Generator 29/12/2002 : 16:57:42
	//
	CWACPCommProtocol();
	//
	//Modified by the State Machine Generator 29/12/2002 : 16:57:42
	//
	virtual void OnCreate();
	//This template method should be used as a the trigger (of an EAB of the relevant state) to perform state activities.
	//By default, here, it doesn't perform any operations.
	//When used in child class, the SMG will automatically generate differentiation code.
	virtual void DoActivities();
	virtual bool IsCommEstablished() const;
	virtual e_COMM_PROTOCOL GetType() const;
	//Returs the command code of the last received command.
	//This is to be used by the "Command Received" Message (byte #1).
	inline BYTE GetLastCommandCode();
	inline void SetDistinguishableBreathMsg (const bool bSet);
	//Returns the Co2 value to be transmitted in the wave message.
	inline float GetCO2ForComm() const;
	//Returns true if a reply to an inquirem breath message should be with distinguashable from a regular breath message (message code 17 rather than 1).
	inline bool IsDistinguishableBreathMsg();
	void SetBTPS (const bool bSet);
	//Returns true if a reply to an inquirem breath message should be with distinguashable from a regular breath message (message code 17 rather than 1).
	inline bool IsBTPSOn();
	//Returns true if BTPS setting item was set up (via communication) since powerup
	//
	inline bool WasBTPSTouched();
	void SetPeakPicking (const CHostInterface::e_PEAK_PICKING pp);
	//Returns true if peak picking setup item was set up (via communication) since powerup
	inline bool WasPeakPickingTouched();
protected:
	
	// protected attributes
	
	// This flag is set if the communication is ready
	bool	m_bIsCommEstablished : 1;
	bool	m_bIsCalibMode : 1;
	bool	m_bIsDistinguishableBreathMsg : 1;
	//Returns true if BTPS setting was modified since powerup.
	bool	m_bWasBTPSTouched : 1;
	bool	m_bWasPeakPickingTouched : 1;
	bool	m_bIsBTPSOn : 1;
	// This flag is set when the textbook wave has to been sent
	bool	m_bHasToSendTextbookWave : 1;
	BYTE	m_byLastCommandCode;
	//Stores the pending acknowledge packet to be sent. In case of no pending acknowledge packet the value of m_eAcknowledgeToBeSent will be NOTHING.
	// Fast periodic message attributes
	BYTE	m_byBreathPeriodicMsgPeriod;
	//Period of periodic message transmission in mSec.
	BYTE	m_byAutoStandbyDelay;
	//CO2 value to be transmitted in the communications wave message.
	//This value is based either on the Capnographs most recent CO2 value or in case of 25 mSec wave period on interpolation.
	float	m_fCO2ForComm;
	//Capnographs previous CO2 value.
	float	m_fCO2Prev;
	CTimer	m_tmFastPeriodicMsg;
	CTimer	m_tmBreathPeriodicMsg;
	int	m_wLastConfigItemValue;

	UINT	m_nWaveMsgindex;
	UINT	m_nWaveIndex;			// Index of wave inside the wave message (for referencing purpose in the annotation message)
	
	// protected roles
	
	// protected operations
	
	//Returns true if breath message is relevant (should be transmitted) in the current operating mode.
	bool IsBreathMessageRelevant();


};


////////////////////////////////////
//       Inlines 
///////////////////////////////////



//Returns the Co2 value to be transmitted in the wave message.
inline
float CWACPCommProtocol::GetCO2ForComm() const
{
	return m_fCO2ForComm;
}


//Returns true if a reply to an inquirem breath message should be with distinguashable from a regular breath message (message code 17 rather than 1).
inline
bool CWACPCommProtocol::IsBTPSOn()
{
	return m_bIsBTPSOn;
}

//Returns true if BTPS setting item was set up (via communication) since powerup
//
inline
bool CWACPCommProtocol::WasBTPSTouched()
{
	return m_bWasBTPSTouched;
}

//Returns true if peak picking setup item was set up (via communication) since powerup
inline
bool CWACPCommProtocol::WasPeakPickingTouched()
{
	return m_bWasPeakPickingTouched;
}



///////////////////////////////////////////////////////////

#endif /* __CWACPCommProtocol */

#endif // Forward declaration
