/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmRNDZStub.c                                                     //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmRNDZStub.h"

/*================================================================================
CFmRNDZStub_RecvMsg : Handles RNDZ messages.
================================================================================*/
boolean CFmRNDZStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmRENDEZVOUS, Genus, Species);

    TOUCH(Buffer);

	switch(Genus)
    {
        case GnCOMMAND:
        {
            /* RNDZ_CCMD_SpSTART_HOST_PROC is sent from the client once RNDZ initialization is
            * completed. 
            */
            if (Species == RNDZ_CCMD_SpSTART_HOST_PROC)
            {
				CRNDZDStd RNDZObj;
				uint8 ObjBuffer[128];
				uint32 ObjSize = 0;

				CRNDZDStd_Init(&RNDZObj);

				ObjSize = CRNDZDStd_SizeAll(&RNDZObj);

                /* If CRNDZDStd is supported, then serialize the object that contains
                 * the GUID and send it out */
                CRNDZDStd_Serialize(&RNDZObj, ObjBuffer); 

                /* Send the HOST READY command and serialize the CRNDZDStd object to send */
                SendMsg(MakeClassID(FmRENDEZVOUS, GnCOMMAND, RNDZ_CCMD_SpSEND_HOST_READY), ObjBuffer, ObjSize, 0);
#if defined(WACP_DEBUG)
                SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CCMD_SpSTART_HOST_PROC. RNDZ Complete\n");
#endif
                bConnected = TRUE;
            }
            /* RNDZ_CCMD_SpSEND_HOST_READY is sent from the host indicating a successful
             * RNDZ authentication. This signals the client to terminate the RNDZ client
             * and to start up the WACP session.
             */
            else if (Species == RNDZ_CCMD_SpSEND_HOST_READY)
            {
                /* RNDZ authentication complete */
#if defined(WACP_DEBUG)
                SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CCMD_SpSEND_HOST_READY. RNDZ Complete\n");
#endif
                bConnected = TRUE;
            }
            break;
        }
        case GnREQUEST:
        {
            if (Species == RNDZ_CRST_SpSTART_RNDZ)
            {
                if(strstr((const char*)Buffer, RNDZ_CONNECT_MSG))
                {   
#if defined(WACP_DEBUG)
                    SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CRST_SpSTART_RNDZ.\n");
#endif

                    /* Send the RNDZ_CRSP_SpRNDZ_ACCEPT Response */
                    SendMsg(MakeClassID(FmRENDEZVOUS, GnRESPONSE, RNDZ_CRSP_SpRNDZ_ACCEPT), NULL, 0, 0);
                }
            }
            break;
        }
        case GnRESPONSE:
        {
            /* RNDZ_CRSP_SpRNDZ_ACCEPT is sent from the client indicating that the request
             * to start RNDZ has been accepted.
             */
            if (Species == RNDZ_CRSP_SpRNDZ_ACCEPT)
            {
				CRNDZDStd RNDZObj;
				uint8 ObjBuffer[128];
				uint32 ObjSize = 0;

				CRNDZDStd_Init(&RNDZObj);

				ObjSize = CRNDZDStd_SizeAll(&RNDZObj);

                /* If CRNDZDStd is supported, then serialize the object that contains
                 * the GUID and send it out */
                CRNDZDStd_Serialize(&RNDZObj, ObjBuffer); 

                /* Send the START HOST PROC command */
                SendMsg(MakeClassID(FmRENDEZVOUS, GnCOMMAND, RNDZ_CCMD_SpSTART_HOST_PROC), ObjBuffer, ObjSize, 0);
            }
            break;
        }
    }

	return MsgResult;
   
}

